(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    529950,      17446]
NotebookOptionsPosition[    438292,      14765]
NotebookOutlinePosition[    460893,      15258]
CellTagsIndexPosition[    459722,      15219]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\
Basic Structures: Sets, Functions, Sequences, Sums, and Matrices\
\>", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 2}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "Chapter 2 of the textbook covers mathematical objects fundamental to the \
study of discrete mathematics. We will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " represents these objects."
}], "Text"],

Cell[TextData[{
 "In Sections 1 and 2 we will see that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s implementation of a list can be used to model the \
mathematical concept of set. We will also see how to extend ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s capabilities to include fuzzy sets. In Section 3 we \
will consider different ways in which the concept of function can be \
represented in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and how these approaches can be used in different circumstances. Section 4 \
will look at how to use lists in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to model finite sequences, how to represent recurrence relations, and how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to compute both finite and symbolic summations. In Section 5, \
we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to list positive rational numbers in a way that demonstrates the fact that \
the rationals are enumerable. And in Section 6, we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to study matrices."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.1 Sets", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "Sets are fundamental to the description of almost all of the discrete \
objects that we will study. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides support for both their representation and manipulation."
}], "Text"],

Cell[CellGroupData[{

Cell["Set Basics", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", sets are modeled as lists. Note that the syntax for a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " list matches the standard syntax for a mathematical set: a list of the \
elements of the set separated by commas and enclosed in braces. The elements \
can be any object known to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Typical examples are shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell["\<\
In the first two examples above, the lists contain the numbers 1, 2, and 3, \
and the characters a, b, and c, respectively (note that the quotation marks \
distinguish them as strings and not symbols). In the third example, the \
elements of the list are themselves lists. The fourth example is the empty \
list. \
\>", "Text"],

Cell[TextData[{
 "Note, however, that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not treat these list objects as sets, that is, it does not \
automatically respect the mathematical properties of sets. In particular, \
repetition and order make lists distinct, unlike mathematical sets. For \
example, consider the lists defined below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set1", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "1", ",", "2"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "1", ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that when ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " echoes the list, it preserves the duplicate entries."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set2", "=", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3", ",", "1"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Also note that the order in which elements are entered is preserved.\
\>", "Text"],

Cell[TextData[{
 "To test whether two lists are the same, use the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") relation. Below, you see that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " considers the lists ",
 StyleBox["set1", "Input"],
 " and ",
 StyleBox["set2", "Input"],
 " different from each other and different from ",
 StyleBox["{1,2,3}", "Input"],
 ", despite the fact that, as sets, they are all identical."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set1", "==", "set2"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set1", "==", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3"}], "}"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set2", "==", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3"}], "}"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "In order to get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to recognize that ",
 StyleBox["set1", "Input"],
 " and ",
 StyleBox["set2", "Input"],
 " are equal as sets, you must force ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to represent them in a canonical fashion. This is achieved with the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function. We will see below how to use ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to find the union of two or more sets. But when applied to a single list, ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " returns the list obtained by sorting the elements and removing \
duplicates."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{"{", 
   RowBox[{"4", ",", "2", ",", "1", ",", "1", ",", "3", ",", "2", ",", "4"}], 
   "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Because ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " puts the elements of a list in a canonical order, the results of applying ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to two different lists that represent the same mathematical set will be \
the same."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Union", "[", "set1", "]"}], "==", 
  RowBox[{"Union", "[", "set2", "]"}]}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "It is a good idea, when doing work involving sets, to apply the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function as part of the definition of the set.",
 " "
}], "Text"],

Cell[CellGroupData[{

Cell["Parts of Sets", "Subsubsection"],

Cell[TextData[{
 "Because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses lists to represent sets, you can use the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " operator to access individual elements and to select subsets based on \
their index.",
 " ",
 "To obtain, for example, the fourth element, enclose 4 in double brackets."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set3", "=", 
  RowBox[{"Union", "[", 
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<f\>\""}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>", ",", "\<\"f\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set3", "[", 
  RowBox[{"[", "4", "]"}], "]"}]], "Input"],

Cell[BoxData["\<\"d\"\>"], "Output"]
}, Open  ]],

Cell["\<\
Negative values count from the right, so the second to last element (in the \
canonical order), is accessed as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set3", "[", 
  RowBox[{"[", 
   RowBox[{"-", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData["\<\"e\"\>"], "Output"]
}, Open  ]],

Cell["\<\
By putting a list of indices within the double brackets, you can obtain any \
subset you wish. Note that the list must be contained in braces within the \
double brackets.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set3", "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4"}], "}"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Finally, you can obtain a range of elements using the ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Input"],
 ") operator. For example, to obtain the second through fifth elements, enter \
the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set3", "[", 
  RowBox[{"[", 
   RowBox[{"2", ";;", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", ",", "\<\"e\"\>"}],
   "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generating Sets", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " contains many functions that can be used to generate sets (lists). Two of \
these are ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " is the simpler function and we will describe it first."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " function is used to generate lists consisting of simple sequences of \
numbers. It has three basic forms. The first form of ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " is with one argument. ",
 "In this case, it produces the list of integers from 1 through the given \
value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", "8", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second form of ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " is with two arguments.",
 " ",
 "In this case, it produces the list of values with the first argument as the \
starting value and the second argument as the maximum."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", 
  RowBox[{
   RowBox[{"-", "3"}], ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "3"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "1"}], ",", "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", 
   "5"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The third form takes three arguments. Again, the first argument is the \
starting value and the second is the maximum. The third argument specifies \
the \[OpenCurlyDoubleQuote]step\[CloseCurlyDoubleQuote], that is, the \
difference between successive values in the list. For example, the following \
produces every third integer beginning with 2 up to 25.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", 
  RowBox[{"2", ",", "25", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "5", ",", "8", ",", "11", ",", "14", ",", "17", ",", "20", ",", 
   "23"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the maximum does not need to be a member of the list. Also note \
that the values given do not need to be integers. The command below produces \
the numbers beginning at 0.25 and increasing by 0.41 up until at most 7.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", 
  RowBox[{"0.25", ",", "7", ",", "0.41"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.25`", ",", "0.6599999999999999`", ",", "1.0699999999999998`", ",", 
   "1.48`", ",", "1.89`", ",", "2.3`", ",", "2.71`", ",", 
   "3.1199999999999997`", ",", "3.53`", ",", "3.94`", ",", "4.35`", ",", 
   "4.76`", ",", "5.17`", ",", "5.58`", ",", "5.989999999999999`", ",", 
   "6.3999999999999995`", ",", "6.81`"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second function, ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", gives you even more control over forming lists. Generally, ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " requires two arguments. The first argument is an expression, often \
involving a symbol called the table index or table variable, that specifies \
the elements of the list being created. The second argument is a list \
defining the values that are to be substituted for the table variable in \
order to produce the list. There are five distinct forms for this second \
argument."
}], "Text"],

Cell[TextData[{
 "The simplest form of ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " is where no variable is used in the first argument and the second argument \
is a number enclosed in braces. This results in a list consisting of that \
many copies of the first argument.",
 " ",
 "For example, to produce a list of seven fives, enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{"5", ",", 
   RowBox[{"{", "7", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "5", ",", "5", ",", "5", ",", "5", ",", "5", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The other forms all use a table index. For these examples, we will use the \
variable ",
 StyleBox["i", "Input"],
 ", and the expression that we give as the first argument will be ",
 Cell[BoxData[
  RowBox[{"i", "^", "2"}]], "myCode"],
 ". This will produce lists whose elements are the squares of the values \
substituted for ",
 StyleBox["i", "Input"],
 ". The second argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", when using a variable, is always a list whose first element is the name \
of the variable, as in ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["var", "CodeVar"], ",", "..."}], "}"}]], "myCode"],
 "."
}], "Text"],

Cell[TextData[{
 "If you evaluate ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with the second argument comprised of the variable and a positive integer, \
the result will be to substitute the integers 1 though the given integer for \
the variable. The following produces the squares of the integers 1 through 9. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "9"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "4", ",", "9", ",", "16", ",", "25", ",", "36", ",", "49", ",", 
   "64", ",", "81"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
In the next form, the second argument is the list consisting of the name of \
the variable and two numbers, a minimum and a maximum. The following produces \
the list of squares of integers from 5 to 12.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "5", ",", "12"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25", ",", "36", ",", "49", ",", "64", ",", "81", ",", "100", ",", "121", 
   ",", "144"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also include a \[OpenCurlyDoubleQuote]step\[CloseCurlyDoubleQuote] \
by adding yet another number to the second argument. The following shows the \
squares of every other integer beginning at ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["4", "2"], TraditionalForm]]],
 " and ending at ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["20", "2"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "4", ",", "20", ",", "2"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16", ",", "36", ",", "64", ",", "100", ",", "144", ",", "196", ",", "256", 
   ",", "324", ",", "400"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In the final form of the second argument, you are able to specify an \
explicit list of values to substitute for the variable. You do this using the \
syntax ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["var", "CodeVar"], ",", 
    RowBox[{"{", 
     StyleBox["values", "CodeVar"], "}"}]}], "}"}]], "myCode"],
 ". That is, the second argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " is a list consisting of two elements: first the name of the variable, and \
second the list of values enclosed in braces. The following computes the \
squares of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", " ", 
    RowBox[{"and", " ", "13"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
      "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "9", ",", "25", ",", "49", ",", "121", ",", "169"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We summarize the allowed forms of the second argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " in the table below."
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
     StyleBox["i", "CodeVar"], " ", "ranges", " ", "from", " ", "1", " ", 
     "to", " ", 
     StyleBox[
      SubscriptBox["i", "max"], "CodeVar"]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
     StyleBox["i", "CodeVar"], " ", "ranges", " ", "from", " ", 
     StyleBox[
      SubscriptBox["i", "min"], "CodeVar"], " ", "to", " ", 
     StyleBox[
      SubscriptBox["i", "max"], "CodeVar"]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], ",", 
      StyleBox["step", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
     StyleBox["i", "CodeVar"], " ", "ranges", " ", "from", " ", 
     StyleBox[
      SubscriptBox["i", "min"], "CodeVar"], " ", "to", " ", 
     StyleBox[
      SubscriptBox["i", "max"], "CodeVar"], " ", "by", " ", 
     StyleBox["step", "CodeVar"]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox["list", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
     StyleBox["i", "CodeVar"], " ", "ranges", " ", "over", " ", "elements", 
     " ", "of", " ", 
     StyleBox["list", "CodeVar"]}]}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " can also be used with more than one variable in the expression. To do \
this, you simply provide an additional argument specifying each table \
variable. The ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function ensures that all combinations of values are included. For \
example, if ",
 StyleBox["f", "Input"],
 " were a function of two values, the following would evaluate the function \
with the first value ranging from 3 to 9 and the second taking the values 0, \
1, and 5."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tableExample", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3", ",", "9"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "5"}], "}"}]}], "}"}]}], "]"}]}]], "Input",\

 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"3", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"4", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"4", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"4", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"5", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"5", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"5", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"6", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"6", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"6", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"7", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"7", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"7", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"8", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"8", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"8", ",", "5"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"9", ",", "0"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"9", ",", "1"}], "]"}], ",", 
     RowBox[{"f", "[", 
      RowBox[{"9", ",", "5"}], "]"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that this produces a nested list. If your goal is to produce a single \
list of all of the values, so as to work with the result as a set, use ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " to eliminate the nested structure and use ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to ensure that the result is canonically ordered and that duplicates have \
been removed.",
 " ",
 "Be sure that ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " is applied first."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{"Flatten", "[", "tableExample", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"3", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"3", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"3", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"4", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"4", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"4", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"5", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"5", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"5", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"6", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"6", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"6", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"7", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"7", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"7", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"8", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"8", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"8", ",", "5"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"9", ",", "0"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"9", ",", "1"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"9", ",", "5"}], "]"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Membership, Subset, and Size", "Subsubsection"],

Cell[TextData[{
 "Perhaps the most basic question one can ask about a set is whether or not a \
particular object is or is not a member of a set. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you do this with the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function. The first argument to ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " is the list and the second is the object being sought."
}], "Text"],

Cell[TextData[{
 "For example, if we use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to define ",
 StyleBox["set4", "Input"],
 ", the set of squares of integers from 0 to 10, we can use ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " to see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "\[Element]", 
    StyleBox["set4", "Input"]}], TraditionalForm]]],
 " but that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"5", "\[NotElement]", 
    StyleBox["set4", "Input"]}], TraditionalForm]]],
 ". As mentioned earlier, when modeling a set as a list, it is a good idea to \
apply ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " when defining it."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"set4", "=", 
  RowBox[{"Union", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"i", "^", "2"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "4", ",", "9", ",", "16", ",", "25", ",", "36", ",", 
   "49", ",", "64", ",", "81", ",", "100"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemberQ", "[", 
  RowBox[{"set4", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemberQ", "[", 
  RowBox[{"set4", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " function returns the number of elements in a list. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "set4", "]"}]], "Input"],

Cell[BoxData["11"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Be careful to keep in mind that if it is the cardinality of a set you are \
after, you must use ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " or risk over-counting."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "2", ",", "3"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Union", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "2", ",", "3"}], "}"}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not have a built-in function for checking whether one set is a subset \
of another. However, it is not difficult to create one. Given finite sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", determining whether or not ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[Subset]", "B"}], TraditionalForm]]],
 " amounts to checking, for every element of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", whether it is also a member of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "We can loop over all members of a given list ",
 StyleBox["A", "Input"],
 " by using the ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function with the ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["var", "CodeVar"], ",", 
   StyleBox["list", "CodeVar"], "}"}]], "myCode"],
 " construction for the second argument (assuming ",
 StyleBox["A", "Input"],
 " is a list). Note that ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " use the same syntax for their iteration specifications. The main \
difference between ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " is that ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " builds a list as its output while ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " simply executes the code in the first argument without producing output \
unless explicitly told to do so. "
}], "Text"],

Cell[TextData[{
 "Within the loop, we use the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function to test whether the object is in the list ",
 StyleBox["B", "Input"],
 ". We embed the loop within a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " and use ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " if we find an element of ",
 StyleBox["A", "Input"],
 " missing from ",
 StyleBox["B", "Input"],
 ". This ensures that we return false as soon as possible rather than \
checking every element of ",
 StyleBox["A", "Input"],
 " if it is not necessary. If the loop completes, we ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". Without that statement, nothing would be thrown and the ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " would output ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"subsetQ", "[", 
   RowBox[{"A_", ",", "B_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "i", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Do", "[", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"MemberQ", "[", 
            RowBox[{"B", ",", "i"}], "]"}]}], ",", 
          RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "A"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["We demonstrate our function on a few sets.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"4", ",", "9", ",", "100"}], "}"}], ",", "set4"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", "set4"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetQ", "[", 
  RowBox[{
   RowBox[{"Range", "[", 
    RowBox[{"3", ",", "7"}], "]"}], ",", 
   RowBox[{"Range", "[", "10", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Power Set and Cartesian Product", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a built-in function to compute the power set of a finite set. The ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function accepts a list as its argument and returns the list of all \
possible sublists. For example, the power set of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], TraditionalForm]]],
 " can be computed as below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Keep in mind that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not automatically remove duplicates. ",
 "So, for example, if we apply ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " to a list with repeated elements, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will compute the power set as if the duplicates were distinct."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<c\>\""}], "}"}],
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "As always, applying ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " before using the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function will help avoid this."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"Union", "[", 
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<c\>\""}], 
    "}"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["Tuples",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tuples"], "FunctionLink"],
 " can be used to compute the Cartesian product of sets.",
 " ",
 "There are two ways to use this function."
}], "Text"],

Cell[TextData[{
 "First, to compute the Cartesian product of two or more sets, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b", ",", "c"}], "}"}], "\[Cross]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}]}], TraditionalForm]]],
 ", apply ",
 StyleBox[ButtonBox["Tuples",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tuples"], "FunctionLink"],
 " with one argument: a list whose elements are the lists to be combined."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tuples", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "2"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the result is a list of lists. We interpret the outer list as a \
set, the Cartesian product, and we interpret the inner lists as the ordered \
tuples. You can extend this to more than two sets just by including more \
lists in the argument to ",
 StyleBox[ButtonBox["Tuples",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tuples"], "FunctionLink"],
 ". "
}], "Text"],

Cell[TextData[{
 "The second syntax is used to compute the Cartesian product of a set with \
itself. For example, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "=", 
    RowBox[{"{", 
     RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], TraditionalForm]]],
 ", you can compute ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "3"], TraditionalForm]]],
 " by giving the list as the first argument and the exponent as the second."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tuples", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], ",", 
   "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example Using the Power Set", "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "As an example of a practical use of ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 ", ",
 "let\[CloseCurlyQuote]s search for the subsets of the first five positive \
integers which have their own cardinality as a member, i.e., those sets ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "S", "|", 
    RowBox[{"\[Element]", "S"}]}], TraditionalForm]]],
 ". ",
 "We\[CloseCurlyQuote]ll do this by considering each subset in turn and \
checking whether its size, obtained using the ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " function, is a member of the subset, using ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 "."
}], "Text"],

Cell["\<\
Here\[CloseCurlyQuote]s the function that will list all subsets of the first \
five positive integers whose cardinalities are members of themselves.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"selfSize", ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"mainSet", ",", "powerSet", ",", "S"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mainSet", "=", 
      RowBox[{"Range", "[", "5", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"powerSet", "=", 
      RowBox[{"Subsets", "[", "mainSet", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{"S", ",", 
           RowBox[{"Length", "[", "S", "]"}]}], "]"}], ",", 
         RowBox[{"Print", "[", "S", "]"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"S", ",", "powerSet"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "After declaring local variables, we form the set of the integers from 1 to \
5 using ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 ". Then we apply the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function to form the power set, ",
 StyleBox["powerSet", "Input"],
 ". Using a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop, we iterate over every member ",
 StyleBox["S", "Input"],
 " of ",
 StyleBox["powerSet", "Input"],
 ". Inside the loop, we print those sets that have their own size as a member."
}], "Text"],

Cell["Now let\[CloseCurlyQuote]s execute it.", "Text"],

Cell[CellGroupData[{

Cell[BoxData["selfSize"], "Input"],

Cell[BoxData[
 RowBox[{"{", "1", "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "4", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "4", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "4", ",", "5"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]], "Print"]
}, Open  ]],

Cell["\<\
If you wish, you can easily modify the function definition to take the main \
set as an argument.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2 Set Operations", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section we will examine the functions ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides for computing set operations. Then we will use these commands and \
the concept of membership tables to see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to prove set identities. Finally, we see how we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to represent and manipulate fuzzy sets."
}], "Text"],

Cell[CellGroupData[{

Cell["Basic Operations", "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides fairly intuitive functions related to the basic set operations of \
union, intersection, and complement. The functions are named ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Intersection",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Intersection"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 ". Consider the following sets."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"primes", "=", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"odds", "=", 
  RowBox[{"Range", "[", 
   RowBox[{"1", ",", "13", ",", "2"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "5", ",", "7", ",", "9", ",", "11", ",", "13"}],
   "}"}]], "Output"]
}, Open  ]],

Cell["We compute their union and intersection as follows:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{"primes", ",", "odds"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "5", ",", "7", ",", "9", ",", "11", ",", 
   "13"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Intersection", "[", 
  RowBox[{"primes", ",", "odds"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "5", ",", "7", ",", "11", ",", "13"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Neither of these functions is restricted to two arguments, but will find the \
union or common intersection of any number of sets. Also, both functions have \
infix forms obtained using the aliases \[EscapeKey]un\[EscapeKey] and \
\[EscapeKey]inter\[EscapeKey]. \
\>", "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 " function is slightly different. Its first argument is the universal set \
and its second argument is the set whose complement is desired.",
 " ",
 "For example, to find the complement of ",
 StyleBox["odds", "Input"],
 " in the universe consisting of integers 1 to 13, you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{
   RowBox[{"Range", "[", "13", "]"}], ",", "odds"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "4", ",", "6", ",", "8", ",", "10", ",", "12"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Recall from the textbook that the complement of a set can be defined in \
terms of set difference. The ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 " function can be thought of as implementing the set difference of the first \
argument minus the second. The following examples compute the differences of \
the ",
 StyleBox["primes", "Input"],
 " and ",
 StyleBox["odds", "Input"],
 " sets and illustrate that, unlike union and intersection, set difference is \
not symmetric, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "-", "B"}], TraditionalForm]]],
 " is generally not the same as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "-", "A"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{"primes", ",", "odds"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "2", "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{"odds", ",", "primes"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "9"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 " function can accept more than two arguments. In this case, it returns the \
list of all elements of the first set that appear in none of the others. That \
is to say, it finds the complement of the union of all of the sets following \
the first relative to the first: ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"Complement", "["}], 
   StyleBox["U", "CodeVar"], ",", 
   StyleBox["A", "CodeVar"], ",", 
   StyleBox["B", "CodeVar"], ",", "...", "]"}]], "myCode"],
 " computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{"U", "-", 
    RowBox[{"(", 
     RowBox[{"A", "\[Union]", "B", "\[Union]", "\[Ellipsis]"}], ")"}]}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{
   RowBox[{"Range", "[", "13", "]"}], ",", "odds", ",", "primes"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "6", ",", "8", ",", "10", ",", "12"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Set Identities and Membership Tables", "Subsection",
 CellTags->"Section2b"],

Cell[TextData[{
 "The textbook discusses how membership tables can be used to prove set \
identities. In this subsection, we use the idea of membership tables to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " verify set identities."
}], "Text"],

Cell["\<\
A membership table is very similar to a truth table. In a membership table, \
each row corresponds to a possible element in the universe. We use 1 and 0 to \
indicate that the element corresponding to that row is or is not in the set.\
\>", "Text"],

Cell[CellGroupData[{

Cell["An Illustration of the General Approach", "Subsubsection"],

Cell[TextData[{
 "We first consider a specific example in detail in order to get an idea of \
how we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to automate the construction of membership tables. Consider the De Morgan\
\[CloseCurlyQuote]s law ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox[
     RowBox[{"A", "\[Union]", "B"}], "_"], "=", 
    RowBox[{
     OverscriptBox["A", "_"], "\[Intersection]", 
     OverscriptBox["B", "_"]}]}], TraditionalForm]]],
 ". We begin the table by considering all possible combinations of 1s and 0s \
for A and B and add columns for the two sides of the identity. "
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {
    RowBox[{"row", " ", "number"}], 
    StyleBox[Cell[TextData[Cell[BoxData[
      FormBox["A", TraditionalForm]]]]], "Text"], 
    StyleBox[Cell["B"], "Text"], Cell[TextData[Cell[BoxData[
     FormBox[
      OverscriptBox[
       RowBox[{"A", "\[Union]", "B"}], "_"], TraditionalForm]]]]], Cell[
    TextData[Cell[BoxData[
     FormBox[
      RowBox[{
       OverscriptBox["A", "_"], "\[Intersection]", 
       OverscriptBox["B", "_"]}], TraditionalForm]]]]]},
   {"1", 
    StyleBox["1", "Text"], 
    StyleBox["1", "Text"], " ", 
    StyleBox[" ", "Text"]},
   {
    StyleBox["2", "Text"], 
    StyleBox["1", "Text"], 
    StyleBox["0", "Text"], 
    StyleBox[" ", "Text"], 
    StyleBox[" ", "Text"]},
   {
    StyleBox["3", "Text"], 
    StyleBox["0", "Text"], 
    StyleBox["1", "Text"], 
    StyleBox[" ", "Text"], 
    StyleBox[" ", "Text"]},
   {
    StyleBox["4", "Text"], 
    StyleBox["0", "Text"], 
    StyleBox["0", "Text"], 
    StyleBox[" ", "Text"], 
    StyleBox[" ", "Text"]}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "We determine the values for the last two columns as follows. Let the \
universe be the set consisting of the row numbers ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], TraditionalForm]]],
 ". Now form sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " as follows: a number in the universe of row numbers is in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " if there is a 1 in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 "\[CloseCurlyQuote]s column in that row. Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "=", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}]}], TraditionalForm]]],
 " because rows 1 and 2 have 1s in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 "\[CloseCurlyQuote]s column. Likewise, we form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "=", 
    RowBox[{"{", 
     RowBox[{"1", ",", "3"}], "}"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rows", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"setA", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"setB", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "3"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Next, compute both sides of the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox[
     RowBox[{"A", "\[Union]", "B"}], "_"], "=", 
    RowBox[{
     OverscriptBox["A", "_"], "\[Intersection]", 
     OverscriptBox["B", "_"]}]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{"rows", ",", 
   RowBox[{"Union", "[", 
    RowBox[{"setA", ",", "setB"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "4", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This indicates that row 4 is the only row with a 1 in the column for",
 " ",
 Cell[BoxData[
  FormBox[
   OverscriptBox[
    RowBox[{"A", "\[Union]", "B"}], "_"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Intersection", "[", 
  RowBox[{
   RowBox[{"Complement", "[", 
    RowBox[{"rows", ",", "setA"}], "]"}], ",", 
   RowBox[{"Complement", "[", 
    RowBox[{"rows", ",", "setB"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "4", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This tells us that row 4 is also the only row with a 1 in the column for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["A", "_"], "\[Intersection]", 
    OverscriptBox["B", "_"]}], TraditionalForm]]],
 ". ",
 "Since the two sets are equal, the two columns must be identical."
}], "Text"],

Cell["\<\
The above indicates the approach that we will be using. First, compute the \
initial entries in the rows of the membership table; each row corresponds to \
a different assignment of 1s and 0s. Second, construct sets whose entries are \
the row numbers corresponding to 1s in the table. And finally, compute both \
sides of the identity. If the resulting sets are equal, then we have \
confirmed the identity.\
\>", "Text"],

Cell[TextData[{
 "Much of what we do here will be very similar to how we created the ",
 StyleBox["myEquivalentQ", "Input"],
 " function in Section 1.3 of this manual. First, we need to be able to \
create expressions representing the two sides of the identity. We\
\[CloseCurlyQuote]ll be using Example 14 of the text as our example: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox[
     RowBox[{"A", "\[Union]", 
      RowBox[{"(", 
       RowBox[{"B", "\[Intersection]", "C"}], ")"}]}], "_"], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       OverscriptBox["C", "_"], "\[Union]", 
       OverscriptBox["B", "_"]}], ")"}], "\[Intersection]", 
     OverscriptBox["A", "_"]}]}], TraditionalForm]]],
 ". We\[CloseCurlyQuote]ll use the symbol ",
 StyleBox["U", "Input"],
 " for the universe and ",
 StyleBox["a", "Input"],
 ", ",
 StyleBox["b", "Input"],
 ", and ",
 StyleBox["c", "Input"],
 " for the names of sets. (We use lowercase set names in order to avoid \
conflict with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s reserved symbol ",
 StyleBox["C", "Input"],
 ".) It is important that these symbols not be assigned to values, so we \
first ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " them."
}], "Text"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"U", ",", "a", ",", "b", ",", "c"}], "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Set Expressions and Delayed Evaluation", "Subsubsection"],

Cell[TextData[{
 "Creating an expression involving set operations and variables is not as \
simple as it is for the logical operators.",
 " ",
 "With the logical operators, we can, for example, create the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[And]", "b"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"And", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"a", "&&", "b"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "But with the set operations, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " requires the arguments to not be symbols. So when we try to enter the \
analogous set expression, a warning is produced."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Intersection", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"a", "\[Intersection]", "b"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The message is telling us that the function we applied is supposed to be a \
list or other compound expression, not a mere symbol (or number or string). \
Beyond the warning message, there is another difficulty in representing \
expressions involving set operations in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Observe what happens when we attempt to enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"A", "\[Intersection]", "B"}], ")"}], "\[Union]", 
    RowBox[{"(", 
     RowBox[{"A", "\[Intersection]", "C"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{
   RowBox[{"Intersection", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"Intersection", "[", 
    RowBox[{"a", ",", "c"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] c\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b \
\[Intersection] c\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(Intersection :: normal\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"a", "\[Intersection]", "b", "\[Intersection]", "c"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " transformed the expression we entered into the intersection of all three \
sets, which does not represent the same set as the original expression. The \
reason for this behavior is that the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function\[CloseCurlyQuote]s arguments are not required to be lists. \
Provided the arguments have the same head, ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " returns the expression consisting of that common head and the union of the \
arguments. This is illustrated below using the symbol ",
 StyleBox["f", "Input"],
 " for the head."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "c", ",", "d"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"b", ",", "d", ",", "e"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In order to work properly, our functions in this section will need to \
ensure that the set expressions they are given are not evaluated. We will do \
this with ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " function accepts a single argument, which can be any expression \
whatsoever. The result is to maintain the expression just as it was given to ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " with no evaluation performed on it."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"holdExample", "=", 
  RowBox[{"Hold", "[", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Intersection", "[", 
      RowBox[{"a", ",", "b"}], "]"}], ",", 
     RowBox[{"Intersection", "[", 
      RowBox[{"a", ",", "c"}], "]"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
   RowBox[{"a", "\[Intersection]", "c"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In the above, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " front end has displayed the set expression using the infix symbols rather \
than the function names (",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " will show that the function names are still in the internal \
representation). However, the expression was retained as we intended it, in \
contrast to what happened to the same expression above."
}], "Text"],

Cell[TextData[{
 "Note also that the result has ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " as its head. This means that the expression will remain held until we \
explicitly cause it to be evaluated using ",
 StyleBox[ButtonBox["ReleaseHold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReleaseHold"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "holdExample", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] c\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b \
\[Intersection] c\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(Intersection :: normal\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"a", "\[Intersection]", "b", "\[Intersection]", "c"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In what follows, we will initially require that all set expressions are \
manually held, by being entered with the ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " function explicitly applied. At the end of this section, we will see how \
to remove this cumbersome requirement."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Revising the ",
 StyleBox["getVars", "Input"],
 " Function"
}], "Subsubsection"],

Cell[TextData[{
 "Much of what follows will parallel the construction of the ",
 StyleBox["myEquivalentQ", "Input"],
 " function from Section 1.3 of this manual. ",
 "First, let\[CloseCurlyQuote]s create expressions representing the two sides \
of the identity from Example 14 of the text.",
 " We\[CloseCurlyQuote]ll use the symbol ",
 StyleBox["U", "Input"],
 " for the universe and ",
 StyleBox["a", "Input"],
 ", ",
 StyleBox["b", "Input"],
 ", and ",
 StyleBox["c", "Input"],
 " for the sets."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14L", "=", 
  RowBox[{"Hold", "[", 
   RowBox[{"Complement", "[", 
    RowBox[{"U", ",", 
     RowBox[{"Union", "[", 
      RowBox[{"a", ",", 
       RowBox[{"Intersection", "[", 
        RowBox[{"b", ",", "c"}], "]"}]}], "]"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"Complement", "[", 
   RowBox[{"U", ",", 
    RowBox[{"a", "\[Union]", 
     RowBox[{"b", "\[Intersection]", "c"}]}]}], "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14R", "=", 
  RowBox[{"Hold", "[", 
   RowBox[{"Intersection", "[", 
    RowBox[{
     RowBox[{"Union", "[", 
      RowBox[{
       RowBox[{"Complement", "[", 
        RowBox[{"U", ",", "c"}], "]"}], ",", 
       RowBox[{"Complement", "[", 
        RowBox[{"U", ",", "b"}], "]"}]}], "]"}], ",", 
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "a"}], "]"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "c"}], "]"}], "\[Union]", 
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "b"}], "]"}]}], ")"}], "\[Intersection]", 
   RowBox[{"Complement", "[", 
    RowBox[{"U", ",", "a"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we will create a version of the ",
 StyleBox["getVars", "Input"],
 " function from Section 1.3. In Section 1.3 of this manual, we built ",
 StyleBox["getVars", "Input"],
 " using the most fundamental ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions possible. Our purpose then was to build the functions from \
scratch as an illustration of essential programming constructions and \
concepts. Here, we will instead take advantage of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s sophisticated built-in functions to illustrate their \
use."
}], "Text"],

Cell[TextData[{
 "Most of the work of the function will be done by using the built-in \
function ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 ". The purpose of the ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " function is to analyze an expression and produce a list of the elements of \
the expression that satisfy a certain condition. A simple example is finding \
the integers in a list of numbers."
}], "Text"],

Cell[TextData[{
 "The first argument to ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " is the expression being searched, for example the list of numbers. The \
second argument must be a \[OpenCurlyDoubleQuote]pattern\
\[CloseCurlyDoubleQuote] that describes which elements are to be matched. \
Patterns in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " are rather involved. Here we will describe only the aspects needed for the \
current task. "
}], "Text"],

Cell[TextData[{
 "We have already used the most basic ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " pattern, the ",
 StyleBox[ButtonBox["Blank",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blank"], "FunctionLink"],
 " (",
 StyleBox["_", "Input"],
 "), which matches anything at all. When you define a function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", as in ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "^", "2"}]}]], "myCode"],
 ", the ",
 StyleBox[ButtonBox["Blank",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blank"], "FunctionLink"],
 " (",
 StyleBox["_", "Input"],
 ") indicates to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that you wish to match any expression as the argument to the function. The \
symbol preceding the blank, in this case, ",
 StyleBox["x", "Input"],
 ", tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that you will be using that symbol to refer to the expression that was \
matched by the blank."
}], "Text"],

Cell["\<\
You can create more specific patterns, that is patterns that are selective \
about the kinds of expressions that they match, by following the blank with \
the name of a head. Consider, for example, the following function definition.\
\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"patternEx", "[", "x_Integer", "]"}], ":=", 
  RowBox[{"x", "^", "2"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Here, ",
 StyleBox["x_Integer", "Input"],
 " is a pattern. As before, the symbol preceding the blank tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that you will refer to the matched expression with ",
 StyleBox["x", "Input"],
 ". The fact that the blank is followed by ",
 StyleBox["Integer", "Input"],
 " tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that you only allow a match when the expression has head ",
 StyleBox["Integer", "Input"],
 ". If you give this function an argument that is not an integer, it will not \
match the pattern that specifies the argument and so the function definition \
will not apply."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"patternEx", "[", "5", "]"}]], "Input"],

Cell[BoxData["25"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"patternEx", "[", "2.9", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"patternEx", "[", "2.9`", "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can use this pattern with ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " in order to find the elements of a list that are integers. We use the \
function with first argument a list of objects and second argument the \
pattern describing the objects we want found."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"5", ",", 
     RowBox[{"-", "3"}], ",", "9", ",", "x", ",", "Pi", ",", "0", ",", 
     "4.7"}], "}"}], ",", "_Integer"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", 
   RowBox[{"-", "3"}], ",", "9", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we were able to leave off the symbol preceding the blank in the \
pattern ",
 StyleBox["_Integer", "Input"],
 ", since we did not need to refer to the integers being matched. Here is \
another example."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", "1"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "q", ",", "3"}], "}"}], ",", "x", ",", "6"}], "}"}], 
   ",", "_Integer"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", 
   RowBox[{"-", "1"}], ",", "6"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In this example, ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " did not return the 5 and the 3 found in the sublist. This is because ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 ", by default, only checks the first level of its argument. To be more \
precise, in the example above, the expression that ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " was applied to was a list with 5 elements. The ",
 StyleBox["2", "Input"],
 ", ",
 StyleBox["-1", "Input"],
 ", and ",
 StyleBox["6", "Input"],
 " were elements with head ",
 StyleBox["Integer", "Input"],
 " and were matched by the pattern. The ",
 StyleBox["x", "Input"],
 " has head ",
 StyleBox["Symbol", "Input"],
 " and is not matched. The other element was the list ",
 StyleBox["{5,q,3}", "Input"],
 ", which has head ",
 StyleBox["List", "Input"],
 " and so the sublist was not matched by the pattern. Moreover, ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " did not delve into that subexpression."
}], "Text"],

Cell[TextData[{
 "You can have ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " dig deeper, that is, analyze elements of subexpressions, by providing a \
level specification. By providing a positive integer as an optional third \
argument, ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " will include everything from the first level down to the specified level. \
For example, to have ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " include the ",
 StyleBox["5", "Input"],
 " and ",
 StyleBox["3", "Input"],
 " in its result for the example above, you just need to tell it to work down \
to level 2."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", "1"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "q", ",", "3"}], "}"}], ",", "x", ",", "6"}], "}"}], 
   ",", "_Integer", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", 
   RowBox[{"-", "1"}], ",", "5", ",", "3", ",", "6"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The symbol ",
 StyleBox["Infinity", "Input"],
 " is used as the level specification to indicate that you want ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " to go as deep as possible."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", 
             RowBox[{"{", 
              RowBox[{"6", ",", 
               RowBox[{"{", 
                RowBox[{"7", ",", 
                 RowBox[{"{", 
                  RowBox[{"8", ",", 
                   RowBox[{"{", "9", "}"}]}], "}"}]}], "}"}]}], "}"}]}], 
            "}"}]}], "}"}]}], "}"}]}], "}"}]}], "}"}], ",", "_Integer", ",", 
   "Infinity"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8", 
   ",", "9"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since the first argument does not have to be a list, but can be any \
expression, we can identify the variables used in an expression by using ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " to search for the symbols in the expression with the pattern ",
 StyleBox["_Symbol", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{"ex14R", ",", "_Symbol", ",", "Infinity"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"U", ",", "c", ",", "U", ",", "b", ",", "U", ",", "a"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We\[CloseCurlyQuote]re almost there, but we want to exclude the universe \
from our list of variables. To do this, we will be assuming, for convenience, \
that the universe is always denoted by U in expressions. The reason for \
excluding the universe is so that the list of variables returned by ",
 StyleBox["getVarsSets", "Input"],
 " corresponds to the columns of the membership table."
}], "Text"],

Cell[TextData[{
 "We\[CloseCurlyQuote]ll exclude ",
 StyleBox["U", "Input"],
 " by making a modification of the pattern specification. ",
 StyleBox[ButtonBox["Except",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Except"], "FunctionLink"],
 " is used in patterns to exclude certain matches. With two arguments, the \
first argument specifies patterns to exclude (in this case ",
 StyleBox["U", "Input"],
 "), and the second argument is the pattern to include."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{"ex14R", ",", 
   RowBox[{"Except", "[", 
    RowBox[{"U", ",", "_Symbol"}], "]"}], ",", "Infinity"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"c", ",", "b", ",", "a"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We apply ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " in order to remove duplicates and put the results in a standard order. \
This is everything we need to create ",
 StyleBox["getVarsSets", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"getVarsSets", "[", "S_", "]"}], ":=", 
  RowBox[{"Union", "[", 
   RowBox[{"Cases", "[", 
    RowBox[{"S", ",", 
     RowBox[{"Except", "[", 
      RowBox[{"U", ",", "_Symbol"}], "]"}], ",", "Infinity"}], "]"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "In order to be sure to include all variables that appear in either ",
 StyleBox["ex14L", "Input"],
 " or ",
 StyleBox["ex14R", "Input"],
 ", we combine the two into a list and pass that list as the argument to ",
 StyleBox["getVarsSets", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14Vars", "=", 
  RowBox[{"getVarsSets", "[", 
   RowBox[{"{", 
    RowBox[{"ex14L", ",", "ex14R"}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", "b", ",", "c"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Producing the Rows of the Table", "Subsubsection"],

Cell[TextData[{
 "In Section 1.3, we created a procedure called ",
 StyleBox["nextTA", "Input"],
 ". This function was responsible for producing the truth value assignments \
for the variables. In other words, it produced the rows of the truth table. \
Look again at the membership table above. Observe that the rows correspond to \
the members of the Cartesian product"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{Cell["\<\
{0,1}\[Times]{0,1}={(0,0), (0,1), (1,0), (1,1)}\
\>"], "."}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "In fact, the Cartesian product is exactly suited to what we need. The rows \
of the table are all the possible choices of 0s and 1s for the variables. The \
Cartesian product of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 " with itself is the collection of all possible tuples with each entry in \
the tuple equal to 0 or to 1."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Tuples",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tuples"], "FunctionLink"],
 " function, described earlier, produces the list of all possible tuples \
whose members are given by the first argument and whose size is given by the \
second."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tuples", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
You can see that the results are identical to the first three columns of \
Table 2 of Section 2.2 in the textbook.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Building Sets to Correspond to the Table Rows", "Subsubsection"],

Cell[TextData[{
 "We need to build sets whose entries are determined by the rows of the \
membership table (i.e., by the elements of a Cartesian power of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 ", as above). The sets, corresponding to what we called ",
 StyleBox["setA", "Input"],
 " and ",
 StyleBox["setB", "Input"],
 " in the example, will be stored in a list. That is, we\[CloseCurlyQuote]ll \
create a list of sets. These sets are identified with the variables in the \
identity to be checked as follows: the set in position ",
 StyleBox["i", "Input"],
 " in the list of sets corresponds to the variable in position ",
 StyleBox["i", "Input"],
 " in the list that results from ",
 StyleBox["getVarsSets", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Begin by initializing a list of the right size (the number of variables) \
whose entries are the empty set.",
 " We use the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function to create multiple copies of the empty set."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"memberTableSets", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "3", "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we can access and modify the lists as usual. For instance, to add \
5 to the second set we use the ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " function, which modifies a list given as the first argument by adding the \
second argument to the end of the list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AppendTo", "[", 
  RowBox[{
   RowBox[{"memberTableSets", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "5", "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["memberTableSets"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Let\[CloseCurlyQuote]s re-initialize this so we can use it below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"memberTableSets", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "3", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We also compute and store the Cartesian product. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cartesianMembership", "=", 
  RowBox[{"Tuples", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "3"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop with a loop variable ",
 StyleBox["rownum", "Input"],
 ". This variable corresponds to the row number in the membership table. The ",
 StyleBox["rownum", "Input"],
 " is also used as the index into the Cartesian product. Remember from the \
discussion above that the row number is added to those sets corresponding to \
1s in the appropriate position. Below, we have the loop print out the index, \
the tuple from the Cartesian product, and the current state of ",
 StyleBox["memberTableSets", "Input"],
 " to show the progress of the sets as they are built."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"rownum", "=", "1"}], ",", 
   RowBox[{"rownum", "\[LessEqual]", 
    RowBox[{"Length", "[", "cartesianMembership", "]"}]}], ",", 
   RowBox[{"rownum", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"currentTuple", "=", 
     RowBox[{"cartesianMembership", "[", 
      RowBox[{"[", "rownum", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"i", "=", "1"}], ",", 
      RowBox[{"i", "\[LessEqual]", "3"}], ",", 
      RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"currentTuple", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{
          RowBox[{"memberTableSets", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "rownum"}], "]"}]}], 
       "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Print", "[", 
     RowBox[{
     "rownum", ",", "\"\< \>\"", ",", "currentTuple", ",", "\"\< \>\"", ",", 
      "memberTableSets"}], "]"}]}]}], "\[IndentingNewLine]", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}]}], "}"}]}],
  SequenceForm[1, " ", {0, 0, 0}, " ", {{}, {}, {}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "}"}]}],
  SequenceForm[2, " ", {0, 0, 1}, " ", {{}, {}, {2}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"3", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "3", "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "}"}]}],
  SequenceForm[3, " ", {0, 1, 0}, " ", {{}, {3}, {2}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"4", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4"}], "}"}]}], "}"}]}],
  SequenceForm[4, " ", {0, 1, 1}, " ", {{}, {3, 4}, {2, 4}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"5", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "5", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4"}], "}"}]}], "}"}]}],
  SequenceForm[5, " ", {1, 0, 0}, " ", {{5}, {3, 4}, {2, 4}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"6", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"5", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "6"}], "}"}]}], "}"}]}],
  SequenceForm[6, " ", {1, 0, 1}, " ", {{5, 6}, {3, 4}, {2, 4, 6}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"7", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "6"}], "}"}]}], "}"}]}],
  SequenceForm[7, " ", {1, 1, 0}, " ", {{5, 6, 7}, {3, 4, 7}, {2, 4, 6}}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"8", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4", ",", "7", ",", "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}], "}"}]}],
  SequenceForm[
  8, " ", {1, 1, 1}, " ", {{5, 6, 7, 8}, {3, 4, 7, 8}, {2, 4, 6, 8}}],
  Editable->False]], "Print"]
}, Open  ]],

Cell["\<\
We also need the universe represented. The universe, in this context, is the \
set of row numbers.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14U", "=", 
  RowBox[{"Range", "[", "8", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8"}],
   "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Evaluating the Identity", "Subsubsection"],

Cell[TextData[{
 "Once the list of sets is built up, all that remains is to evaluate the \
identity with these sets in place of the names. We do this with the same \
technique as in Section 1.3 of this manual. We use ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " to turn the list of variables obtained by ",
 StyleBox["getVarsSets", "Input"],
 " and the list of sets stored as ",
 StyleBox["memberTableSets", "Input"],
 " into a list of rules."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"memberRules", "=", 
  RowBox[{"MapThread", "[", 
   RowBox[{"Rule", ",", 
    RowBox[{"{", 
     RowBox[{"ex14Vars", ",", "memberTableSets"}], "}"}]}], "]"}]}]], "Input",\

 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}]}], ",", 
   RowBox[{"b", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "4", ",", "7", ",", "8"}], "}"}]}], ",", 
   RowBox[{"c", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We add to this a rule for the universe.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AppendTo", "[", 
  RowBox[{"memberRules", ",", 
   RowBox[{"U", "\[Rule]", "ex14U"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}]}], ",", 
   RowBox[{"b", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "4", ",", "7", ",", "8"}], "}"}]}], ",", 
   RowBox[{"c", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}], ",", 
   RowBox[{"U", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", 
      "8"}], "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The question we\[CloseCurlyQuote]re trying to answer is whether the set \
expressions stored in ",
 StyleBox["ex14L", "Input"],
 " and ",
 StyleBox["ex14R", "Input"],
 " are identical. We use ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") to substitute the lists in for the variables in the two expressions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14L", "/.", "memberRules"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"Complement", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", 
      "8"}], "}"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}], "\[Union]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"3", ",", "4", ",", "7", ",", "8"}], "}"}], "\[Intersection]", 
      
      RowBox[{"{", 
       RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}]}]}], "]"}], 
  "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex14R", "/.", "memberRules"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", 
         "8"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}], "]"}], 
     "\[Union]", 
     RowBox[{"Complement", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", 
         "8"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "7", ",", "8"}], "}"}]}], "]"}]}], ")"}], 
   "\[Intersection]", 
   RowBox[{"Complement", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", 
       "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}]}], "]"}]}], 
  "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that the substitution happened within the ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 ". To tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that it should now evaluate the set, we apply ",
 StyleBox[ButtonBox["ReleaseHold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReleaseHold"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", 
  RowBox[{"ex14L", "/.", "memberRules"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", 
  RowBox[{"ex14R", "/.", "memberRules"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") operator confirms the equality of the sets. To be safe, we apply ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " before using ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==)", "Input"],
 " so as to ensure we do not get an incorrect result due to order or \
duplication."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Union", "[", 
   RowBox[{"ReleaseHold", "[", 
    RowBox[{"ex14L", "/.", "memberRules"}], "]"}], "]"}], "==", 
  RowBox[{"Union", "[", 
   RowBox[{"ReleaseHold", "[", 
    RowBox[{"ex14R", "/.", "memberRules"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Function", "Subsubsection"],

Cell["\<\
Finally, we combine it all into a single function that accepts two held \
expressions as arguments and uses the technique of membership tables to \
determine if the expressions are a set identity.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"membershipTable", "[", 
   RowBox[{"L_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "vars", ",", "numvars", ",", "cartesian", ",", "setList", ",", "rownum", 
      ",", "currentTuple", ",", "i", ",", "universe", ",", "ruleList"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vars", "=", 
      RowBox[{"getVarsSets", "[", 
       RowBox[{"{", 
        RowBox[{"L", ",", "R"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numvars", "=", 
      RowBox[{"Length", "[", "vars", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"cartesian", "=", 
      RowBox[{"Tuples", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}], ",", "numvars"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"setList", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", "numvars", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"rownum", "=", "1"}], ",", 
       RowBox[{"rownum", "\[LessEqual]", 
        RowBox[{"2", "^", "numvars"}]}], ",", 
       RowBox[{"rownum", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"currentTuple", "=", 
         RowBox[{"cartesian", "[", 
          RowBox[{"[", "rownum", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"i", "=", "1"}], ",", 
          RowBox[{"i", "\[LessEqual]", "numvars"}], ",", 
          RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"currentTuple", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "1"}], ",", 
            RowBox[{"AppendTo", "[", 
             RowBox[{
              RowBox[{"setList", "[", 
               RowBox[{"[", "i", "]"}], "]"}], ",", "rownum"}], "]"}]}], 
           "]"}]}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"universe", "=", 
      RowBox[{"Range", "[", 
       RowBox[{"2", "^", "numvars"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ruleList", "=", 
      RowBox[{"MapThread", "[", 
       RowBox[{"Rule", ",", 
        RowBox[{"{", 
         RowBox[{"vars", ",", "setList"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"ruleList", ",", 
       RowBox[{"U", "\[Rule]", "universe"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Union", "[", 
       RowBox[{"ReleaseHold", "[", 
        RowBox[{"L", "/.", "ruleList"}], "]"}], "]"}], "==", 
      RowBox[{"Union", "[", 
       RowBox[{"ReleaseHold", "[", 
        RowBox[{"R", "/.", "ruleList"}], "]"}], "]"}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[TextData[{
 "We now use our function to check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", "-", "B"}], ")"}], "-", "C"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", "-", "C"}], ")"}], "-", 
     RowBox[{"(", 
      RowBox[{"B", "-", "C"}], ")"}]}]}], TraditionalForm]]],
 ". Recall that ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 " is in fact an implementation of set difference. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"membershipTable", "[", 
  RowBox[{
   RowBox[{"Hold", "[", 
    RowBox[{"Complement", "[", 
     RowBox[{
      RowBox[{"Complement", "[", 
       RowBox[{"a", ",", "b"}], "]"}], ",", "c"}], "]"}], "]"}], ",", 
   RowBox[{"Hold", "[", 
    RowBox[{"Complement", "[", 
     RowBox[{
      RowBox[{"Complement", "[", 
       RowBox[{"a", ",", "c"}], "]"}], ",", 
      RowBox[{"Complement", "[", 
       RowBox[{"b", ",", "c"}], "]"}]}], "]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox[
     RowBox[{"A", "\[Union]", "B"}], "_"], "\[NotEqual]", 
    RowBox[{
     OverscriptBox["A", "_"], "\[Union]", 
     OverscriptBox["B", "_"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"membershipTable", "[", 
  RowBox[{
   RowBox[{"Hold", "[", 
    RowBox[{"Complement", "[", 
     RowBox[{"U", ",", 
      RowBox[{"Union", "[", 
       RowBox[{"a", ",", "b"}], "]"}]}], "]"}], "]"}], ",", 
   RowBox[{"Hold", "[", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"Complement", "[", 
       RowBox[{"U", ",", "a"}], "]"}], ",", 
      RowBox[{"Complement", "[", 
       RowBox[{"U", ",", "b"}], "]"}]}], "]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Holding the Argument Automatically", "Subsubsection"],

Cell[TextData[{
 "We have constructed the ",
 StyleBox["membershipTable", "Input"],
 " function with the restriction that the arguments must be manually held \
when the function is called. Here, we will remove this inconvenience. "
}], "Text"],

Cell[TextData[{
 "An important consequence of ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " is that we are able to manipulate the contents of an expression with head ",
 StyleBox["Hold", "Input"],
 " without it being evaluated. For example, we might want to change the head ",
 StyleBox["Union", "Input"],
 " to a ",
 StyleBox["List", "Input"],
 ". To achieve this, recall that we can use ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") to access parts of any expression, not just lists. For example, consider \
the expression below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "=", 
  RowBox[{"f", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"{", 
     RowBox[{"c", ",", "d"}], "}"}], ",", "e"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"c", ",", "d"}], "}"}], ",", "e"}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
By accessing the 0 position, we can examine the head of the expression.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "[", 
  RowBox[{"[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData["f"], "Output"]
}, Open  ]],

Cell["Positions 1 through 4 access the arguments.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData["a"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "[", 
  RowBox[{"[", "4", "]"}], "]"}]], "Input"],

Cell[BoxData["e"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The third argument is itself an expression and consequently its head and \
arguments can be accessed by ",
 StyleBox["[[3,\[Ellipsis]]]", "Input"],
 ", where the comma is followed by the position within the subexpression, \
i.e., ",
 StyleBox["[[3,0]]", "Input"],
 " for the head, ",
 StyleBox["[[3,1]]", "Input"],
 " for the first argument, etc."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "[", 
  RowBox[{"[", 
   RowBox[{"3", ",", "0"}], "]"}], "]"}]], "Input"],

Cell[BoxData["List"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partExample", "[", 
  RowBox[{"[", 
   RowBox[{"3", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData["c"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Recall the example ",
 StyleBox["holdExample", "Input"],
 " we created above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["holdExample"], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
   RowBox[{"a", "\[Intersection]", "c"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In this held expression, the index ",
 StyleBox["[[1]]", "Input"],
 " refers to the set expression, while ",
 StyleBox["[[1,0]]", "Input"],
 " refers to the head of the set expression. So assigning that to the ",
 StyleBox["List", "Input"],
 " head will transform the union into a list. And this change is accomplished \
without any part of the held expression being evaluated."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"holdExample", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "0"}], "]"}], "]"}], "=", "List"}], 
  ";"}], "\[IndentingNewLine]", "holdExample"}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Intersection]", "b"}], ",", 
    RowBox[{"a", "\[Intersection]", "c"}]}], "}"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " function allows you to pull out a part of an expression and nearly \
simultaneously apply a head to the part accessed. To do this, the first \
argument is the main expression you wish to extract from, the second argument \
is a list specifying the position being extracted, and the third argument is \
the head you wish to apply to the extracted part. For example, we can extract \
the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[Intersection]", "c"}], TraditionalForm]]],
 " from ",
 StyleBox["holdExample", "Input"],
 " and ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " it, as follows. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{"holdExample", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"a", "\[Intersection]", "c"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "While ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " acts on an expression to explicitly prevent its evaluation, ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " is an attribute that tells a function to delay evaluation of its \
arguments. Ordinarily, when you evaluate an expression involving a function \
call, any arguments to the function are evaluated first, before they are sent \
to the function. Our ultimate goal in this section is to write functions that \
can operate on set expressions. In particular, we would like to be able to \
call expressions like the one shown below to create a membership table for a \
set."
}], "Text"],

Cell[BoxData[
 RowBox[{"membershipTable", "[", 
  RowBox[{
   RowBox[{"Complement", "[", 
    RowBox[{"U", ",", 
     RowBox[{"Union", "[", 
      RowBox[{"a", ",", "b"}], "]"}]}], "]"}], ",", "\n", "    ", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "a"}], "]"}], ",", 
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "b"}], "]"}]}], "]"}]}], "]"}]], "myCode",
 Evaluatable->False,
 InitializationCell->False],

Cell[TextData[{
 "In order for the above to work, either we must include a ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " as part of the argument each time we want to use the function, as we did \
above, or we have to prevent ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from evaluating the argument long enough for us to explicitly place a ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " on the argument within the body of the function definition. Clearly, \
avoiding the need for including ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " with every function call is preferable. The ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " attribute allows us to do this."
}], "Text"],

Cell[TextData[{
 "To see how this works, we will design a small function to use as an \
example. This function will print its argument as entered by applying ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " to the argument. ",
 "Here is the definition of the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printHold", "[", "x_", "]"}], ":=", 
  RowBox[{"Print", "[", 
   RowBox[{"Hold", "[", "x", "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We expect this function to take an argument, for instance ",
 StyleBox["1+2", "Input"],
 ", and print ",
 StyleBox["1+2", "Input"],
 " (with the ",
 StyleBox["Hold", "Input"],
 " head). For the moment, however, it does not."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", "3", "]"}]], "Print"]
}, Open  ]],

Cell[TextData[{
 "The problem is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is evaluating the argument, ",
 StyleBox["1+2", "Input"],
 ", before actually applying the function. So when the body of the function \
is called, ",
 StyleBox["x", "Input"],
 " has value ",
 StyleBox["3", "Input"],
 ", not ",
 StyleBox["1+2", "Input"],
 ". To fix this, we assign the ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " attribute to the function by calling ",
 StyleBox[ButtonBox["SetAttributes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetAttributes"], "FunctionLink"],
 " with the function name and the name of the attribute as arguments."
}], "Text"],

Cell[BoxData[
 RowBox[{"SetAttributes", "[", 
  RowBox[{"printHold", ",", "HoldAll"}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now when we execute the function, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not evaluate the argument, so that the explicit ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " is able to be applied to the expression ",
 StyleBox["1+2", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Print"]
}, Open  ]],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " attribute is a bit of a misnomer. It does not cause the arguments to the \
function to be held, in the sense of applying the ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " function to them. It is a temporary hold, and the first time the argument \
is used within the body of the function definition, it will be evaluated, \
unless it is explicitly held. In other words, the ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " within the ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " is required, or else 1+2 will be evaluated to 3 within the execution of ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "In addition to calling ",
 StyleBox["membershipTable", "Input"],
 " with set expressions, we also want to be able to apply the function to \
symbols that store set expressions. Using ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 "), we can assign an expression to a symbol without the expression being \
evaluated."
}], "Text"],

Cell[BoxData[
 RowBox[{"delayedExample", ":=", 
  RowBox[{"Union", "[", 
   RowBox[{
    RowBox[{"Intersection", "[", 
     RowBox[{"a", ",", "b"}], "]"}], ",", 
    RowBox[{"Intersection", "[", 
     RowBox[{"a", ",", "c"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Because we used ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 "), the symbol ",
 StyleBox["delayedExample", "Input"],
 " holds the unevaluated expression. However, if we evaluate the symbol, we \
are faced with the warnings and unwanted simplification that we saw before."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["delayedExample"], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] c\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b \
\[Intersection] c\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(Intersection :: normal\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"a", "\[Intersection]", "b", "\[Intersection]", "c"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["printHold", "Input"],
 " function, applied to the symbol, does not print the expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold", "[", "delayedExample", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", "delayedExample", "]"}]], "Print"]
}, Open  ]],

Cell[TextData[{
 "We seem to have a conundrum. We cannot allow the symbol to be evaluated or \
it will produce errors, but if we do not evaluate it, all we see is the name \
of the symbol. To deal with this, we will use ",
 StyleBox[ButtonBox["OwnValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/OwnValues"], "FunctionLink"],
 ", a function that reveals the value assigned to a symbol, expressed as a \
list of transformation rules. Observe what happens when we apply ",
 StyleBox[ButtonBox["OwnValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/OwnValues"], "FunctionLink"],
 " to ",
 StyleBox["delayedExample", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OwnValues", "[", "delayedExample", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"HoldPattern", "[", "delayedExample", "]"}], "\[RuleDelayed]", 
   RowBox[{
    RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
    RowBox[{"a", "\[Intersection]", "c"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s explain this output a bit. First, the ",
 StyleBox[ButtonBox["HoldPattern",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldPattern"], "FunctionLink"],
 " function, which is applied to the name ",
 StyleBox["delayedExample", "Input"],
 ", can be thought of, in this context, as a hold for the left hand side of a \
rule. The \[RuleDelayed] symbol is for ",
 StyleBox[ButtonBox["RuleDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RuleDelayed"], "FunctionLink"],
 ", indicating that our assignment was delayed. Finally, the right hand side \
of the \[RuleDelayed] is our original assignment."
}], "Text"],

Cell[TextData[{
 "By looking at this in ",
 ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"],
 ", we can more easily see how we can access the expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"OwnValues", "[", "delayedExample", "]"}], "//", 
  "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", 
    RowBox[{"RuleDelayed", "[", 
     RowBox[{
      RowBox[{"HoldPattern", "[", "delayedExample", "]"}], ",", 
      RowBox[{"Union", "[", 
       RowBox[{
        RowBox[{"Intersection", "[", 
         RowBox[{"a", ",", "b"}], "]"}], ",", 
        RowBox[{"Intersection", "[", 
         RowBox[{"a", ",", "c"}], "]"}]}], "]"}]}], "]"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This expression is a list containing a single element. That element has \
head ",
 StyleBox[ButtonBox["RuleDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RuleDelayed"], "FunctionLink"],
 ", which itself has two arguments, the second of which is our expression. \
Consequently, we can access the expression by use of ",
 StyleBox["[[1,2]]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"OwnValues", "[", "delayedExample", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Intersection", "::", "normal"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Nonatomic expression expected at position \
\[NoBreak]\\!\\(1\\)\[NoBreak] in \[NoBreak]\\!\\(a \[Intersection] b \
\[Intersection] c\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/normal\\\", ButtonNote -> \
\\\"Intersection::normal\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"a", "\[Intersection]", "b", "\[Intersection]", "c"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Sadly, as soon as we access the expression, it is evaluated. So we use ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " instead."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{
   RowBox[{"OwnValues", "[", "delayedExample", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
   RowBox[{"a", "\[Intersection]", "c"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now know how to handle the argument to our function, whether it is an \
expression or a symbol that is assigned to an expression. To distinguish \
between these, we only need to test whether the head of the input is ",
 StyleBox[ButtonBox["Symbol",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Symbol"], "FunctionLink"],
 " or not. Given the argument to a function with ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " attribute, we can determine the head of the argument, while still \
preventing the argument from being evaluated, as follows: apply ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " explicitly and then access ",
 StyleBox["[[1,0]]", "Input"],
 ". This is the correct \
\[OpenCurlyDoubleQuote]address\[CloseCurlyDoubleQuote] for the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator because the 1 refers to the argument of the ",
 ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"],
 ", and the 0 then refers to the head of what is being held."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hold", "[", "delayedExample", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "0"}], "]"}], "]"}]], "Input"],

Cell[BoxData["Symbol"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Here now is our improved version of the ",
 StyleBox["printHold", "Input"],
 " function. Recall that it was previously given the ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 " attribute."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printHold", "[", "x_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "y", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Hold", "[", "x", "]"}], "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "0"}], "]"}], "]"}], "===", "Symbol"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"OwnValues", "[", "x", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "=", 
        RowBox[{"Hold", "[", "x", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "y", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[TextData[{
 "If we pass a variable to ",
 StyleBox["printHold", "Input"],
 ", the if condition identifies it as a symbol and uses ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["OwnValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/OwnValues"], "FunctionLink"],
 " to get access to the definition."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold", "[", "delayedExample", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
   RowBox[{"a", "\[Intersection]", "c"}]}], "]"}]], "Print"]
}, Open  ]],

Cell["\<\
On the other hand, if we give it an expression, then the argument is not a \
symbol and is held just as it was called.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Print"]
}, Open  ]],

Cell[TextData[{
 "We also want the function to behave properly if it is given a held \
expression. In particular, we want to avoid nesting holds. To do this, we\
\[CloseCurlyQuote]ll replace the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement with a ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 ". The first argument of a ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " is evaluated. In this case, the first argument will evaluate to the head \
of the input. The ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " then looks at the even indexed arguments until it finds one that matches \
the result of the first argument and it evaluates the argument after the one \
that matches. We include a ",
 StyleBox[ButtonBox["Blank",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blank"], "FunctionLink"],
 " (",
 StyleBox["_", "Input"],
 ") as the next to last argument, which means that the final argument will \
serve as a default."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printHold", "[", "x_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "y", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Switch", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Hold", "[", "x", "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "0"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       "Hold", ",", 
       RowBox[{"y", "=", "x"}], ",", "\[IndentingNewLine]", "Symbol", ",", 
       RowBox[{"y", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"OwnValues", "[", "x", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]}], ",", 
       "\[IndentingNewLine]", "_", ",", 
       RowBox[{"y", "=", 
        RowBox[{"Hold", "[", "x", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "y", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We\[CloseCurlyQuote]ll be using this technique several times in what follows, \
so we\[CloseCurlyQuote]ll encapsulate it as its own function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"SetAttributes", "[", 
  RowBox[{"holdArgument", ",", "HoldAll"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"holdArgument", "[", "x_", "]"}], ":=", 
  RowBox[{"Switch", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Hold", "[", "x", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "0"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
    "Hold", ",", "x", ",", "\[IndentingNewLine]", "Symbol", ",", 
    RowBox[{"Extract", "[", 
     RowBox[{
      RowBox[{"OwnValues", "[", "x", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}], ",", 
    "\[IndentingNewLine]", "_", ",", 
    RowBox[{"Hold", "[", "x", "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "This allows us to embed this part of the argument processing into the \
declaration of the module variables, as shown in the following alternate \
version of ",
 StyleBox["printHold", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printHold2", "[", "x_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"y", "=", 
      RowBox[{"holdArgument", "[", "x", "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Print", "[", "y", "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"SetAttributes", "[", 
  RowBox[{"printHold2", ",", "HoldAll"}], "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold2", "[", "delayedExample", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"a", "\[Intersection]", "b"}], "\[Union]", 
   RowBox[{"a", "\[Intersection]", "c"}]}], "]"}]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold2", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "+", "2"}], "]"}]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printHold2", "[", 
  RowBox[{"Hold", "[", 
   RowBox[{"p", "&&", "q"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"p", "&&", "q"}], "]"}]], "Print"]
}, Open  ]],

Cell[TextData[{
 "We now use the ",
 StyleBox["holdArgument", "Input"],
 " function to update ",
 StyleBox["membershipTable", "Input"],
 "."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"membershipTable", ",", "HoldAll"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"membershipTable", "[", 
    RowBox[{"LS_", ",", "RS_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"holdArgument", "[", "LS", "]"}]}], ",", 
       RowBox[{"R", "=", 
        RowBox[{"holdArgument", "[", "RS", "]"}]}], ",", "vars", ",", 
       "numvars", ",", "cartesian", ",", "setList", ",", "rownum", ",", 
       "currentTuple", ",", "i", ",", "universe", ",", "ruleList"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vars", "=", 
       RowBox[{"getVarsSets", "[", 
        RowBox[{"{", 
         RowBox[{"L", ",", "R"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"numvars", "=", 
       RowBox[{"Length", "[", "vars", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"cartesian", "=", 
       RowBox[{"Tuples", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}], ",", "numvars"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"setList", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", "numvars", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"rownum", "=", "1"}], ",", 
        RowBox[{"rownum", "\[LessEqual]", 
         RowBox[{"2", "^", "numvars"}]}], ",", 
        RowBox[{"rownum", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"currentTuple", "=", 
          RowBox[{"cartesian", "[", 
           RowBox[{"[", "rownum", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"i", "=", "1"}], ",", 
           RowBox[{"i", "\[LessEqual]", "numvars"}], ",", 
           RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"currentTuple", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "1"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{
               RowBox[{"setList", "[", 
                RowBox[{"[", "i", "]"}], "]"}], ",", "rownum"}], "]"}]}], 
            "]"}]}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"universe", "=", 
       RowBox[{"Range", "[", 
        RowBox[{"2", "^", "numvars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"ruleList", "=", 
       RowBox[{"MapThread", "[", 
        RowBox[{"Rule", ",", 
         RowBox[{"{", 
          RowBox[{"vars", ",", "setList"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"ruleList", ",", 
        RowBox[{"U", "\[Rule]", "universe"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Union", "[", 
        RowBox[{"ReleaseHold", "[", 
         RowBox[{"L", "/.", "ruleList"}], "]"}], "]"}], "==", 
       RowBox[{"Union", "[", 
        RowBox[{"ReleaseHold", "[", 
         RowBox[{"R", "/.", "ruleList"}], "]"}], "]"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now, we can omit the explicit calls to ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"membershipTable", "[", 
  RowBox[{
   RowBox[{"Complement", "[", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{"a", ",", "b"}], "]"}], ",", "c"}], "]"}], ",", 
   RowBox[{"Complement", "[", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{"a", ",", "c"}], "]"}], ",", 
     RowBox[{"Complement", "[", 
      RowBox[{"b", ",", "c"}], "]"}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"membershipTable", "[", 
  RowBox[{
   RowBox[{"Complement", "[", 
    RowBox[{"U", ",", 
     RowBox[{"Union", "[", 
      RowBox[{"a", ",", "b"}], "]"}]}], "]"}], ",", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "a"}], "]"}], ",", 
     RowBox[{"Complement", "[", 
      RowBox[{"U", ",", "b"}], "]"}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computer Representation of Fuzzy Sets", "Subsection",
 CellTags->"Section2c"],

Cell[TextData[{
 "The textbook describes a way to represent sets as bit strings in order to \
efficiently store and compute with them. Here, we will explore this idea \
further in order to see how we can represent fuzzy sets in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Fuzzy sets are described in the preamble to Exercise 63 in Section 2.2."
}], "Text"],

Cell[CellGroupData[{

Cell["Two Representations of Fuzzy Sets", "Subsubsection"],

Cell["\<\
In a fuzzy set, every element has an associated degree of membership, which \
is a real number between 0 and 1. We will represent fuzzy sets in two \
different ways.\
\>", "Text"],

Cell[TextData[{
 "The first way we can represent a fuzzy set in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is to combine the element with the degree of membership as a two-element \
list. For example, if the elements of our fuzzy set are the letters \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote], \[OpenCurlyDoubleQuote]b\
\[CloseCurlyDoubleQuote], and \
\[OpenCurlyDoubleQuote]e\[CloseCurlyDoubleQuote], where \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote] has degree of membership \
0.3, \[OpenCurlyDoubleQuote]b\[CloseCurlyDoubleQuote] has degree 0.7, and \
\[OpenCurlyDoubleQuote]e\[CloseCurlyDoubleQuote] has degree 0.1, then we \
would represent the set as:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyRoster", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "0.3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<b\>\"", ",", "0.7"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<e\>\"", ",", "0.1"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "0.3`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "0.7`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"e\"\>", ",", "0.1`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We refer to this as the \[OpenCurlyDoubleQuote]roster representation\
\[CloseCurlyDoubleQuote] of the fuzzy set.\
\>", "Text"],

Cell["\<\
The second approach is to use a \[OpenCurlyDoubleQuote]fuzzy-bit string\
\[CloseCurlyDoubleQuote] in essentially the same way as described in the \
text. First we need to specify the universe and impose an order on it. Let\
\[CloseCurlyQuote]s say the universe consists of the letters \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote] through \
\[OpenCurlyDoubleQuote]g\[CloseCurlyDoubleQuote] ordered alphabetically. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyU", "=", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
    "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\""}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then the fuzzy-bit string for the set ",
 StyleBox["fuzzyR", "Input"],
 " will be the list of the degrees of membership of each element of the \
universe with 0 indicating non-membership."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyBitString", "=", 
  RowBox[{"{", 
   RowBox[{
   "0.3", ",", "0.7", ",", "0", ",", "0", ",", "0.1", ",", "0", ",", "0"}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.3`", ",", "0.7`", ",", "0", ",", "0", ",", "0.1`", ",", "0", ",", "0"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converting from Bit String to Roster Representation", "Subsubsection"],

Cell[TextData[{
 "Converting from a fuzzy-bit string to the roster representation is fairly \
straightforward. Use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop with index running from 1 to the number of elements in the universe. \
For each index, if the entry in the fuzzy-bit string is non-zero, then we add \
to the roster the pair consisting of the element from the universe and the \
degree of membership."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bitToRoster", "[", 
   RowBox[{"bitstring_", ",", "universe_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"S", "=", 
       RowBox[{"{", "}"}]}], ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "universe", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"bitstring", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "\[NotEqual]", "0"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"S", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"universe", "[", 
              RowBox[{"[", "i", "]"}], "]"}], ",", 
             RowBox[{"bitstring", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}], "}"}]}], "]"}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "S"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we give ",
 StyleBox["S", "Input"],
 " as the last expression so that it will be the output of the function. \
Otherwise, the ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop will produce no output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitToRoster", "[", 
  RowBox[{"fuzzyBitString", ",", "fuzzyU"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "0.3`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "0.7`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"e\"\>", ",", "0.1`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converting from Roster to Bit String Representation", "Subsubsection"],

Cell[TextData[{
 "In the other direction, we initialize a bit string to the 0-string. Then we \
consider each member of the roster representation in turn, using a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop over the elements of the roster. Recall that with second argument of \
the form ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["var", "CodeVar"], ",", 
   StyleBox["list", "CodeVar"], "}"}]], "myCode"],
 " the variable will be assigned to each element of the list in turn. In this \
case, ",
 Cell[BoxData[
  FormBox["list", CodeVar]], "myCode"],
 " will be the roster representation and so the variable will be assigned to \
each of the lists consisting of the element/membership pairs."
}], "Text"],

Cell[TextData[{
 "For every member of the set, we will need to determine the position of the \
set member in the universe in order to change the correct bit in the \
fuzzy-bit string.",
 " To do this, we will make use of the ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " function. Given two arguments, ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " will return a list of lists specifying all the positions at which the \
second argument appears in the first. The following illustrates using ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " to determine that 2 appears in locations 3 and 6 in the list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "3", ",", "5", ",", "2", ",", "9", ",", "4", ",", "2", ",", "1", ",", 
     "8"}], "}"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "6", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reason ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " returns a list of this form is in case of nesting in the first argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"9", ",", "7", ",", "1", ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "0", ",", "7"}], "}"}], ",", "5"}], "}"}], ",", "7"}],
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The above shows that 7 appears in position 2 and in position 3 of the sublist \
at location 4.\
\>", "Text"],

Cell[TextData[{
 "In the current context, we will assume that our data is well formed, in \
which case when we apply ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " to the universe, there is only one instance of the sought-after object and \
there is no nesting. Therefore, we can access the appropriate index by \
applying the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Position", "[", 
   RowBox[{"fuzzyU", ",", "\"\<b\>\""}], "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we can write the ",
 StyleBox["rosterToBit", "Input"],
 " function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rosterToBit", "[", 
   RowBox[{"roster_", ",", "universe_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"B", ",", "e", ",", "position"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"B", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{"Length", "[", "universe", "]"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"position", "=", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"universe", ",", 
            RowBox[{"e", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"B", "[", 
          RowBox[{"[", "position", "]"}], "]"}], "=", 
         RowBox[{"e", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"e", ",", "roster"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "B"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rosterToBit", "[", 
  RowBox[{"fuzzyRoster", ",", "fuzzyU"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.3`", ",", "0.7`", ",", "0", ",", "0", ",", "0.1`", ",", "0", ",", "0"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3 Functions", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we will see ways to represent functions in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and explore a variety of the concepts described in the text relative to \
these different representations."
}], "Text"],

Cell[CellGroupData[{

Cell["Functions", "Subsection",
 CellTags->"Section3a"],

Cell[TextData[{
 "In this manual, we have already seen several examples of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions. In some ways, a computer program is the ultimate generalization \
of a mathematical function. As an example, consider the ",
 StyleBox["getVars", "Input"],
 " function. This function assigns to each valid input (a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " logical expression) a unique output (a list of the symbols appearing in \
the expression). Setting ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " equal to the set consisting of all logic expressions and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " equal to the set of all possible lists of valid symbols, ",
 StyleBox["getVars", "Input"],
 " satisfies the definition of being a function from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
We discussed functions in some depth in the introductory chapter. Here, we \
will discuss the concept of domain as they relate to programs via the \
computer programming concept of type.\
\>", "Text"],

Cell[TextData[{
 "In Example 5 of Section 2.3, the text gives examples from Java and C++ \
showing how domain and codomain are specified in those programming languages. \
The function below illustrates how to specify the domain of a function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"floor1", "[", "x_Real", "]"}], ":=", 
  RowBox[{"Floor", "[", "x", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The body of our ",
 StyleBox["floor1", "Input"],
 " function is merely a call to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in ",
 StyleBox[ButtonBox["Floor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Floor"], "FunctionLink"],
 " command. But the example illustrates how you can specify the domain (i.e., \
the type of a parameter) of a function."
}], "Text"],

Cell[TextData[{
 "In the example above, we declared the type of the parameter ",
 StyleBox["x", "Input"],
 " by following the blank (",
 StyleBox["_", "Input"],
 ") with the symbol ",
 StyleBox["Real", "Input"],
 ", which is the head used for floating-point numbers in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". If the function is called with an argument whose head does not match, \
then the function definition is not applied. In this case, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " merely returns the expression unevaluated."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"floor1", "[", "\"\<hello\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"floor1", "[", "\<\"hello\"\>", "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "There are four built-in numerical types in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ": ",
 StyleBox["Integer", "Input"],
 ", ",
 StyleBox["Rational", "Input"],
 ", ",
 StyleBox["Real", "Input"],
 ", and ",
 StyleBox["Complex", "Input"],
 ". But any head can be used in the same manner. For example, to create a \
function whose argument must be a list, you would define the function as \
below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"last", "[", "L_List", "]"}], ":=", 
  RowBox[{"L", "[", 
   RowBox[{"[", 
    RowBox[{"-", "1"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
This function will determine the last element of any list, but will not \
execute if it is given an object that is not a list.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"last", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"last", "[", "5", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"last", "[", "5", "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The expressions ",
 StyleBox["x_Real", "Input"],
 " and ",
 StyleBox["L_List", "Input"],
 " are examples of patterns. When you define a function, you are providing ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with a rule that tells it how to evaluate an expression matching the \
pattern. Specifically, the definition for ",
 StyleBox["last", "Input"],
 " creates an evaluation rule that tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " what to do with an expression that has head ",
 StyleBox["List", "Input"],
 ". "
}], "Text"],

Cell[TextData[{
 "You can further specify the allowed domain of a function by creating a \
pattern that uses a Boolean-valued function to determine whether the argument \
is allowable. For example, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " contains a function ",
 StyleBox[ButtonBox["EvenQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EvenQ"], "FunctionLink"],
 ", which returns true for even integers and false otherwise. The function \
below will apply only to even integers."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"half", "[", 
   RowBox[{"x_", "?", "EvenQ"}], "]"}], ":=", 
  RowBox[{"x", "/", "2"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"half", "[", "8", "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"half", "[", "9", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"half", "[", "9", "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This is called a ",
 ButtonBox["PatternTest",
  BaseStyle->"Hyperlink",
  ButtonData->{
    FrontEnd`FileName[{"pacler:ref"}, "PatternTest", CharacterEncoding -> 
     "UTF-8"], None}],
 ". The blank is followed by a question mark which is followed by the name of \
a function. When the function returns true, then the argument is considered \
to match and otherwise it does not match. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes several useful built-in functions, like ",
 StyleBox[ButtonBox["EvenQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EvenQ"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["OddQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/OddQ"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["NumberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NumberQ"], "FunctionLink"],
 ", which matches any kind of number. You can also define your own functions \
to use as the test."
}], "Text"],

Cell[TextData[{
 "Specifying the allowable type in a function definition is useful because it \
helps to ensure that the function is never applied to invalid input, which \
may have undesirable consequences. For example, consider the functions below. \
First, we define a function ",
 StyleBox["posIntQ", "Input"],
 " that returns true only for positive integers, making use of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["IntegerQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerQ"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Positive",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Positive"], "FunctionLink"],
 " functions. Then we define the function ",
 StyleBox["loopy", "Input"],
 ", which prints and then decreases its argument by 1 (using the ",
 StyleBox[ButtonBox["Decrement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Decrement"], "FunctionLink"],
 " (",
 StyleBox["--", "Input"],
 ") operator) until it reaches 0."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"posIntQ", "[", "x_", "]"}], ":=", 
  RowBox[{
   RowBox[{"IntegerQ", "[", "x", "]"}], "&&", 
   RowBox[{"Positive", "[", "x", "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"loopy", "[", 
   RowBox[{"n_", "?", "posIntQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", "=", "n"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"While", "[", 
     RowBox[{
      RowBox[{"m", "\[NotEqual]", "0"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Print", "[", "m", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"m", "--"}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Applying this function with a positive integer has the desired result.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"loopy", "[", "3", "]"}]], "Input"],

Cell[BoxData["3"], "Print"],

Cell[BoxData["2"], "Print"],

Cell[BoxData["1"], "Print"]
}, Open  ]],

Cell[TextData[{
 "If you call the function with a value that does not satisfy the ",
 StyleBox["posIntQ", "Input"],
 " function, i.e., a value not in the domain, the function is not evaluated."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"loopy", "[", 
  RowBox[{"-", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"loopy", "[", 
  RowBox[{"-", "5"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Without the restriction that the parameter be a positive integer, applying ",
 StyleBox["loopy", "Input"],
 " to -5 would have resulted in an infinite loop."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides a syntax for imposing conditions without the need to write \
additional functions like ",
 StyleBox["posIntQ", "Input"],
 ". A ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") can be used after any pattern and can make use of the named elements of \
the pattern to create a test. The syntax is to enter the pattern, followed by \
",
 StyleBox["/;", "Input"],
 " (read \[OpenCurlyDoubleQuote]provided\[CloseCurlyDoubleQuote]), followed \
by an expression that evaluates to true on valid input. In the case of a \
function definition, the ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") can be placed within the arguments, between the closing bracket ending \
the arguments and the ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ") operator, or at the end of the function definition. In this manual, we \
typically enter conditions immediately before the ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ") operator. "
}], "Text"],

Cell[TextData[{
 "We use this method to revise the loopy function without using the ",
 StyleBox["posIntQ", "Input"],
 " auxiliary function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"loopy2", "[", "n_", "]"}], "/;", 
   RowBox[{
    RowBox[{"IntegerQ", "[", "n", "]"}], "&&", 
    RowBox[{"Positive", "[", "n", "]"}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", "=", "n"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"While", "[", 
     RowBox[{
      RowBox[{"m", "\[NotEqual]", "0"}], ",", 
      RowBox[{
       RowBox[{"Print", "[", "m", "]"}], ";", 
       RowBox[{"m", "--"}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "You can provide alternative argument forms using the ",
 StyleBox[ButtonBox["Alternatives",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Alternatives"], "FunctionLink"],
 " (",
 StyleBox["|", "Input"],
 ") operator. For example, the function below accepts either a two-element \
list or a list of three elements where the third element must be a 1."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"alternateEx", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_"}], "}"}], "|", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_", ",", "1"}], "}"}]}], "]"}], ":=", 
  RowBox[{"x", "+", "y"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
This function applies when either pattern is matched, but not otherwise. Note \
that it is important that the same variable names are used in both \
alternatives.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alternateEx", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "2"}], "}"}], "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alternateEx", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "2", ",", "1"}], "}"}], "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alternateEx", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "2", ",", "7"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"alternateEx", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "2", ",", "7"}], "}"}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
This same effect can be created by giving multiple definitions, one for each \
argument pattern.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"alternateEx2", "[", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_"}], "}"}], "]"}], ":=", 
   RowBox[{"x", "+", "y"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"alternateEx2", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_", ",", "1"}], "}"}], "]"}], ":=", 
  RowBox[{"x", "+", "y"}]}]}], "Input"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will apply whichever definition, if either, matches."
}], "Text"],

Cell[TextData[{
 "Sometimes, you may wish to give names to larger pieces of patterns. For \
this, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the name-colon-pattern syntax. For example, in the function \
below, ",
 StyleBox["p", "Input"],
 " refers to the list consisting of an integer and a string."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"repeat", "[", 
   RowBox[{"p", ":", 
    RowBox[{"{", 
     RowBox[{"_Integer", ",", "_String"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"p", "[", 
     RowBox[{"[", "2", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", "[", 
      RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"repeat", "[", 
  RowBox[{"{", 
   RowBox[{"5", ",", "\"\<Hello\>\""}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Hello\"\>", ",", "\<\"Hello\"\>", ",", "\<\"Hello\"\>", 
   ",", "\<\"Hello\"\>", ",", "\<\"Hello\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "While a single underscore matches a single expression, two underscores, \
called ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " (",
 StyleBox["_\[ThinSpace]_", "Input"],
 "), matches a comma-separated sequence of one or more objects. If you follow \
a ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " with a head or with a ",
 StyleBox["?", "Input"],
 " and function, then every element of the sequence must have the head or \
pass the test in order for the sequence to match. This provides, for example, \
a way to insist that a list have all integer members, as shown in the \
function below, which multiplies each integer by a power of 10 commensurate \
with its location in the list."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"intshift", "[", 
   RowBox[{"L", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"result", "=", "0"}], ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "L", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"result", "=", 
        RowBox[{"result", "+", 
         RowBox[{
          RowBox[{"L", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "*", 
          RowBox[{"10", "^", 
           RowBox[{"(", 
            RowBox[{"i", "-", "1"}], ")"}]}]}]}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "result"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
The result of this function is clearest to see when applied to individual \
digits.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"intshift", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], "]"}]], "Input"],

Cell[BoxData["4321"], "Output"]
}, Open  ]],

Cell["\<\
But if any element of the list is not an integer, it will not be applied.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"intshift", "[", 
  RowBox[{"{", 
   RowBox[{"2", ",", "7", ",", "1", ",", "3.4"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"intshift", "[", 
  RowBox[{"{", 
   RowBox[{"2", ",", "7", ",", "1", ",", "3.4`"}], "}"}], "]"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pure Functions", "Subsection",
 CellTags->"Section3b"],

Cell[TextData[{
 "A pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is a way to define a simple function and is typically used when you have a \
function that you will only use once or when you want to define the function \
within the body of another function."
}], "Text"],

Cell[TextData[{
 "The typical syntax for a pure function is to enter the function body \
followed by an ampersand (",
 StyleBox["&", "Input"],
 "). When the function accepts a single argument, the pound (",
 StyleBox["#", "Input"],
 ") symbol (called a ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "Input"],
 ") is used as the name of the argument. For example, the following is the \
pure function for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[Rule]", 
    RowBox[{
     SuperscriptBox["x", "2"], "+", "1"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"#", "^", "2"}], "+", "1"}], "&"}]], "myCode",
 Evaluatable->False,
 InitializationCell->False],

Cell[TextData[{
 "There are two main applications for a pure function. First, you can pass it \
a value for its argument by following the ampersand with the argument \
enclosed in brackets. The following computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "2"], "+", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"#", "^", "2"}], "+", "1"}], "&"}], "[", "3", "]"}]], "Input"],

Cell[BoxData["10"], "Output"]
}, Open  ]],

Cell[TextData[{
 "More usefully, you can use a pure function any place you would normally \
give the name of a function. For example, recall the discussion of ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") in Section 1.1 of this manual. Given a function and a list, ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " applies the function to each element of the list. The following computes \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "+", "1"}], TraditionalForm]]],
 " for each member of the given list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"#", "^", "2"}], "+", "1"}], "&"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "10", ",", "26", ",", "50", ",", "122", ",", "170"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The following is identical, but using the ",
 StyleBox["/@", "Input"],
 " operator syntax."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"#", "^", "2"}], "+", "1"}], "&"}], "/@", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
   "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "10", ",", "26", ",", "50", ",", "122", ",", "170"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If you need more than one argument, you can follow the ampersand by an \
index starting at 1: ",
 StyleBox["#1", "Input"],
 " for the first argument, ",
 StyleBox["#2", "Input"],
 " for the second, etc. The following expression uses ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " to compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["y", "3"]}], TraditionalForm]]],
 " with values of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " coming from the two lists."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"#1", "^", "2"}], "+", 
     RowBox[{"#2", "^", "3"}]}], "&"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "6", ",", "8"}], "}"}]}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"9", ",", "68", ",", "225", ",", "528"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can, if you wish, name pure functions, using ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") just like you assign values to variables. Once the name is assigned, you \
evaluate the function as usual."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pureExample", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"#1", "^", "2"}], "+", 
    RowBox[{"#1", "*", "#2"}], "+", 
    RowBox[{"#2", "^", "2"}]}], "&"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["#1", "2"], "+", 
   RowBox[{"#1", " ", "#2"}], "+", 
   SuperscriptBox["#2", "2"]}], "&"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pureExample", "[", 
  RowBox[{"2", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["19"], "Output"]
}, Open  ]],

Cell["\<\
Also note that applying the pure function to symbols produces an expression \
for the function in terms of the given symbols.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pureExample", "[", 
  RowBox[{"s", ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["s", "2"], "+", 
  RowBox[{"s", " ", "t"}], "+", 
  SuperscriptBox["t", "2"]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides a functional syntax for defining pure functions with ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 ". This syntax has more flexibility, but is used less frequently, so we will \
not discuss it here."
}], "Text"],

Cell[CellGroupData[{

Cell["Composition of Functions", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s support for algebraic combination of functions is not \
as natural as you might hope, but pure functions provide a useful way to make \
exploring these ideas fairly easy. "
}], "Text"],

Cell[TextData[{
 "Consider two functions, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "=", 
    RowBox[{
     SuperscriptBox["x", "2"], "+", "1"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "=", 
    SuperscriptBox["x", "3"]}], TraditionalForm]]],
 ". We define these two functions in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " defined in the usual way and ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " as a pure function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", 
  RowBox[{
   RowBox[{"x", "^", "2"}], "+", "1"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g", "=", 
  RowBox[{
   RowBox[{"#", "^", "3"}], "&"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["#1", "3"], "&"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that you use exactly the same syntax to apply them to values.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "2", "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g", "[", "5", "]"}]], "Input"],

Cell[BoxData["125"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To create a function that is the algebraic combination of these two, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "=", 
    RowBox[{"f", "/", "g"}]}], TraditionalForm]]],
 ", we can define the combined function as a pure function obtained by \
evaluating ",
 StyleBox["f", "Input"],
 " and ",
 StyleBox["g", "Input"],
 " on the argument ",
 StyleBox["#", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"h", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "#", "]"}], "/", 
    RowBox[{"g", "[", "#", "]"}]}], "&"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"f", "[", "#1", "]"}], 
   RowBox[{"g", "[", "#1", "]"}]], "&"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we can evaluate ",
 StyleBox["h", "Input"],
 " at values or obtain a formula."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"h", "[", "3", "]"}]], "Input"],

Cell[BoxData[
 FractionBox["10", "27"]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"h", "[", "t", "]"}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{"1", "+", 
   SuperscriptBox["t", "2"]}], 
  SuperscriptBox["t", "3"]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "For composition of functions, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["Composition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Composition"], "FunctionLink"],
 " function. The following defines ",
 StyleBox["h1", "Input"],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[SmallCircle]", "g"}], TraditionalForm]]],
 " and ",
 StyleBox["h2", "Input"],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[SmallCircle]", "h"}], TraditionalForm]]],
 " and applies them both to the symbol ",
 StyleBox["t", "Input"],
 " to obtain formulas."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"h1", "=", 
   RowBox[{"Composition", "[", 
    RowBox[{"f", ",", "g"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"h1", "[", "t", "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"1", "+", 
  SuperscriptBox["t", "6"]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"h2", "=", 
   RowBox[{"Composition", "[", 
    RowBox[{"g", ",", "f"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"h2", "[", "t", "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"(", 
   RowBox[{"1", "+", 
    SuperscriptBox["t", "2"]}], ")"}], "3"]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Note that the arguments of ",
 StyleBox[ButtonBox["Composition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Composition"], "FunctionLink"],
 " can be pure functions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Composition", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"#", "^", "2"}], "&"}], ",", 
    RowBox[{
     RowBox[{"#", "+", "3"}], "&"}]}], "]"}], "[", "x", "]"}]], "Input"],

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"(", 
   RowBox[{"3", "+", "x"}], ")"}], "2"]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plotting Graphs of Functions", "Subsubsection"],

Cell[TextData[{
 "You can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draw the graph of a function by using the ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 " function. The function requires two arguments: the first is the function \
to be graphed in terms of a variable, and the second is a list containing the \
variable and the minimum and maximum values of the domain of the variable. \
Typically, you provide the function as an algebraic expression as in the \
example below, which graphs ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "-", "x", "+", "2"}], TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "5"}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["5", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", "2"}], "-", "x", "+", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwtmHk01N//x2ds2bKNZURkqSyRMPZ63ZKlhaRIFCX5lESW7PTxISKtStlK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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-5, 5}, {0., 31.999997755102083`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]],

Cell["\<\
The first argument may also be based on a named function, as the following \
examples illustrate. The essential requirement is that the first argument \
must evaluate to a numeric value whenever the variable is assigned a value.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJw1mHk41U/0x5EtlOWW7JFIlIhwLZ1BtixJReibLKm0kD1bi7W0kCVLkpSi
omhTNNeulD37VvYlPvdyubb87u95fr+/5nk9z8w5M+8z58w8R8bF0+Y0GwsL
ixArC8v/jqGjlrS1NRKlsry8d22NjEaChendKySKT9idG/IMMrIS6F/4uEii
ZPMwXKwIMpIme616zpIoDxJftWUNkFH5rfvcAyMkSv6lhJt2ZWTErdwuRflJ
onzQOS757wYZJfqeMr+WTqJIkr+uaHKR0UtWn6csmiSKbtly2kmSFuI4bbJX
9D8hylq9insySQOROqouPIkXpGR+jynNOaiObCrSXHd+F6BYphdJWD7ai4Jn
9C7EDfJTFHDWxoxBFfQpIM2j4NBGSuT9smN7Pigjq/vHpP+l8lEw6U61c5wS
MskotTKd4qEEzUgrVecooJyaK20+juspJZ5JjhNb5NHJiMTNDS+4KHT1p3u3
h8misFPr8nlJnBT22ztO62jLIJbdW7FhJDvlhIrGsauHpVBXbGr0y19slKpi
A6dRJTEkwXM2d86AlfJhmF/0VKYwOr9PkaNbYg1zE/f3sEqTkKtm34uKimX8
tEhqOJxXACkGaq9desrA6vUC3rOLPCir90jlFX065mRV+u4nxYn2Du1dalWn
4/zzouX5GzhR5YSgiooCHXvod3ykrnCgkYWG9JGNdCyuXPMwqZsDKQpZ+Bzp
mcNn+TwsjqVyoELjA9t2B8zhtz0VBu+EOVBlgfq1gVez+F9dTSwhwo5Gr2/W
NRGh4YBhgXsuOmyIJ7RgJoGHhq/I1kyNK7Eh5UCz7IEVKpaoLuQIlWBD/pdC
eYJ+U/F+suK56lVWxOk43PEqj4pX21LnhymsaLv6ez8hPSp2IIKVjpuxIpeR
owW9zgR+mGa+yu3KgqJ+T7soHiVw7IX8Mq5jLCivJ0Y4wJjAotnG+kImLIjW
XBIqoETgTfu3eB9UYkFsV81SosZmMKudt/q9sTUA1fGmwxtmcGDtGveEwBp8
TlIwGrP9i33TVG9o+awCt+OoK6vuX8zBLheZ6rAKttI5N8Rk/uImw1o5QYNV
oOXJUiwmp7CIsRQ6ILgKOylSOm+vTuEzx7g7i9+sQMrEJtWgF5OYM7M1/c/s
MvgBqyTf0jjW77M8rRi7BOXsFB25/nEsz68r5eS3BALfwxz2V45jU+Eu3zyn
JXh1dOWB191xrP1cbjlCfQmGzi0I/do2jvtbls6b9S+CTeJf7kfmY1jheEYf
P3kRVMY75/ZkjODPfBVcCasL4Evs3rklYgSHt/6OHZhcgE8L1//75zGCE/ib
v+p3LYA+p2L1D60RfLNfXhU+LoCNbNCDc63DuGYky0TeewH8/hPTzuYdxnH3
e3WeTc7D5yaHq8JBg9jG6by++186/OvIL1o9NYgX733K3NlHB4MBtrEhk0E8
otb+iqWBDnXTedZFmwfxg18xLeNv6NDDuyxj/fYPNr9WavPCjw5rRumVN8d+
YxafCyJtbHQ48Ll7/artAE79OUCZU5qD/Lxn5BqdARxiFu8tLjUHIume5+Kl
B3By5nkTe4E5aFvgvG/e2o99uzIcxOizIBdsK5Cx1IcbbzHkC/EslF+n8yGT
XszfFmWnZT8LyS43L7QK9GJfeqtOk8UsnDOU/HG2qwcLzFgYXkezIMBhFBt/
sQfTvD5/kVeYBaeYxPWDCd3YtiDWJppBg5W7apxRA504R5hLJDODBo2eNafF
8jrx7U6act59GmRbO1bl+3TiPH/ucz+iaXBQKDyinbMTx8FbY1tvGqQmNbPt
3N2BV3kXn181pYHGw8trdVfa8Il7zmw7F6nglVvAECS1YDWULNx2ngoqGy+F
Gww048fltHpLNypQvXdt9HndjJckiwo7T1DBWzdXttWkGfs5Oa3stqKCX8MT
q+SQJhy5wT2lVpUKsxNKZvt+NGAh4V2zVSsE+L57cTLZvQFb7qh2vjZPwHyo
nO8CSwP2F6xctiIIYAhszSzeV4/veE2+VRgiYEVDiK6X+QO324/J1/8ggDOc
8djY+xtu60hT+51FQLSF34fnfN/wQKD38b6HBHAL0+q4n9fidurrFzMPCODJ
nZz/3l2DYy9Z11ndJYC/oc/ikFE15k8wFF0LJiAuxdHlTX8VvrZETs0IIEDI
pSNAMKgK92knHrD2IWATvelJS34lPmXsK0nzIEBEvIphJ1KBr8f8ad/hSEDq
kP7G4sJybP9xofCMHQFi+V9lxSzLsRT2Kv5yhABJ/U9WPdfLsKf9XdPHFgQY
1tualbVhvKs4wNMRCDB2tvYN58X4fMHLZ7q6BJjOmWUaoa84JcyQay+ZAAsx
Pfq33BL88fkjC3s1AmzdZbOaQ4qxoNy6gVQFAm5d9om5//4Tjh6VqVSQJ6A0
pMLTZvoj/hLQzlMnS8D2BNf9zU4fsAMrF8VsKwHHHxXJ3U95j6uWrinLSxIQ
m7tug03TO/y3LH5tszgBNJzd3WRQhP2KA77t2UKAfN1seXxwIW4dtyi330yA
fZth3uF3bzF19JPkAxIBlKnBwCa5N9jZ8tiKuQABYzNnKNtK8/H7d5FXKzYS
cIiT+94T5de4ZtzitNUGAqTUTHc9EczD0qWZxx/xEBBpNrYkE/4Cn3NUcndd
T8CUU8y3rLkcfKnGsk2Hm4Aj/gopMu7PsCorV+4OLgI+3651z2rPxm+57frk
OQmQyT67T8bsCZ7u1fMncxAQU8zNnvX5MR679sXLiZ2AvszDz/SPP8KZ1Iaf
KesIeP0j3kBfJx2TVILvDLMRELzY1I+kUrCRbmmBEZPN5IVCEWsSVhNOVi9m
JWDLERsxNBSP+0s5JBCT69a2e169ewebqvG5d7EQcNb+WW7Y0Wh8yT9XKIbJ
7EVyQ6Fi1/Hh6F4ZMyYjUQu9JYofpp5+kSDJZCIl1Y2a6oKthHjOcjA5hC2x
yljLDM7eZstkYbJymmvNJg1PQK3xWvxM7hN9O83LHwKt4x/VVZl8RK2jJEo/
AuR/+ia4M/lDp92N6OBbsC+s9mgBkzdQJmUrHeKAdbUwkJe5X0h4GVUZkAC3
zNTXgph82f38eGViMjS6WoytMvkJWcmi6m0qtJgvqyUyz9/KN5lfVf8QEln1
/+gx9eIcyBOsnswEwUjJ2SUmP7Tfp+0akQUWbXHudUx99xkmnR8UyoYD9DTd
Aqb+9bvoD12znsLibx2/HGZ8WFjer7l+fQ5p2w02NjDjmTq+SXXIIheigp5f
YmHGf2+Lr4tbdx7seZStasRLgFuOepUb4zUUW53q52ben9V7ifNDUQWQ1i39
PZqfqb/uUkLhrTegpRG4RUyQOV/eoWqvaCG8PDrZGbiJABVxnj7vgUK4JsJ1
yEiYuZ7/83zh8yIg5RXLy4kQkLwoqqCm8R48NQZXpSQIqPnReUvtyCdgnzMx
X97OjGdgV42jUjF4TB6ae7CDWU+2d7NHrPsMdrG/LporEnAwuCestegL1FXa
N8/vIeDHzn5vv00YSFLGzk7M/L3wq78gYwpDsb3O3UJmfkchr3kXFwqM6uXH
ShgSsGPKSy1Wuwx61vgY6geZ/gy9X3dPlIOcR35ygD0BWqm0uhnbCuCbiDO5
/h/T/4z3xLqKCggSotZmOROQm+azY1d6JcycUNTbwaxHE1TfrBDzatDuuDZp
GML0/zggSfLVN7A5W+pU+ZQA3fmFIlWR71DsFSSik0cAn0Vgs1HEd8iVeZ9V
W8DMh4VA/ksn6mBraLacwhcCpq2Cbn7l+wkXxdxmGc0EfAr7+rVAoxFip31z
fnNS4XHuv5/FFxrB5yrfmaoNVIhp3d9b8aQRBj/JMPAmKhxXxMvtG5vAUa4q
dnEbFRZ/YS2W0SZ4NTQTGIaooL27rND6QQs0rrR6NFylwr7WtuSv31pA9Jm2
SXw0870JngratdICx9s6WM/co4L8ty2G3M6t8LHlq6BVJhVIpy+1YMVfsEJW
fchDocLUQ/E55a9tkPKuwFGPgwajhqodD4k22D+VIa63kQZ/JoxLeGTbofNV
ZbH1Fhp0anlHjES3A7X2S1zRThpUt9ZuenS4A55dsM49c4gGmXz++zYMd4Ks
Cyny5iMapBXFigZv6YLtO28p1LygQbJD1uqYWRdIXCHniRTR4PaLH1WV+V0w
VaRvulBDg8ADsrYhgd0gmMSYfkKjgU1Io/8kby94crCMFpnPApv/5h430QEI
feR3vYRvDhxO5TV/Uh8AkmFZe7TYHBQehG981gMQcTg1wkNhDly2nvvwLmoA
Vl5O//Y7MAfltSVx7HMDoGOn/PJp6ByEi7sZPa3/DctRrIEuc3PAXlaYPxg+
CDfMF5ZOztLhxEuTZ5qZg5Ci1LhtM/s8vEvqSY/9PAixfvuy/myaBzcPzptq
1EFgN3mWVaExD1UkB9fwk0NgOiD6NiN4HqLcWUVkycMgOGXLU7h+Abj4rK+7
TI/Ai7PSOY6aDDi+w4GisX4UQkp+vX58kAF5Bm5rPNtH4doBqtrKfwywuhIY
WmQ/CkJUhy/zEQxIHnl8hb1qFFxKGB8WWxggX0a9nJM+BqXiJ501/RfByD/B
ZcJ0AqxXG27E1y1B+ECboXf2X5gzt2ZQilehr9j8YmDJX5iyu7yD0bgK2gmU
5LBff0HbQX3MYGwVCKO88Vtc09Cq/1iKtOUfnHwZei/7/DRwZfA7PfL7B9r+
27tb1WbALdxC+ZTGGixykIvFXQj4F7013JT5n3+THmUdRqOCReK6+hYBNlRH
ihrlZ6FBb2nYkKA0GxqJjQzL2kADBl9JiOMeNiQREvGqUoEGDeKhewWt2FDM
iRvcvE40WFoWKeK8zYZOSYZRHtTRwLL6I7cYzzokkOmn8vbpLAzciOf4s5Ed
eT5xExiypUNp+nujBW1OdNIjTTL6OgPO1bMwbCt50M3D/mS1h8tQPlE8FNfM
j9g3JY1cXfoH9d/cTr36J4Ta6yKJCH5WVN1VcHu9qjA6FuyikVbBhh7dEQVv
VjF0gXdyW70DOwL1d284hyXR1pfVyZwTHMhrn6F8B6sMCv5keIQ/nQttt97j
9uucLPLKufBBY896tKC8VZ63WQ7xsWm6KA/zoF02n3FrmgIyung0STGBD6U7
1x7dE6GERIolPY22b0SZ7gqWSm7K6L53TLxKOz+ya/d+cLBdBdlEHdqd8UEA
3Rm9mHLSby9SdIw49jRUEL2yXt/aLqWO2kbWP0k4JoT8FG78OcamgepdhvWF
tpJQD+NoEVeXJvrN/pyRtI2E8nacp176o4nmcs4WiMqTkMli+ef2CU0kOjkp
IbObhEYjNGdeL2siNx/awh4dEmrRU++/IKmFlsLXXlvakZClRki3urMWkn8m
KnbzHglV6hsb/R3XQmST7ka+BBI68j1uMpKmhSzGH0bHJZPQW7qEpPSyFrq8
W5r+IINpT31Jxp6PjEreyzfmvCSh8qO1hb3KZGRTrRZVWUNCBuJFhIovGZ0+
S9c1rSOhwsYt5n9CyCiQ5+NsXT0JEQpfVZIiySjDStu55RfTXllDxloyGY21
Id0/gyR0IIlzbaSYjJYC2WZPj5KQjHNg0dNyMtogXpk7PkFCDQ1Ks651ZLTX
yWQLlUpC95PsNo32kJER6/p6XzpTn6kGet4wGR3P/h7BYJBQ/MhNF69pMvIw
uq0TssLcX3ikg+YCGf1fPwT9fz/kfwChpYaO
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-5, 5}, {0., 25.999997959183716`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"g", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwtWWc014//tcnI+MgqJSQjRRkZeb3tRJRZhDLaIStZLZEiyciohFIkGYUk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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-5, 5}, {-124.99998469387819`, 124.99998469387819`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Floor", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {Hue[0.67, 0.6, 0.6], 
     LineBox[{{-4.9999997959183675`, -5.}, {-4.901856591951532, -5.}, \
{-4.803713387984696, -5.}, {-4.590916429145413, -5.}, {-4.392221164298479, \
-5.}, {-4.294822338054859, -5.}, {-4.197423511811239, -5.}, \
{-4.144596460963027, -5.}, {-4.091769410114814, -5.}, {-4.0653558846907085`, \
-5.}, {-4.038942359266602, -5.}, {-4.025735596554549, -5.}, \
{-4.012528833842495, -5.}, {-4.009227143164482, -5.}, {-4.005925452486469, \
-5.}, {-4.003188775510204, -5.}}], 
     LineBox[{{-3.9968112244897958`, -4.}, {-3.9927186897744154`, -4.}, \
{-3.986115308418389, -4.}, {-3.8875120537181376`, -4.}, {-3.788908799017886, \
-4.}, {-3.575191738711772, -4.}, {-3.3653722907653534`, -4.}, \
{-3.2675134137883175`, -4.}, {-3.169654536811282, -4.}, \
{-3.1165974605963616`, -4.}, {-3.063540384381441, -4.}, {-3.037011846273981, \
-4.}, {-3.0104833081665205`, -4.}, {-3.007167240903088, -4.}, \
{-3.0038511736396556`, -4.}, {-3.0031887755102042`, -4.}}], 
     LineBox[CompressedData["
1:eJxTTMoPSmViYGAQAWIQ7XfwtGbFT/YDDGDAceBR2I7LO94g+Md1Nq5Vvo/g
T7qxKPHnYQQ/rmWK6PkVCH7ep9OJb++ywfl1CczruIURfAZd+f3OrSxw/q3u
me2rrzLB+b9PNUgE8CD4MlwZK784McL5Tn6b71+6ygDnZ076J7oxA8EvSp64
OMMXwa8xUTFUMELwGee+/V6tguC3sW7fd10MwT+xU3oRqwCCz5XX0GrMieBv
sjO51s6G4AMAxthbmg==
      "]], 
     LineBox[{{-1.996811224489796, -2.}, {-1.9711883622067754`, -2.}, \
{-1.9445448114159614`, -2.}, {-1.8399194144325521`, -2.}, \
{-1.735294017449143, -2.}, {-1.5401449174746724`, -2.}, \
{-1.3284852665945908`, -2.}, {-1.229706288150724, -2.}, {-1.130927309706857, \
-2.}, {-1.0774101827585207`, -2.}, {-1.0238930558101846`, -2.}, \
{-1.0172034149416427`, -2.}, {-1.0105137740731007`, -2.}, \
{-1.0071689536388297`, -2.}, {-1.0038241332045588`, -2.}, \
{-1.003188775510204, -2.}}], 
     LineBox[{{-0.9968112244897959, -1.}, {-0.9837552105989325, -1.}, \
{-0.9703759288618484, -1.}, {-0.9436173653876803, -1.}, {-0.9168588019135122, \
-1.}, {-0.8117733541966873, -1.}, {-0.7066879064798623, -1.}, \
{-0.5106187050385604, -1.}, {-0.2980389526916474, -1.}, \
{-0.19879992351436487`, -1.}, {-0.09956089433708235, -1.}, \
{-0.05091578283836483, -1.}, {-0.003188775510204082, -1.}}], 
     LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ3X7s14UI+Sx7BijQm5V8XMQsH86/J7nxHTd/DZwf
bHxjT5tjC5y/7WZ4U3t1F5zPe+C18pGoCXB+YVr2yyNTpsH5iyy1fY5unAnn
X+F5ve7ouTlw/pxIU6vkloVwPgPD1v/J+5bD+TNfihg+8VkJ5xtdLklKub0K
zk9ZZnI05cdaBF8t6qiR5CY4//iZm13GwTvg/IyKW8ejtXfC+Rwqt1lamHfB
+V7Vd+qubN4N55/RvF9UKrIfod+5aO3tV4fg/JwF5VNl15yE8610D24KmH4Z
zmcqE72TIvkAzmfnCWhMevcMzo9Qjzpgxvkczl/llPKfSwXB96usqN0cieBP
e7agkuUogq928GPhstkv4HzXsslJrzxeIdRPnLtwXzKC/2LN8geT6hD8nse7
46y3IPhXAp5E9ci/RoSntmmw3jcEv/nBNeeixW8R6WWnd27FHgTfavKBaXVX
EfwPrqtedrG/g/PjVtf2L85G8He1fNqxsgXBF41Lf7R+LoJ/RiDQdM95BN+q
TOX2FeP3CP/5z2S57YPgf9Tg03uYiuCvuP294e10BJ9la86qzxsQ/Pi+h5d/
nkTwd6WH/f3/CMG3ZNVf/uApgg8AyS7YZg==
      "]], 
     LineBox[{{1.003188775510204, 1.}, {1.0056240441405122`, 1.}, {
      1.0086787401946014`, 1.}, {1.0117334362486905`, 1.}, {
      1.0178428283568688`, 1.}, {1.0239522204650469`, 1.}, {1.072827357330472,
       1.}, {1.1217024941958975`, 1.}, {1.227708043379554, 1.}, {
      1.3337135925632102`, 1.}, {1.5316229969381747`, 1.}, {
      1.7256347889534445`, 1.}, {1.8308959604138848`, 1.}, {
      1.9361571318743254`, 1.}, {1.9607097129903919`, 1.}, {
      1.9852622941064584`, 1.}, {1.9914004393854752`, 1.}, {1.996811224489796,
       1.}}], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQvcnO5Fo7G4MDAwQ4LH4RtqqdHcFfddV73wVeBH/T
IYdLkiII/vkyESU7XQT/jdaLwiQ3BD/pt9nGz6sQ/OunWz+08DPC+dOUSyb3
v0TwQ6uTzGYdZoLzc7hfK52LYoHz5Vcfm8b2ihXOP2vI+rXrEBucX73DOZh/
NjucH+VedjJwC4JvdW2F/ZSzCP4q2UNVprcQfMnU21uvPUPwL3BkBKu/RfA7
1nz5UP4Zwfc7eFqz4ieCDwAVN036
      "]], 
     LineBox[{{3.0031887755102042`, 3.}, {3.024715605746873, 3.}, {
      3.049383199546294, 3.}, {3.098718387145135, 3.}, {3.148053574743976, 
      3.}, {3.2549792253944636`, 3.}, {3.3619048760449513`, 3.}, {
      3.5718585649862313`, 3.}, {3.767710559935164, 3.}, {3.873891832862436, 
      3.}, {3.9800731057897076`, 3.}, {3.986268757410401, 3.}, {
      3.992464409031095, 3.}, {3.995562234841442, 3.}, {3.9968112244897958`, 
      3.}}], LineBox[{{4.003188775510204, 4.}, {4.004855712272482, 4.}, {
      4.0172470155138695`, 4.}, {4.029638318755257, 4.}, {4.054420925238031, 
      4.}, {4.079203531720806, 4.}, {4.128768744686354, 4.}, {
      4.178333957651903, 4.}, {4.275515577403153, 4.}, {4.3726971971544035`, 
      4.}, {4.583570987562515, 4.}, {4.780343083978279, 4.}, {
      4.890171439948324, 4.}, {4.9999997959183675`, 4.}}]}, {}}, {{}, {}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-5, 5}, {-5., 4.}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Indexed Variables", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "For finite domains, an ",
 ButtonBox["indexed variable",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/MakingDefinitionsForIndexedObjects"],
 " can be used to represent a function. For example, suppose ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " is the function whose domain is the set of students in a class and that \
maps each student to their grade on an exam. Let ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " be defined by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "Ann", ")"}], "=", "83"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "Bob", ")"}], "=", "79"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "Carla", ")"}], "=", "91"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "Dave", ")"}], "=", "72"}], TraditionalForm]]],
 ". We model ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " as an indexed variable named ",
 StyleBox["exams", "Input"],
 " by issuing assignments of the form ",
 Cell[BoxData[
  RowBox[{"exams", "[", 
   StyleBox["name", "CodeVar"], "]", "=", 
   StyleBox["score", "CodeVar"]}]], "myCode"],
 ". Below, we give all four assignments in a single input, using semicolons \
to suppress the output, which would just be an echo of the assigned values."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"exams", "[", "\"\<Ann\>\"", "]"}], "=", "83"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"exams", "[", "\"\<Bob\>\"", "]"}], "=", "79"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"exams", "[", "\"\<Carla\>\"", "]"}], "=", "91"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"exams", "[", "\"\<Dave\>\"", "]"}], "=", "72"}], ";"}]}], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "Once the indexed variable has been assigned, you can obtain the value ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "Carla", ")"}], TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exams", "[", "\"\<Carla\>\"", "]"}]], "Input"],

Cell[BoxData["91"], "Output"]
}, Open  ]],

Cell["\<\
You can also modify values just by entering the appropriate assignment, and \
you can always add new values.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"exams", "[", "\"\<Ann\>\"", "]"}], "=", "84"}]], "Input",
 InitializationCell->True],

Cell[BoxData["84"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"exams", "[", "\"\<Ernie\>\"", "]"}], "=", "86"}]], "Input",
 InitializationCell->True],

Cell[BoxData["86"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To see the entire definition of the indexed variable, you can use the ",
 StyleBox[ButtonBox["Definition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Definition"], "FunctionLink"],
 " function or the ",
 StyleBox["?", "Input"],
 " operator, as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "exams"}]], "Input"],

Cell["Global`exams", "Print", "PrintUsage",
 CellTags->"Info3580371076-5514480"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"exams", "[", "\<\"Ann\"\>", "]"}], "=", "84"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"exams", "[", "\<\"Bob\"\>", "]"}], "=", "79"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"exams", "[", "\<\"Carla\"\>", "]"}], "=", "91"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"exams", "[", "\<\"Dave\"\>", "]"}], "=", "72"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"exams", "[", "\<\"Ernie\"\>", "]"}], "=", "86"}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`exams],
  Editable->False]], "Print",
 CellTags->"Info3580371076-5514480"]
}, Open  ]],

Cell[TextData[{
 "While this is informative, the ",
 StyleBox["?", "Input"],
 " operator does not produce a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expression that you can subsequently manipulate. To obtain an expression \
that we can work with programmatically, we use the ",
 StyleBox[ButtonBox["DownValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DownValues"], "FunctionLink"],
 " function. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", when you make an assignment of the form \[OpenCurlyDoubleQuote]symbol = \
value\[CloseCurlyDoubleQuote] (with = or :=), it is said that you are setting \
an \[OpenCurlyDoubleQuote]ownvalue\[CloseCurlyDoubleQuote] for the symbol, \
that is, a value of the symbol itself. You assign a \
\[OpenCurlyDoubleQuote]downvalue\[CloseCurlyDoubleQuote] when you make an \
assignment using a combination of a symbol and an argument, as we did with ",
 StyleBox["exams", "Input"],
 " above, or in the definition of a function as in:"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", 
  RowBox[{"x", "^", "2"}]}]], "myCode",
 Evaluatable->False,
 InitializationCell->False],

Cell["\<\
These are called \[OpenCurlyDoubleQuote]downvalues\[CloseCurlyDoubleQuote] \
because they are not associated with the symbol itself, but rather are \
attached to expressions obtained by combining the symbol with arguments. More \
precisely, you must look down the expression tree to determine the value (see \
Section 11.3 of the textbook for information about representing expressions \
as trees). There are also \[OpenCurlyDoubleQuote]upvalues,\
\[CloseCurlyDoubleQuote] but these will not be discussed here.\
\>", "Text"],

Cell[TextData[{
 "Applying the function ",
 StyleBox[ButtonBox["DownValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DownValues"], "FunctionLink"],
 " to a symbol produces a list of all of the downvalues associated to the \
symbol."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "exams", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"exams", "[", "\<\"Ann\"\>", "]"}], "]"}], "\[RuleDelayed]", 
    "84"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"exams", "[", "\<\"Bob\"\>", "]"}], "]"}], "\[RuleDelayed]", 
    "79"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"exams", "[", "\<\"Carla\"\>", "]"}], "]"}], "\[RuleDelayed]", 
    "91"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"exams", "[", "\<\"Dave\"\>", "]"}], "]"}], "\[RuleDelayed]", 
    "72"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"exams", "[", "\<\"Ernie\"\>", "]"}], "]"}], "\[RuleDelayed]", 
    "86"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result is a list of transformation rules. Note that ",
 StyleBox[ButtonBox["HoldPattern",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldPattern"], "FunctionLink"],
 " is used to prevent the left hand side of each rule from being evaluated to \
the associated value. "
}], "Text"],

Cell[TextData[{
 "We can access individual rules within the list as usual, using ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DownValues", "[", "exams", "]"}], "[", 
  RowBox[{"[", "2", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"HoldPattern", "[", 
   RowBox[{"exams", "[", "\<\"Bob\"\>", "]"}], "]"}], "\[RuleDelayed]", 
  "79"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This expression, in turn has two parts, corresponding to the left hand side \
and right hand side of the ",
 StyleBox[ButtonBox["RuleDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RuleDelayed"], "FunctionLink"],
 " (\[RuleDelayed]) operator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DownValues", "[", "exams", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"HoldPattern", "[", 
  RowBox[{"exams", "[", "\<\"Bob\"\>", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DownValues", "[", "exams", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData["79"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can further access the index, in this case \[OpenCurlyDoubleQuote]Bob\
\[CloseCurlyDoubleQuote], as the first part of the first part of the ",
 StyleBox[ButtonBox["HoldPattern",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldPattern"], "FunctionLink"],
 " expression (since \[OpenCurlyDoubleQuote]Bob\[CloseCurlyDoubleQuote] is \
the argument of the argument of ",
 StyleBox[ButtonBox["HoldPattern",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldPattern"], "FunctionLink"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DownValues", "[", "exams", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "1", ",", "1", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData["\<\"Bob\"\>"], "Output"]
}, Open  ]],

Cell["\<\
This gives us all of the tools we need in order to transform the indexed \
variable into a list of pairs, i.e., the graph of the function (see \
Definition 11 of Section 2.3 in the textbook).\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"graph", "[", "f_Symbol", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "dv"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dv", "=", 
      RowBox[{"DownValues", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"dv", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
         RowBox[{"dv", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "dv", "]"}]}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"graph", "[", "exams", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"Ann\"\>", ",", "84"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Bob\"\>", ",", "79"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Carla\"\>", ",", "91"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Dave\"\>", ",", "72"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Ernie\"\>", ",", "86"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["graph", "Input"],
 " function uses ",
 StyleBox[ButtonBox["DownValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DownValues"], "FunctionLink"],
 " to obtain the definition of the indexed variable (which has head ",
 StyleBox[ButtonBox["Symbol",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Symbol"], "FunctionLink"],
 ") and stores the list of rules as ",
 StyleBox["dv", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function produces the list obtained by evaluating its first argument with \
the table variable varied according to the specification in the second \
argument. In this case, the first argument consists of the two-element list \
obtained by accessing the index and the value from the table of downvalues, \
",
 StyleBox["dv", "Input"],
 ". The second argument specifies that the variable ",
 StyleBox["i", "Input"],
 " should range from 1 up to the ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " of the table of downvalues."
}], "Text"],

Cell[CellGroupData[{

Cell["Domain and Range", "Subsubsection"],

Cell[TextData[{
 "Since indexed variables represent finite functions, we can write functions \
to check various properties. First, we\[CloseCurlyQuote]ll find the domain \
(technically, the domain of definition) and range of a function defined as an \
indexed variable. In the ",
 StyleBox["exams", "Input"],
 " function, the students\[CloseCurlyQuote] names (Ann, Bob, etc.) form the \
domain (also called the ",
 StyleBox["indices",
  FontSlant->"Italic"],
 ") and the scores (84, 79, etc.) are the range (or ",
 StyleBox["values",
  FontSlant->"Italic"],
 ")."
}], "Text"],

Cell[TextData[{
 "To define functions for obtaining the domain and range, we merely need to \
replicate the ",
 StyleBox["graph", "Input"],
 " function, but include only the indices or the values, rather than the pair \
of both, in the table. We also apply ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to the resulting list so as to remove duplicates and order the elements."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"domain", "[", "f_Symbol", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"i", ",", "dv"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dv", "=", 
       RowBox[{"DownValues", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Union", "[", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"dv", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", "dv", "]"}]}], "}"}]}], "]"}], 
       "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"range", "[", "f_Symbol", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"i", ",", "dv"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dv", "=", 
       RowBox[{"DownValues", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Union", "[", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"dv", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", "dv", "]"}]}], "}"}]}], "]"}], 
       "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Using these functions, we can easily find the domain and range of ",
 StyleBox["exams", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"domain", "[", "exams", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Ann\"\>", ",", "\<\"Bob\"\>", ",", "\<\"Carla\"\>", 
   ",", "\<\"Dave\"\>", ",", "\<\"Ernie\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"range", "[", "exams", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"72", ",", "79", ",", "84", ",", "86", ",", "91"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Injective and Surjective", "Subsubsection"],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s create a few more examples. Then we will write \
functions to check for injectivity and surjectivity. The examples below \
correspond to the functions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "1"], "(", "x", ")"}], "=", 
    SuperscriptBox["x", "2"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "2"], "(", "x", ")"}], "=", 
    SuperscriptBox["x", "3"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "3"], "(", "x", ")"}], "=", 
    RowBox[{"|", "x", "|"}]}], TraditionalForm]]],
 " on the domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"D", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "5"}], ",", 
      RowBox[{"-", "4"}], ",", "\[Ellipsis]", ",", "5"}], "}"}]}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f1", "[", "x", "]"}], "=", 
    RowBox[{"x", "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f2", "[", "x", "]"}], "=", 
    RowBox[{"x", "^", "3"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f3", "[", "x", "]"}], "=", 
    RowBox[{"Abs", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Observe how we used the ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function to automate the assignments of values. Since the ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function executes the statement given as the first argument after \
substituting the values specified by the second argument into the variable, \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " never evaluates the expression ",
 StyleBox["f1[x]=x^2", "Input"],
 ", for example. It only evaluates the assignments after having replaced the \
variable with the numerical values, so ",
 StyleBox["f1[3]", "Input"],
 " holds a value, but ",
 StyleBox["f1[x]", "Input"],
 " is not assigned."
}], "Text"],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s confirm that ",
 StyleBox["f1", "Input"],
 " was assigned correctly by checking it using ",
 StyleBox[ButtonBox["DownValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DownValues"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "f1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", 
      RowBox[{"-", "5"}], "]"}], "]"}], "\[RuleDelayed]", "25"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", 
      RowBox[{"-", "4"}], "]"}], "]"}], "\[RuleDelayed]", "16"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", 
      RowBox[{"-", "3"}], "]"}], "]"}], "\[RuleDelayed]", "9"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", 
      RowBox[{"-", "2"}], "]"}], "]"}], "\[RuleDelayed]", "4"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", 
      RowBox[{"-", "1"}], "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "0", "]"}], "]"}], "\[RuleDelayed]", "0"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "1", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "2", "]"}], "]"}], "\[RuleDelayed]", "4"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "3", "]"}], "]"}], "\[RuleDelayed]", "9"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "4", "]"}], "]"}], "\[RuleDelayed]", "16"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f1", "[", "5", "]"}], "]"}], "\[RuleDelayed]", "25"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, the ",
 StyleBox["domain", "Input"],
 " and ",
 StyleBox["range", "Input"],
 " functions produce the expected results."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"domain", "[", "f1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "5"}], ",", 
   RowBox[{"-", "4"}], ",", 
   RowBox[{"-", "3"}], ",", 
   RowBox[{"-", "2"}], ",", 
   RowBox[{"-", "1"}], ",", "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", 
   "5"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"range", "[", "f1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "4", ",", "9", ",", "16", ",", "25"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We can check to see if a function represented by an indexed variable is \
surjective for a specified codomain by comparing the codomain to the range.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"surjectiveQ", "[", 
   RowBox[{"f_Symbol", ",", "codomain_List"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"range", "[", "f", "]"}], "==", 
   RowBox[{"Union", "[", "codomain", "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we applied ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to the second argument to provide assurance that both sets being compared \
are without duplicates and in standard order (recall that ",
 StyleBox["range", "Input"],
 " applies ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " before ending)."
}], "Text"],

Cell[TextData[{
 "As expected, ",
 StyleBox["f1", "Input"],
 " is not onto ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 ", but ",
 StyleBox["f3", "Input"],
 " is. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"surjectiveQ", "[", 
  RowBox[{"f1", ",", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "5"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"surjectiveQ", "[", 
  RowBox[{"f3", ",", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "5"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can check for injectivity by making sure that no entry value is \
repeated. The easiest way to do this is to check that the number of values in \
the result of ",
 StyleBox["range", "Input"],
 " is the same as the number in the domain returned by ",
 StyleBox["domain", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"injectiveQ", "[", "f_Symbol", "]"}], ":=", 
  RowBox[{
   RowBox[{"Length", "[", 
    RowBox[{"range", "[", "f", "]"}], "]"}], "\[Equal]", 
   RowBox[{"Length", "[", 
    RowBox[{"domain", "[", "f", "]"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "This function will confirm that ",
 StyleBox["f1", "Input"],
 " is not injective but that ",
 StyleBox["f2", "Input"],
 " is."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"injectiveQ", "[", "f1", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"injectiveQ", "[", "f2", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphing a Function Defined as an Indexed Variable", "Subsubsection"],

Cell[TextData[{
 "Finally, let\[CloseCurlyQuote]s see how we can graph a function defined by \
an indexed variable. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " makes this quite easy with the use of ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 ". Given a list of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " pairs (represented as 2-element lists), ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " will display a graph of the function. Conveniently, the ",
 StyleBox["graph", "Input"],
 " function we created above produces just such a list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{"graph", "[", "f1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, 
   {RGBColor[0.24720000000000014`, 0.24, 0.6], 
    PointBox[{{-5., 25.}, {-4., 16.}, {-3., 9.}, {-2., 4.}, {-1., 1.}, {0., 
     0.}, {1., 1.}, {2., 4.}, {3., 9.}, {4., 16.}, {5., 25.}}]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0., 0},
  Method->{},
  PlotRange->{{-5., 5.}, {0, 25.}},
  PlotRangeClipping->True,
  PlotRangePadding->{{0.2, 0.2}, {0.5, 0.5}}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Two common options used in conjunction with ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " are ",
 StyleBox[ButtonBox["PlotRange",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotRange"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Joined",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Joined"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["PlotRange",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotRange"], "FunctionLink"],
 " can be used to explicitly choose the span in both the x and y directions. \
To use the ",
 StyleBox[ButtonBox["PlotRange",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotRange"], "FunctionLink"],
 " option, give ",
 Cell[BoxData[
  RowBox[{"PlotRange", "->", 
   RowBox[{"{", "{"}], 
   StyleBox["xmin", "CodeVar"], 
   StyleBox["xmax", "CodeVar"], "}", ",", "{", 
   StyleBox["ymin", "CodeVar"], ",", 
   StyleBox["ymax", "CodeVar"], 
   RowBox[{"}", "}"}]}]], "myCode"],
 " as an argument to ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 ". You can substitute ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " for either of the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " ranges to include all the data. "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Joined",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Joined"], "FunctionLink"],
 " option causes ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to \[OpenCurlyDoubleQuote]connect the dots\[CloseCurlyDoubleQuote]. You \
invoke it by including ",
 Cell[BoxData[
  RowBox[{"Joined", "->", "True"}]], "myCode"],
 " in the call to ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"graph", "[", "f2", "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "5"}], "}"}], ",", "All"}], "}"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {RGBColor[0.24720000000000014`, 0.24, 0.6], 
     LineBox[{{-5., -125.}, {-4., -64.}, {-3., -27.}, {-2., -8.}, {-1., -1.}, \
{0., 0.}, {1., 1.}, {2., 8.}, {3., 27.}, {4., 64.}, {5., 125.}}]}}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 5}, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["PlotStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotStyle"], "FunctionLink"],
 " option can be used to set a variety of visual aspects of the plot \
including color (e.g., ",
 StyleBox[ButtonBox["Red",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Red"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Blue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blue"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Green",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Green"], "FunctionLink"],
 ", etc.) and point size (",
 StyleBox[ButtonBox["PointSize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PointSize"], "FunctionLink"],
 "), as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"graph", "[", "f3", "]"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"PointSize", "[", ".02", "]"}]}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, 
   {RGBColor[1, 0, 0], PointSize[0.02], 
    PointBox[{{-5., 5.}, {-4., 4.}, {-3., 3.}, {-2., 2.}, {-1., 1.}, {0., 
     0.}, {1., 1.}, {2., 2.}, {3., 3.}, {4., 4.}, {5., 5.}}]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0., 0},
  Method->{},
  PlotRange->{{-5., 5.}, {0, 5.}},
  PlotRangeClipping->True,
  PlotRangePadding->{{0.2, 0.2}, {0.1, 0.1}}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Some Important Functions", "Subsection",
 CellTags->"Section3d"],

Cell[TextData[{
 "We\[CloseCurlyQuote]ve already seen that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a built-in function ",
 StyleBox[ButtonBox["Floor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Floor"], "FunctionLink"],
 ". It also includes ",
 StyleBox[ButtonBox["Ceiling",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Ceiling"], "FunctionLink"],
 " for computing the ceiling of a real number."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Floor", "[", "2.7", "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ceiling", "[", "2.7", "]"}]], "Input"],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " contains some additional related functions. The ",
 StyleBox[ButtonBox["Round",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Round"], "FunctionLink"],
 " function rounds a number to the nearest integer. The ",
 StyleBox[ButtonBox["IntegerPart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPart"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["FractionalPart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FractionalPart"], "FunctionLink"],
 " functions, as their names imply, compute the integral or fractional part \
of a real number."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Round", "[", "2.7", "]"}]], "Input"],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPart", "[", "2.7", "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPart", "[", 
  RowBox[{"-", "2.7"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FractionalPart", "[", "2.7", "]"}]], "Input"],

Cell[BoxData["0.7000000000000002`"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The textbook also discussed the factorial function. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you compute the factorial of a number using the ",
 StyleBox[ButtonBox["Factorial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Factorial"], "FunctionLink"],
 " (",
 StyleBox["!", "Input"],
 ") function, typically with the operator ",
 StyleBox["!", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"6", "!"}]], "Input"],

Cell[BoxData["720"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factorial", "[", "6", "]"}]], "Input"],

Cell[BoxData["720"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.4 Sequences and Summations", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to create and manipulate sequences, and in particular, we will \
see a way to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate the terms of a recurrence sequence. We will also look at \
summations and see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s symbolic computation abilities can be used to explore \
both finite and infinite series."
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you represent a finite sequence as a list. The \
\[OpenCurlyDoubleQuote]empty list\[CloseCurlyDoubleQuote] is represented by \
the expression ",
 StyleBox["{}", "Input"],
 ". "
}], "Text"],

Cell[TextData[{
 "There are a wide variety of ways that lists can be created in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Here we will describe three of the most important: the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function, ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " and related functions, and the ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " functions."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Building Lists: the ",
 StyleBox["Table", "Input"],
 " Function"
}], "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "We have already seen several examples of the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function. We briefly summarize some of the ways it can be called."
}], "Text"],

Cell[TextData[{
 "The most common way to use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " is demonstrated in the following example, which creates the geometric \
sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", ",", " ", 
    RowBox[{"a", "\[CenterDot]", "r"}], ",", " ", 
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["r", "2"]}], ",", "\[Ellipsis]"}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "3"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "=", "4"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"3", "*", 
    RowBox[{"4", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
   "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first argument is an expression which may involve an index variable, in \
this case ",
 StyleBox["i", "Input"],
 ". The second argument is a list whose elements are the variable and the \
minimum and maximum values for the index variable. "
}], "Text"],

Cell[TextData[{
 "The minimum value can be omitted, in which case it will default to 1. The \
following shows the sequence beginning with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "=", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"3", "*", 
    RowBox[{"4", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", "12288", ",", 
   "49152", ",", "196608", ",", "786432", ",", "3145728"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
By including three numeric arguments, in addition to the name of the \
variable, you can control the step, i.e., the amount by which the index \
variable is incremented. For example, the expression below will produce every \
other term of the same geometric sequence as above.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"3", "*", 
    RowBox[{"4", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "10", ",", "2"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The bounds of the range for the index variable and the step do not \
necessarily need to be integers. For example, the following produces the list \
of numbers beginning with ",
 StyleBox["2.3", "Input"],
 ", increasing by ",
 StyleBox["0.25", "Input"],
 ", up to an upper bound of ",
 StyleBox["5.1", "Input"],
 ". Note that the maximum is not included."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{"i", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "2.3", ",", "5.1", ",", ".25"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2.3`", ",", "2.55`", ",", "2.8`", ",", "3.05`", ",", "3.3`", ",", "3.55`", 
   ",", "3.8`", ",", "4.05`", ",", "4.3`", ",", "4.55`", ",", "4.8`", ",", 
   "5.05`"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " function can be thought of as an abbreviation of ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " for situations like the previous example, where you simply want to produce \
the list of numbers without evaluating an expression. It accepts one, two, or \
three numerical arguments, with the same interpretation as the numerical \
elements of the second argument of ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", "10", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8", 
   ",", "9", ",", "10"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", 
  RowBox[{"0", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", 
   ",", "8", ",", "9", ",", "10"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Range", "[", 
  RowBox[{"0", ",", "10", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "2", ",", "4", ",", "6", ",", "8", ",", "10"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " can also be used to evaluate an expression for a specific list of values. \
This is illustrated in the example below, which finds the squares of the \
first six primes. Note that the second argument in this formulation is a list \
with two elements: the first is the name of the index variable and the second \
is the list of values to be substituted."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13"}], 
      "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "9", ",", "25", ",", "49", ",", "121", ",", "169"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The order of the provided list determines the order of the output. For \
example, if we rearrange the primes, the output is affected accordingly.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "5", ",", "7", ",", "13", ",", "11", ",", "3"}], 
      "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "25", ",", "49", ",", "169", ",", "121", ",", "9"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Apply ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " to the result if you want to think about the output as a set or to impose \
numerical order on the output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"i", "^", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "5", ",", "7", ",", "13", ",", "11", ",", "3"}], 
       "}"}]}], "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "9", ",", "25", ",", "49", ",", "121", ",", "169"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " can handle more than one variable just by providing an additional argument \
specifying the values for each variable. For example, the following finds the \
values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "i"], 
    SuperscriptBox["3", "j"]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " from 2 to 12 by 3 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "\[Element]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], TraditionalForm]]],
 ".",
 Cell[BoxData[
  FormBox[Cell[""], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", "i"}], "*", 
    RowBox[{"3", "^", "j"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "2", ",", "12", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"12", ",", "36", ",", "108"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"96", ",", "288", ",", "864"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"768", ",", "2304", ",", "6912"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6144", ",", "18432", ",", "55296"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the result is a list of lists, with the sublists composed of the \
values computed from a single value of the first variable. That is, the first \
sublist is the output with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "=", "2"}], TraditionalForm]]],
 " and all values of ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", the second sublist is the output with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "=", "5"}], TraditionalForm]]],
 " (the second value of ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ") and all the values of ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", etc."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Building Lists: the ",
 StyleBox["Append", "Input"],
 " Function"
}], "Subsection",
 CellTags->"Section4b"],

Cell[TextData[{
 "To add an element to the end of a list, use the ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " function requires two arguments: the first is a list to be modified and \
the second is a single object to be added to the end of the list. For \
example, to add ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 " to the end of the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 ", apply ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Append", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " function can also accept a symbol storing a list as its first argument. \
However, it will not modify the stored list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleList", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Append", "[", 
  RowBox[{"exampleList", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["exampleList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In order to have the new list stored in the name, you either need to \
reassign the result of ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " back to the name, or you can use ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " which automatically updates the list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleList", "=", 
  RowBox[{"Append", "[", 
   RowBox[{"exampleList", ",", "4"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["exampleList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AppendTo", "[", 
  RowBox[{"exampleList", ",", "5"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["exampleList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " requires that the first argument is a symbol. If you try to call it with \
an explicit list as the first argument, it will raise an error."
}], "Text"],

Cell[TextData[{
 "Related to ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " are ",
 StyleBox[ButtonBox["Prepend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Prepend"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["PrependTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrependTo"], "FunctionLink"],
 ", which have the same syntax but add the element to the front of the list \
rather than the end."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrependTo", "[", 
  RowBox[{"exampleList", ",", "6"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We\[CloseCurlyQuote]ve seen earlier in this manual that you can modify \
elements of a list by using the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " operator (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") and assigning a new value. The following changes the third element of our \
list to 7."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"exampleList", "[", 
   RowBox[{"[", "3", "]"}], "]"}], "=", "7"}]], "Input",
 InitializationCell->True],

Cell[BoxData["7"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["exampleList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "1", ",", "7", ",", "3", ",", "4", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplacePart"], "FunctionLink"],
 " function can also be used to modify an element of a list, but is more \
general. The most basic syntax of ",
 StyleBox[ButtonBox["ReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplacePart"], "FunctionLink"],
 " involves a list and a rule of the form ",
 Cell[BoxData[
  RowBox[{
   StyleBox["part", "CodeVar"], "->", 
   StyleBox["new", "CodeVar"]}]], "myCode"],
 ". The output is the list with the entry at position ",
 StyleBox["part", "CodeVar"],
 " replaced by the expression ",
 StyleBox["new", "CodeVar"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReplacePart", "[", 
  RowBox[{"exampleList", ",", 
   RowBox[{"4", "\[Rule]", "11"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "1", ",", "7", ",", "11", ",", "4", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that this does not modify the list stored in ",
 StyleBox["exampleList", "Input"],
 " without explicit reassignment. There are a variety of other syntax options \
for the second argument of ",
 StyleBox[ButtonBox["ReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplacePart"], "FunctionLink"],
 " to produce more general effects. These will be discussed only when they \
are needed."
}], "Text"],

Cell[TextData[{
 "If you wish to add an element in the middle of the list, rather than \
overwriting an element, you use the ",
 StyleBox[ButtonBox["Insert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Insert"], "FunctionLink"],
 " function with the list, the element to be added, and the index at which it \
is to appear. The other members of the list are shifted to accommodate the \
new one. Note that ",
 StyleBox[ButtonBox["Insert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Insert"], "FunctionLink"],
 " does not modify a list stored as a symbol without explicitly reassigning \
it."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleList", "=", 
  RowBox[{"Insert", "[", 
   RowBox[{"exampleList", ",", "8", ",", "4"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "1", ",", "7", ",", "8", ",", "3", ",", "4", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Similarly, you can remove an element by calling ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " with the list and the position of the element to be removed. Like ",
 StyleBox[ButtonBox["Insert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Insert"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " does not automatically modify a variable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleList", "=", 
  RowBox[{"Delete", "[", 
   RowBox[{"exampleList", ",", "2"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "7", ",", "8", ",", "3", ",", "4", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Building Lists: ",
 StyleBox["Sow", "Input"],
 " and ",
 StyleBox["Reap", "Input"]
}], "Subsection",
 CellTags->"Section4c"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " (or ",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 ") function can be used within a function to build a sequence or other list. \
For example, the function defined below builds the first ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " terms of the geometric sequence with parameters ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"geometricSequence", "[", 
   RowBox[{"a_", ",", "r_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"S", "=", 
       RowBox[{"{", "}"}]}], ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"S", ",", 
         RowBox[{"a", "*", 
          RowBox[{"r", "^", "i"}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "S"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"geometricSequence", "[", 
  RowBox[{"3", ",", "4", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
   "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, the pair of functions ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " tend to be a much more efficient approach to this task. The following \
recreates the function using this method."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"geometricSequence2", "[", 
   RowBox[{"a_", ",", "r_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "i", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Reap", "[", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"Sow", "[", 
        RowBox[{"a", "*", 
         RowBox[{"r", "^", "i"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"geometricSequence2", "[", 
  RowBox[{"3", ",", "4", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
     "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
      "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
     "}"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s analyze the function above. The ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " function accepts any expression (or multiple expressions separated by \
semicolons) as its argument. Its result is a two-member list, the first \
element of which is the output obtained by evaluating the expression. In the \
above, ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " always outputs ",
 StyleBox["Null", "Input"],
 ", so that is the first element of the output. "
}], "Text"],

Cell[TextData[{
 "The second element of the output of ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " is a list of lists, whose members are determined by the use of ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " within the ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 ". In the simplest form, as above, the second element of the output is a \
list which contains a single list comprised of all of the results of \
evaluating ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 ". That is, each time ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " is encountered, its argument is evaluated and added to this list. We can \
modify ",
 StyleBox["geometricSequence2", "Input"],
 " to produce the same output as ",
 StyleBox["geometricSequence", "Input"],
 " just by accessing the ",
 StyleBox["[[2,1]]", "Input"],
 " position of the result of ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"geometricSequence2", "[", 
   RowBox[{"a_", ",", "r_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "i", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"Do", "[", 
       RowBox[{
        RowBox[{"Sow", "[", 
         RowBox[{"a", "*", 
          RowBox[{"r", "^", "i"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"geometricSequence2", "[", 
  RowBox[{"3", ",", "4", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
   "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "While the ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " approach may seem more complicated than using ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 ", the resulting function is significantly more efficient, as the following \
illustrates. The ",
 StyleBox[ButtonBox["Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Timing"], "FunctionLink"],
 " function returns the list whose first element is the time it took to \
execute the expression and the second argument is the output. The semicolons \
suppress the output of the functions so we can focus on the time."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"geometricSequence", "[", 
    RowBox[{"3", ",", "4", ",", "10000"}], "]"}], ";"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.3265819999999999834194852610380621627`5.534592157623627", ",", "Null"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"geometricSequence2", "[", 
    RowBox[{"3", ",", "4", ",", "10000"}], "]"}], ";"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.02782799999999999857047683349264843855`4.465081907965236", ",", "Null"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second element of the output of ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " is always a list of lists. This is to allow for more fine-tuned creation \
of lists through the use of optional tags. For example, you could separate \
the terms in our geometric sequence based on whether their index is even or \
odd as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", "\[IndentingNewLine]", 
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"EvenQ", "[", "i", "]"}], ",", 
      RowBox[{"Sow", "[", 
       RowBox[{
        RowBox[{"3", "*", 
         RowBox[{"4", "^", "i"}]}], ",", "even"}], "]"}], ",", 
      RowBox[{"Sow", "[", 
       RowBox[{
        RowBox[{"3", "*", 
         RowBox[{"4", "^", "i"}]}], ",", "odd"}], "]"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
       "3145728"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"12", ",", "192", ",", "3072", ",", "49152", ",", "786432"}], 
      "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result of ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " is still a two-element list with first element ",
 StyleBox["Null", "Input"],
 " (the output of ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 "). However, the second element of ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 "\[CloseCurlyQuote]s output is now a list with two sublists: one consisting \
of the elements ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 "\[CloseCurlyQuote]n with tag ",
 StyleBox["even", "Input"],
 " and one list for those with tag ",
 StyleBox["odd", "Input"],
 ". "
}], "Text"],

Cell[TextData[{
 "The second argument of ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " can, in fact, be a list of tags, in which case elements can appear in more \
than one sublist. For example, the following produces three lists (as \
sublists in the second element of the output): the even-indexed terms, the \
odd-indexed terms, and all of the terms."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", "\[IndentingNewLine]", 
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"EvenQ", "[", "i", "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Sow", "[", 
       RowBox[{
        RowBox[{"3", "*", 
         RowBox[{"4", "^", "i"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"all", ",", "even"}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Sow", "[", 
       RowBox[{
        RowBox[{"3", "*", 
         RowBox[{"4", "^", "i"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"all", ",", "odd"}], "}"}]}], "]"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
       "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
       "3145728"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"12", ",", "192", ",", "3072", ",", "49152", ",", "786432"}], 
      "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " also accepts a second argument as a way to limit which tags are included \
in the output. For example, the following will output only the even-indexed \
terms."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"EvenQ", "[", "i", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "even"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "odd"}], "}"}]}], "]"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
   "\[IndentingNewLine]", ",", "even"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
     "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
      "3145728"}], "}"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second argument to ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " is actually a pattern, not a tag, so you can use pattern construction \
elements, such as ",
 StyleBox["|", "Input"],
 " to include options. The result will be the list of all lists whose tags \
match the pattern."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"EvenQ", "[", "i", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "even"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "odd"}], "}"}]}], "]"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
   "\[IndentingNewLine]", ",", 
   RowBox[{"all", "|", "even"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
       "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
       "3145728"}], "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " function can also accept a third argument: a function on two arguments. \
For each tag matching the pattern in ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 "\[CloseCurlyQuote]s second argument, the function is applied to the tag and \
the list associated with that tag. One useful construction is shown below and \
has the effect of turning the second element of the result into a list of \
rules identifying the tags with the associated lists."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"EvenQ", "[", "i", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "even"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sow", "[", 
        RowBox[{
         RowBox[{"3", "*", 
          RowBox[{"4", "^", "i"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"all", ",", "odd"}], "}"}]}], "]"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
   "\[IndentingNewLine]", ",", "_", ",", "Rule"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"all", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
        "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}],
        "}"}]}], ",", 
     RowBox[{"even", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
        "3145728"}], "}"}]}], ",", 
     RowBox[{"odd", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"12", ",", "192", ",", "3072", ",", "49152", ",", "786432"}], 
       "}"}]}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the second argument used above is a blank (",
 StyleBox["_", "Input"],
 "), ensuring that all tags are included in the output. The function ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " is given as the third argument, and it is applied to each tag and \
associated list creating three rules in the output. By storing this list of \
rules as a symbol, and then using a ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") in conjunction with a ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") that identifies a symbol with a tag, you can assign one of the lists to a \
symbol."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ruleList", "=", 
  RowBox[{
   RowBox[{"Reap", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"EvenQ", "[", "i", "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Sow", "[", 
          RowBox[{
           RowBox[{"3", "*", 
            RowBox[{"4", "^", "i"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"all", ",", "even"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Sow", "[", 
          RowBox[{
           RowBox[{"3", "*", 
            RowBox[{"4", "^", "i"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"all", ",", "odd"}], "}"}]}], "]"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", "0", ",", "10"}], "}"}]}], "]"}], 
     "\[IndentingNewLine]", ",", "_", ",", "Rule"}], "]"}], "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"all", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
      "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
     "}"}]}], ",", 
   RowBox[{"even", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", 
      "3145728"}], "}"}]}], ",", 
   RowBox[{"odd", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"12", ",", "192", ",", "3072", ",", "49152", ",", "786432"}], 
     "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"evenList", "=", 
  RowBox[{"even", "/.", "ruleList"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["evenList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "48", ",", "768", ",", "12288", ",", "196608", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You might wonder about using ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " instead of ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " as the final argument to ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 ". This will have the effect of assigning the appropriate lists to the \
symbols used as the tags. However, there is a significant drawback. \
Specifically, once the ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " expression has been evaluated and the assignment of tags to lists made, \
you can no longer use those symbols as tags without first removing the \
assignment with ",
 ButtonBox["Unset",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Unset"],
 " (",
 StyleBox["=.", "Input"],
 ") or ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 ". This does not, however, apply in contexts where the tags are made local, \
such as inside of a ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 ", as is done below. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"geometricSequence3", "[", 
   RowBox[{"a_", ",", "r_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "seq"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"Sow", "[", 
          RowBox[{
           RowBox[{"a", "*", 
            RowBox[{"r", "^", "i"}]}], ",", "seq"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}], 
       "\[IndentingNewLine]", ",", "_", ",", "Set"}], "]"}], ";", 
     "\[IndentingNewLine]", "seq"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"geometricSequence3", "[", 
  RowBox[{"3", ",", "4", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "12", ",", "48", ",", "192", ",", "768", ",", "3072", ",", 
   "12288", ",", "49152", ",", "196608", ",", "786432", ",", "3145728"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In ",
 StyleBox["geometricSequence3", "Input"],
 ", the symbol ",
 StyleBox["seq", "Input"],
 " is localized to the module. It is then used as the tag to ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 ". The enclosing ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " uses the ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " function, which has the effect of setting ",
 StyleBox["seq", "Input"],
 " to the list containing the geometric sequence. Once the ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " function is closed, we have the function evaluate ",
 StyleBox["seq", "Input"],
 " so that it becomes the output."
}], "Text"],

Cell[TextData[{
 "Note that making ",
 StyleBox["seq", "Input"],
 " local means that it does not store the sequence after the function is \
complete and that it can be reused. If we did not make it local, the function \
could only be executed once. Consequently, you should use this approach with \
caution."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Recurrence Relations", "Subsection",
 CellTags->"Section4d"],

Cell[TextData[{
 "Next, we will see how we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore sequences that arise from recurrence relations. We will go into \
much more depth, especially in regards to ",
 StyleBox["Mathematica",
  FontSlant->"Italic",
  \ FontSlant -> "Italic"],
 StyleBox["\[CloseCurlyQuote]",
  \ FontSlant -> "Italic"],
 "s functions related to solving recurrence relations, in Chapter 8. Here, we \
will only explore how we can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " compute terms of sequences defined by recurrence relations."
}], "Text"],

Cell[TextData[{
 "As an example, consider the Fibonacci sequence, which has recurrence \
relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "n"], "=", 
    RowBox[{
     SubscriptBox["f", 
      RowBox[{"n", "-", "1"}]], "+", 
     SubscriptBox["f", 
      RowBox[{"n", "-", "2"}]]}]}], TraditionalForm]]],
 " and initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "1"], "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "2"], "=", "1"}], TraditionalForm]]],
 ". (Note that the textbook uses ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " as the first index for a sequence, but ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses 1 as the first index for lists, and we will follow the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " convention.) To produce this sequence in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can use an indexed variable. Define a function named ",
 StyleBox["fib", "Input"],
 " to represent the recurrence relation as follows."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fib", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"fib", "[", 
    RowBox[{"n", "-", "1"}], "]"}], "+", 
   RowBox[{"fib", "[", 
    RowBox[{"n", "-", "2"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell["Next, we set the initial values as follows.", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"fib", "[", "1", "]"}], "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fib", "[", "2", "]"}], "=", "1"}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will compute values of the sequence."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fib", "[", "8", "]"}]], "Input"],

Cell[BoxData["13"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To display a list of elements of the sequence, use the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"fib", "[", "i", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "1", ",", "2", ",", "3", ",", "5", ",", "8", ",", "13", 
   ",", "21", ",", "34", ",", "55", ",", "89", ",", "144", ",", "233", ",", 
   "377", ",", "610", ",", "987", ",", "1597", ",", "2584", ",", "4181"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "While the above approach for calculating recurrence relations is convenient \
and intuitive, it is also very inefficient. Observe that the definition of ",
 StyleBox["fib", "Input"],
 " contains only the recurrence relation and initial conditions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fib"}]], "Input"],

Cell["Global`fib", "Print", "PrintUsage",
 CellTags->"Info3580371082-5514480"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fib", "[", "1", "]"}], "=", "0"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib", "[", "2", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib", "[", "n_", "]"}], ":=", 
         RowBox[{
          RowBox[{"fib", "[", 
           RowBox[{"n", "-", "1"}], "]"}], "+", 
          RowBox[{"fib", "[", 
           RowBox[{"n", "-", "2"}], "]"}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`fib],
  Editable->False]], "Print",
 CellTags->"Info3580371082-5514480"]
}, Open  ]],

Cell[TextData[{
 "This tells us that computed values are not being stored, and so ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " must recompute all but the first two values of the sequence every time a \
value is desired. We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " calculate the sequence more efficiently by having it remember all of the \
values it computes. This is demonstrated below."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"fib2", "[", "1", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fib2", "[", "2", "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fib2", "[", "n_", "]"}], ":=", 
   RowBox[{
    RowBox[{"fib2", "[", "n", "]"}], "=", 
    RowBox[{
     RowBox[{"fib2", "[", 
      RowBox[{"n", "-", "1"}], "]"}], "+", 
     RowBox[{"fib2", "[", 
      RowBox[{"n", "-", "2"}], "]"}]}]}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We combined both the recurrence relation and initial conditions into one \
input expression. The above can be understood most easily by distinguishing \
between the ",
 StyleBox["function",
  FontSlant->"Italic"],
 " ",
 StyleBox["fib2", "Input"],
 " and the ",
 StyleBox["indexed variable",
  FontSlant->"Italic"],
 " ",
 StyleBox["fib2", "Input"],
 ". The first two lines assign values to the indexed variable ",
 StyleBox["fib2", "Input"],
 " with indices 1 and 2. The third line defines the function ",
 StyleBox["fib2", "Input"],
 ". This function accepts one argument, named ",
 StyleBox["n", "Input"],
 ". The body of the function is an assignment. Specifically, the function \
assigns a value to the indexed variable ",
 StyleBox["fib2", "Input"],
 ". That value is obtained by either looking up or calculating earlier values."
}], "Text"],

Cell[TextData[{
 "This means that values will be stored as they are calculated. You can see \
this by looking at the definition of ",
 StyleBox["fib2", "Input"],
 " before and after evaluating ",
 StyleBox["fib2[3]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fib2"}]], "Input"],

Cell["Global`fib2", "Print", "PrintUsage",
 CellTags->"Info3580371083-5514480"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fib2", "[", "1", "]"}], "=", "0"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib2", "[", "2", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib2", "[", "n_", "]"}], ":=", 
         RowBox[{
          RowBox[{"fib2", "[", "n", "]"}], "=", 
          RowBox[{
           RowBox[{"fib2", "[", 
            RowBox[{"n", "-", "1"}], "]"}], "+", 
           RowBox[{"fib2", "[", 
            RowBox[{"n", "-", "2"}], "]"}]}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`fib2],
  Editable->False]], "Print",
 CellTags->"Info3580371083-5514480"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fib2", "[", "3", "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "fib2"}]], "Input"],

Cell["Global`fib2", "Print", "PrintUsage",
 CellTags->"Info3580371083-5514480"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fib2", "[", "1", "]"}], "=", "0"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib2", "[", "2", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib2", "[", "3", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fib2", "[", "n_", "]"}], ":=", 
         RowBox[{
          RowBox[{"fib2", "[", "n", "]"}], "=", 
          RowBox[{
           RowBox[{"fib2", "[", 
            RowBox[{"n", "-", "1"}], "]"}], "+", 
           RowBox[{"fib2", "[", 
            RowBox[{"n", "-", "2"}], "]"}]}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`fib2],
  Editable->False]], "Print",
 CellTags->"Info3580371083-5514480"]
}, Open  ]],

Cell["\<\
To illustrate the difference in performance, let\[CloseCurlyQuote]s see how \
long it takes to compute the 100th Fibonacci number.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"fib2", "[", "100", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00041699999999999999870589628692130191`2.6407359682533813", ",", 
   "218922995834555169026"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that if we repeat the computation using ",
 StyleBox["fib2", "Input"],
 ":"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"fib2", "[", "100", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6.00000000000000015200514458246772`0.7987511636632681*^-6", ",", 
   "218922995834555169026"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The total time take drops dramatically. This is because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " doesn\[CloseCurlyQuote]t need to compute the value again, it just looks it \
up."
}], "Text"],

Cell[TextData[{
 "In comparison, consider the ",
 StyleBox["fib", "Input"],
 " function applied to 30."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"fib", "[", "30", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1.35853700000000010561507224338129162788`6.15367138428308", ",", 
   "514229"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
By storing the results of computations in a recursively generated sequence, \
we can get higher terms much more quickly.\
\>", "Text"],

Cell[TextData[{
 "Note that attempting to call ",
 StyleBox["fib2", "Input"],
 " with larger values, such as 1000, will raise an error, specifically:"
}], "Text"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"$RecursionLimit", "::", "reclim"}], "MessageName"], ":", 
  " ", "\<\"Recursion depth of \[NoBreak]\\!\\(256\\)\[NoBreak] exceeded. \\!\
\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/$RecursionLimit/reclim\\\", ButtonNote -> \
\\\"$RecursionLimit::reclim\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " puts a limit on how far down a recursive definition it will go. This is to \
prevent ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from crashing your computer by trying to exceed the machine\
\[CloseCurlyQuote]s stack space. To get around this, you can temporarily \
override the default recursion limit as in the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"$RecursionLimit", "=", "1000"}], "}"}], ",", 
   RowBox[{"fib2", "[", "1000", "]"}]}], "]"}]], "Input",
 Evaluatable->False],

Cell[BoxData[\
"26863810024485359386146727202142923967616609318986952340123175997617981700247\
881689338369654483356564191827856161443356312976673642210350324634850410377680\
367334151172899169723197082763985615764450078474174626"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Block",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Block"], "FunctionLink"],
 " is similar to ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 ", in that it defines a scope and the first argument declares the variables \
local to that scope. In the above, ",
 StyleBox[ButtonBox["Block",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Block"], "FunctionLink"],
 " is used to ensure that the change to the recursion limit is only \
temporary. The built-in recursion limit is there for a reason, and overriding \
it is inherently dangerous. If you were to change it permanently and then \
unwittingly execute an expression that caused a large recursive descent, you \
could crash your ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session and lose your work. "
}], "Text"],

Cell[TextData[{
 "Note that the purely recursive implementation ",
 StyleBox["fib", "Input"],
 " cannot be used to compute the thousandth Fibonacci number. In fact, to \
compute the thousandth Fibonacci number, ",
 StyleBox["fib", "Input"],
 " would need to be invoked approximately"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fib2", "[", "999", "]"}]], "Input",
 Evaluatable->False],

Cell[BoxData[\
"16602747662452097049541800472897701834948051198384828062358553091918573717701\
170201065510185595898605104094736918879278462233015981029522997836311232618760\
539199036765399799926731433239718860373345088375054249"], "Output"]
}, Open  ]],

Cell["\<\
times in order to handle all the recursive sub-calls that are made. (The \
reader is encouraged to prove this fact.)\
\>", "Text"],

Cell["Even at a billion calls per second, this would require", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fib2", "[", "999", "]"}], "/", "1000000000."}]], "Input",
 Evaluatable->False],

Cell[BoxData["1.6602747662452098`*^199"], "Output"]
}, Open  ]],

Cell["seconds, or", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fib2", "[", "999", "]"}], "/", 
  RowBox[{"(", 
   RowBox[{"1000000000.", "*", "60", "*", "60", "*", "24", "*", "365"}], 
   ")"}]}]], "Input",
 Evaluatable->False],

Cell[BoxData["5.2646967473528974`*^191"], "Output"]
}, Open  ]],

Cell["years to complete.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Summations", "Subsection",
 CellTags->"Section4e"],

Cell[TextData[{
 "Finally, we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to compute with summations, both numerically for finite sums \
and symbolically for infinite sums."
}], "Text"],

Cell[TextData[{
 "To add a finite list of values, we use the ",
 StyleBox[ButtonBox["Total",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Total"], "FunctionLink"],
 " function. Applied to a list, ",
 StyleBox[ButtonBox["Total",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Total"], "FunctionLink"],
 " returns the sum."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Total", "[", 
  RowBox[{"{", 
   RowBox[{
   "1", ",", "2", ",", "4", ",", "6", ",", "9", ",", "11", ",", "14"}], "}"}],
   "]"}]], "Input"],

Cell[BoxData["47"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The argument to ",
 StyleBox[ButtonBox["Total",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Total"], "FunctionLink"],
 " can be a function that returns a list. For example, the following computes \
the sum of the integers from 1 to 100."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Total", "[", 
  RowBox[{"Range", "[", "100", "]"}], "]"}]], "Input"],

Cell[BoxData["5050"], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides the function ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 ", which can be used in place of composing ",
 StyleBox[ButtonBox["Total",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Total"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " accepts arguments in the same form as ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", but instead of producing the list, it provides you with the sum of the \
sequence."
}], "Text"],

Cell[TextData[{
 "The first argument to ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " should be an expression involving a variable (the index of summation). The \
second argument is generally a list, whose first element is the index of \
summation (e.g., ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ") and whose other arguments specify the values that the index takes on, as \
detailed in the table below."
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], Cell[TextData[{
     "sum from ",
     Cell[BoxData[
      FormBox[
       RowBox[{"i", "=", "1"}], TraditionalForm]]],
     " to ",
     Cell[BoxData[
      FormBox[
       SubscriptBox["i", "max"], TraditionalForm]]]
    }]]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], 
    RowBox[{"sum", " ", "from", " ", Cell[TextData[{
      Cell[BoxData[
       FormBox[
        RowBox[{"i", "=", 
         SubscriptBox["i", "min"]}], TraditionalForm]]],
      " to ",
      Cell[BoxData[
       FormBox[
        SubscriptBox["i", "max"], TraditionalForm]]]
     }]]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], ",", 
      StyleBox["step", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{"sum", " ", "from", " ", Cell[TextData[{
      Cell[BoxData[
       FormBox[
        RowBox[{"i", "=", 
         SubscriptBox["i", "min"]}], TraditionalForm]]],
      " to ",
      Cell[BoxData[
       FormBox[
        SubscriptBox["i", "max"], TraditionalForm]]],
      " by ",
      Cell[BoxData[
       FormBox["step", TraditionalForm]]]
     }]]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox["list", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
    "sum", " ", "over", " ", Cell["i\[Element]list"], Cell[TextData[StyleBox[
     "", "Input"]]]}]}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "For example, to compute the sum of the squares of the first ten positive \
integers, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{" ", 
      RowBox[{"i", "=", "1"}]}], 
     RowBox[{" ", "10"}]], 
    SuperscriptBox["i", "2"]}], TraditionalForm]]],
 ", we enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["385"], "Output"]
}, Open  ]],

Cell[TextData[{
 "And to compute the sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{" ", 
      RowBox[{"i", "=", "10"}]}], 
     RowBox[{" ", "30"}]], 
    FractionBox["1", "i"]}], TraditionalForm]]],
 ", you enter:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"1", "/", "i"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10", ",", "30"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox["2715762396337", "2329089562800"]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that nested sums can be computed simply by providing additional \
specifications for the index of summation, just as is done with ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". Also note that there is a similar command for products, with identical \
syntax, called ",
 StyleBox[ButtonBox["Product",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Product"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function can also be used for symbolic sums. For example, to compute the \
sum of the squares of the integers from 1 to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{" ", 
      RowBox[{"i", "=", "1"}]}], 
     RowBox[{" ", "n"}]], 
    SuperscriptBox["i", "2"]}], TraditionalForm]]],
 ", you simply give the maximum for the index of summation as ",
 StyleBox["n", "Input"],
 ", provided the symbol ",
 StyleBox["n", "Input"],
 " has not been assigned a value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"i", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "n"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "6"], " ", "n", " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", "n"}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "n"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We also compute the sum of every other term from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "n"}], TraditionalForm]]],
 " in the general geometric series."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"a", "*", 
    RowBox[{"r", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", 
     RowBox[{"2", "n"}], ",", "2"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{"a", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["r", 
      RowBox[{"2", "+", 
       RowBox[{"2", " ", 
        RowBox[{"Floor", "[", "n", "]"}]}]}]]}], ")"}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   SuperscriptBox["r", "2"]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You are also allowed to use the symbol ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 " to compute infinite sums. The following computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{" ", 
      RowBox[{"i", "=", "1"}]}], 
     RowBox[{" ", "\[Infinity]"}]], 
    SuperscriptBox["ix", 
     RowBox[{"i", "-", "1"}]]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"i", "*", 
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"i", "-", "1"}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "Infinity"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox["1", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "x"}], ")"}], "2"]]], "Output"]
}, Open  ]],

Cell["\<\
You can confirm that these results match the formulas given in Table 2 of \
Section 2.4.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.5 Cardinality of Sets", "Section",
 PageBreakWithin->False,
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will explore the countability of the positive rational \
numbers. In Example 4 of Section 2.5 of the text, it is shown that the \
positive rationals are countable by describing how to list them all. Here, we \
will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to implement this listing algorithm. We will also consider the following \
two questions. First, given a positive rational number, what is its position \
in the list? Second, given a positive integer, what fraction is located at \
that position within the list?"
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[TextData[{
 "We begin by reviewing the description in Example 4. The first element of \
the list is the rational number ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "1"], TraditionalForm]]],
 ". Then we list the positive rationals ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "=", "3"}], TraditionalForm]]],
 ". Then come the rationals with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "=", "4"}], TraditionalForm]]],
 ", excluding ",
 Cell[BoxData[
  FormBox[
   FractionBox["2", "2"], TraditionalForm]]],
 ", which is already in the list, being equivalent to ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "1"], TraditionalForm]]],
 ". This continues for each ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ": we list the fractions ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "=", "n"}], TraditionalForm]]],
 ", excluding those equivalent to fractions already in the list."
}], "Text"],

Cell[TextData[{
 "In our function, we refer to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as the stage, so that in stage 5, for example, we\[CloseCurlyQuote]re \
listing the fractions ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "=", "5"}], TraditionalForm]]],
 ". The stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " will range from ",
 Cell[BoxData[
  FormBox["2", TraditionalForm]]],
 " up to some maximum value. This maximum value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " will be the parameter to the function. We\[CloseCurlyQuote]ll implement \
this as a loop with index variable ",
 StyleBox["n", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Within each stage, i.e., within the loop, we need to generate the rational \
numbers ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " and add them to the list, provided they are not already in it. We can \
rewrite ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "=", "n"}], TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", 
    RowBox[{"n", "-", "q"}]}], TraditionalForm]]],
 ". By allowing ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " to range from ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " and calculating ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", we will produce all the potential rationals in stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". The ",
 StyleBox[ButtonBox["FreeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FreeQ"], "FunctionLink"],
 " function, which has the same syntax but opposite meaning of the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function discussed in Section 2.1 in relation to sets, will be used to \
avoid duplicates. So, for each ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", we will form the fraction ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " (with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", 
    RowBox[{"n", "-", "q"}]}], TraditionalForm]]],
 "), use ",
 StyleBox[ButtonBox["FreeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FreeQ"], "FunctionLink"],
 " to test whether this is already in our list of positive rationals, and, if \
not, add it to the list using ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 "."
}], "Text"],

Cell["Here is the complete function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"listRationals", "[", "max_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}], ",", "n", ",", "p", ",", "q"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"n", "=", "2"}], ",", 
       RowBox[{"n", "\[LessEqual]", "max"}], ",", 
       RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"q", "=", "1"}], ",", 
         RowBox[{"q", "\[LessEqual]", 
          RowBox[{"n", "-", "1"}]}], ",", 
         RowBox[{"q", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"p", "=", 
           RowBox[{"n", "-", "q"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"FreeQ", "[", 
             RowBox[{"L", ",", 
              RowBox[{"p", "/", "q"}]}], "]"}], ",", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"L", ",", 
              RowBox[{"p", "/", "q"}]}], "]"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "L"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "Applying this function with argument ",
 Cell[BoxData[
  FormBox["6", TraditionalForm]]],
 ", we obtain the list of rationals through stage ",
 Cell[BoxData[
  FormBox["6", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listRationals", "[", "6", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", 
   FractionBox["1", "2"], ",", "3", ",", 
   FractionBox["1", "3"], ",", "4", ",", 
   FractionBox["3", "2"], ",", 
   FractionBox["2", "3"], ",", 
   FractionBox["1", "4"], ",", "5", ",", 
   FractionBox["1", "5"]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding the Position Given a Positive Rational", "Subsection",
 CellTags->"Section5a"],

Cell[TextData[{
 "Suppose we want to determine the position of a particular fraction within \
the list. Take for example ",
 Cell[BoxData[
  FormBox[
   FractionBox["29", "35"], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"29", "+", "35"}], "=", "64"}], TraditionalForm]]],
 ", we know that this fraction would first appear in stage 64. So we compute \
the list up to stage 64."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"ratsTo64", "=", 
   RowBox[{"listRationals", "[", "64", "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["We suppress the output because this is a long list:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "ratsTo64", "]"}]], "Input"],

Cell[BoxData["1259"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we work backwards from the end of the list until we find the desired \
fraction. A simple loop will help with this. Recall that the third argument \
to ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " allows us to specify how the index variable is changed each time. Using \
the decrement operator (",
 StyleBox["--", "Input"],
 ") causes the loop to step backwards by 1 each iteration. Once we find the \
location of the desired fraction, we want the loop to end and output the \
location of the fraction. We do this using a ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " and wrapping the entire loop in a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Catch", "[", "\[IndentingNewLine]", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1259"}], ",", 
    RowBox[{"i", "\[GreaterEqual]", "1"}], ",", 
    RowBox[{"i", "--"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"ratsTo64", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", 
       RowBox[{"29", "/", "35"}]}], ",", 
      RowBox[{"Throw", "[", "i", "]"}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}], "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData["1245"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can make this process into a function. Given a fraction, the ",
 StyleBox[ButtonBox["Numerator",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Numerator"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Denominator",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Denominator"], "FunctionLink"],
 " commands will extract the numerator and denominator, respectively."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Numerator", "[", 
  RowBox[{"29", "/", "35"}], "]"}]], "Input"],

Cell[BoxData["29"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Denominator", "[", 
  RowBox[{"29", "/", "35"}], "]"}]], "Input"],

Cell[BoxData["35"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Our function will accept a rational number as its argument. For the sake of \
robustness, we will ensure that the input is rational by insisting that the \
argument has head ",
 StyleBox["Rational", "Input"],
 ", and we will also check that it is positive using a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ")."
}], "Text"],

Cell[TextData[{
 "After confirming that the input is positive, we\[CloseCurlyQuote]ll sum the \
results of ",
 StyleBox[ButtonBox["Numerator",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Numerator"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Denominator",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Denominator"], "FunctionLink"],
 " to determine the stage. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"locateRational", "[", "r_Rational", "]"}], "/;", 
   RowBox[{"r", ">", "0"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"stage", ",", "L", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"stage", "=", 
      RowBox[{
       RowBox[{"Numerator", "[", "r", "]"}], "+", 
       RowBox[{"Denominator", "[", "r", "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"L", "=", 
      RowBox[{"listRationals", "[", "stage", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", 
         RowBox[{"Length", "[", "L", "]"}]}], ",", 
        RowBox[{"i", "\[GreaterEqual]", "1"}], ",", 
        RowBox[{"i", "--"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "r"}], ",", 
          RowBox[{"Throw", "[", "i", "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"locateRational", "[", 
  RowBox[{"75", "/", "197"}], "]"}]], "Input"],

Cell[BoxData["22566"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Finding the Rational in a Given Position", "Subsection",
 CellTags->"Section5b"],

Cell["\<\
On the other hand, suppose we want to know which fraction is at a particular \
position. For instance, say we want to know which is the hundredth fraction \
listed. If we knew which stage of the process would yield a list of at least \
100 rational numbers, we could just generate the list up to that stage. We \
can guess and check until we find a stage that produced a long enough list.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Putting a Lower Bound on the Number of Stages", "Subsubsection"],

Cell[TextData[{
 "We can guide our guesses a bit, however. Remember that at stage 2, the \
process generates 1 fraction. At stage 3, it generates 2 fractions. At stage \
4, it generates 3 fractions, although one of them is discarded because it is \
a repeat. At stage ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", the process generates ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " rational numbers, some of which may be discarded as repeats. So we know \
that, after stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is complete, the number of rational numbers in our list contains ",
 StyleBox["at most",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{" ", 
       RowBox[{"k", "=", "2"}]}], 
      RowBox[{" ", "n"}]], "k"}], "-", "1"}], TraditionalForm]]],
 " rational numbers. We can use the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function described in the previous section to find a formula for this \
summation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"k", "-", "1"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "2", ",", "n"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "n"}], "+", 
    SuperscriptBox["n", "2"]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In other words, the number of rational numbers in the list produced by ",
 StyleBox["listRationals", "Input"],
 " at the conclusion of stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is at most ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SuperscriptBox["n", "2"], "-", "n"}], "2"], TraditionalForm]]],
 ". Define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 " to be the number of positive rational numbers produced by the ",
 StyleBox["listRationals", "Input"],
 " algorithm at the conclusion of stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Equivalently, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 " is the number of distinct positive rational numbers ",
 Cell[BoxData[
  FormBox[
   FractionBox["p", "q"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "+", "q"}], "\[LessEqual]", "n"}], TraditionalForm]]],
 ". We have determined that"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"F", "(", "n", ")"}], "\[LessEqual]", 
   FractionBox[
    RowBox[{
     SuperscriptBox["n", "2"], "-", "n"}], "2"]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "Now we return to the question of how many stages we need to compute in \
order to find the 100th rational number. We can restate this as follows: find \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "n", ")"}], "\[GreaterEqual]", "100"}], 
   TraditionalForm]]],
 ". Combining our inequalities, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["n", "2"], "-", "n"}], "2"], "\[GreaterEqual]", "100"}], 
   TraditionalForm]]],
 ". ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " command will solve the equation for us. The ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " function requires two arguments: the expression to be simplified and the \
variable. (Note that since an approximation is sufficient, we apply the \
function ",
 StyleBox[ButtonBox["N",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/N"], "FunctionLink"],
 ", so that the output will be given as decimals. It is common to apply ",
 StyleBox[ButtonBox["N",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/N"], "FunctionLink"],
 " in conjunction with the postfix operator (",
 StyleBox["//", "Input"],
 ").)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Reduce", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"n", "^", "2"}], "-", "n"}], ")"}], "/", "2"}], 
     "\[GreaterEqual]", "100"}], ",", "n"}], "]"}], "//", "N"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n", "\[LessEqual]", 
   RowBox[{"-", "13.650971698084906`"}]}], "||", 
  RowBox[{"n", "\[GreaterEqual]", "14.650971698084906`"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This indicates that a stage of 14 is ",
 StyleBox["not enough",
  FontSlant->"Italic"],
 ". But it gives us a place to start guessing."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ratsTo15", "=", 
   RowBox[{"listRationals", "[", "15", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "ratsTo15", "]"}]], "Input"],

Cell[BoxData["71"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ratsto17", "=", 
   RowBox[{"listRationals", "[", "17", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "ratsto17", "]"}]], "Input"],

Cell[BoxData["95"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ratsTo18", "=", 
   RowBox[{"listRationals", "[", "18", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "ratsTo18", "]"}]], "Input"],

Cell[BoxData["101"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ratsTo18", "[", 
  RowBox[{"[", "100", "]"}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox["5", "13"]], "Output"]
}, Open  ]],

Cell[TextData[{
 "So the 100th rational, in the order specified by this particular algorithm, \
is ",
 Cell[BoxData[
  FormBox[
   FractionBox["5", "13"], TraditionalForm]]],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["How Tight is the Bound?", "Subsubsection"],

Cell[TextData[{
 "We just saw how the formula ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SuperscriptBox["n", "2"], "-", "1"}], "n"], TraditionalForm]]],
 " is an upper bound for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 ", the number of positive rationals listed by the end of stage ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We\[CloseCurlyQuote]ll conclude this section by exploring how good of a \
bound this is. In Section 2.3, we saw how to use the ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 " function to graph functions. Let\[CloseCurlyQuote]s use that technique to \
graph the upper bound ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SuperscriptBox["n", "2"], "-", "n"}], "2"], TraditionalForm]]],
 " from 1 to 100. Recall that the first argument to ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 " is the function and the second is a list consisting of the name of the \
independent variable and the bounds."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"boundPlot", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"n", "^", "2"}], "-", "n"}], ")"}], "/", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "1", ",", "100"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjHk41IkDhyfkijBuxi2EJtaRlnw/pNYxbtZXYmPVym5YyfFQJEcJRRQ6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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{1, 100}, {0., 4949.99979896939}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we stored the plot as the symbol ",
 StyleBox["boundPlot", "Input"],
 " so as to be able to reuse it later."
}], "Text"],

Cell[TextData[{
 "To find the actual values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 ", we need to find the size of the list returned by ",
 StyleBox["listRationals", "Input"],
 " applied to ",
 StyleBox["n", "Input"],
 ". In other words, we apply ",
 StyleBox["Length", "Input"],
 " to the result of ",
 StyleBox["listRationals[n]", "Input"],
 ". We use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to form the list of these values with ",
 StyleBox["n", "Input"],
 " ranging from 1 to 100."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dataTable", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"listRationals", "[", "n", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "100"}], "}"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " function, applied to a list of values, will interpret the members of the \
list as the ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "-coordinates of points whose ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinates are the index of the number, which in this case is the value \
of ",
 StyleBox["n", "Input"],
 ". We\[CloseCurlyQuote]ll make this graph red using ",
 StyleBox[ButtonBox["PlotStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotStyle"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dataPlot", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"dataTable", ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[{{}, 
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJxd1DtoE3EABvDDycGhg0iQDlWKFCmhatX47Km11netVeug1WJFStCKWkVq
OXxGrFrfVYPGZ6F0cHAQkZJBnBwcHR1FRDJ0FFH5/zKkgfDxy5cvXJK7m9N9
rL1nWhRFh/49/2d4lJqiykdc+fr0uDKr5MwpOVem5DxZLRfIGrl4ym6FrJVN
lZ+TNHMdt/B83sL1vJ3TIYs7uIF3Vh5XcQ8vDBnv5UX6fdzIXZXf48BBXhKy
0M1L9T2c4cO8LOS3I7xc38t+l0KWV4asOcqrQn7p49Uh246z37HhBMd8MmTC
faf0a7hfz6XT+rUh35zRc+msfl3IqgE9D5/T+x+HB/WcS/TrQ37nhDPn9f73
r1w+D0Yu6DeEbL2o53eX9K0+77Kec1f0G0PW5fRlX9VvcnyccO6afnPI1JCe
O6/rnaepG3rO3NRvDTljWM+THG0L2XhLz7W39c77PJevg/o7+raQ4xxz9q73
8wcu8sd79q6j5vv2PMkJdzyw5+yIfXvIzxzzz4f2PPDInqsf27tuxznmfN6e
f3P5Om98Yt8R8i3HPPTUnmcV7Pk9R7tC/uCYe5/Z8wQXOf3cfnfIQY45/cKe
93ORf3HkPtXy0p4nOOHZr+x5jKNOx88xj7225z9c5P5Re/fFT6PxX/5HxGw=

     "]]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 100.}, {0, 3043.}},
  PlotRangeClipping->True,
  PlotRangePadding->{{2., 2.}, {60.86, 60.86}}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can combine the two graphs using the ",
 StyleBox[ButtonBox["Show",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Show"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Show",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Show"], "FunctionLink"],
 " function allows you to combine graphics objects into one. We ensure that \
the entirety of both graphs is displayed by setting the option ",
 StyleBox[ButtonBox["PlotRange",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotRange"], "FunctionLink"],
 " to ",
 StyleBox["All", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"boundPlot", ",", "dataPlot", ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjHk41IkDhyfkijBuxi2EJtaRlnw/pNYxbtZXYmPVym5YyfFQJEcJRRQ6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      "]]}}, {{}, 
    {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJxd1DtoE3EABvDDycGhg0iQDlWKFCmhatX47Km11netVeug1WJFStCKWkVq
OXxGrFrfVYPGZ6F0cHAQkZJBnBwcHR1FRDJ0FFH5/zKkgfDxy5cvXJK7m9N9
rL1nWhRFh/49/2d4lJqiykdc+fr0uDKr5MwpOVem5DxZLRfIGrl4ym6FrJVN
lZ+TNHMdt/B83sL1vJ3TIYs7uIF3Vh5XcQ8vDBnv5UX6fdzIXZXf48BBXhKy
0M1L9T2c4cO8LOS3I7xc38t+l0KWV4asOcqrQn7p49Uh246z37HhBMd8MmTC
faf0a7hfz6XT+rUh35zRc+msfl3IqgE9D5/T+x+HB/WcS/TrQ37nhDPn9f73
r1w+D0Yu6DeEbL2o53eX9K0+77Kec1f0G0PW5fRlX9VvcnyccO6afnPI1JCe
O6/rnaepG3rO3NRvDTljWM+THG0L2XhLz7W39c77PJevg/o7+raQ4xxz9q73
8wcu8sd79q6j5vv2PMkJdzyw5+yIfXvIzxzzz4f2PPDInqsf27tuxznmfN6e
f3P5Om98Yt8R8i3HPPTUnmcV7Pk9R7tC/uCYe5/Z8wQXOf3cfnfIQY45/cKe
93ORf3HkPtXy0p4nOOHZr+x5jKNOx88xj7225z9c5P5Re/fFT6PxX/5HxGw=

      "]]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->All,
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.6 Matrices", "Section",
 CellTags->"Section6"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides extensive support for calculating with matrices. We begin this \
section by describing a variety of tools for constructing matrices in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Then we\[CloseCurlyQuote]ll consider matrix arithmetic and operations on \
zero-one matrices."
}], "Text"],

Cell[CellGroupData[{

Cell["Constructing Matrices", "Subsection",
 CellTags->"Section6a"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", matrices are represented as lists of lists, with the inner lists \
corresponding to the rows of the matrix. Consequently, the methods for \
working with lists apply also to matrices."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[CellGroupData[{

Cell["Specifying Matrices by Listing the Rows", "Subsubsection"],

Cell["\<\
The simplest way to create a matrix is to explicitly enter the values. The \
matrix is represented as a list of lists, with the first sublist consisting \
of the entries of the first row, the second sublist the second row, and so on.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"m1", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " display the result in the typical format of a rectangular matrix, apply \
the function ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 ". This is often done in conjunction with the postfix operator (",
 StyleBox["//", "Input"],
 "). "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"m1", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "2", "3"},
     {"4", "5", "6"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Be cautious, however, in using ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " in conjunction with assignment. If you were to include the application of ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " in the assignment of ",
 StyleBox["m1", "Input"],
 " above, then the ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " head would be permanently associated with the symbol and other functions \
will not behave as expected."
}], "Text"],

Cell[TextData[{
 "Using ",
 StyleBox[ButtonBox["Partition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Partition"], "FunctionLink"],
 ", you can list the entries of the matrix in a single list and have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " break it into rows of a size you specify. For example, the following \
creates the same matrix as ",
 StyleBox["m1", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"m2", "=", 
  RowBox[{"Partition", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
    ",", "3"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first argument is the list of all the entries in the matrix, from top \
left to bottom right. The second argument is the number of columns, that is, \
the number of elements in each row. Note that you can use ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") to compare matrices, which confirms that these two matrices are identical."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"m1", "\[Equal]", "m2"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Creating Matrices with ",
 StyleBox["Table", "Input"]
}], "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function can be used to create matrices in essentially the same way you \
use it to create lists. In this case, the first argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " will involve two variables, and there will be two table variable \
specifications, with the first associated to the columns and the second to \
the rows."
}], "Text"],

Cell["\<\
For example, the following creates the 4 by 5 matrix whose entries are the \
sum of their indices.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m3", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"i", "+", "j"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "5"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "m3", "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"2", "3", "4", "5", "6"},
     {"3", "4", "5", "6", "7"},
     {"4", "5", "6", "7", "8"},
     {"5", "6", "7", "8", "9"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the iteration specifications determine the values used while \
calculating the entries, but do not necessarily have to correspond to the \
position within the matrix. Below, we create the matrix whose entries are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "i"], 
    SuperscriptBox["y", "j"]}], TraditionalForm]]],
 " with the exponents to ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " ranging from 3 to 5 and the exponents of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " determined by the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "7", ",", "2", ",", "5"}], "}"}], TraditionalForm]]],
 ". We emphasize that the first iteration specification corresponds to the \
columns of the matrix and the second to the rows, which is clear from this \
output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m4", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", "i"}], " ", 
      RowBox[{"y", "^", "j"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "7", ",", "2", ",", "5"}], "}"}]}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"m4", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "5"]}]},
     {
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "5"]}]},
     {
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "5"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Manipulating and Combining Matrices", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IdentityMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IdentityMatrix"], "FunctionLink"],
 " function takes a positive integer as its argument and produces an identity \
matrix of that size."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IdentityMatrix", "[", "3", "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"0", "1", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " function can be used to create a list (including a matrix) all of whose \
entries are identical. Its first argument is the constant with which to \
populate the list and the second argument specifies the size of the \
structure. "
}], "Text"],

Cell[TextData[{
 "If the second argument is an integer, ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " produces a list of that length."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConstantArray", "[", 
  RowBox[{"x", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"x", ",", "x", ",", "x", ",", "x", ",", "x"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
If the second argument is a list consisting of a pair of integers, it \
produces a matrix of that size.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m5", "=", 
   RowBox[{"ConstantArray", "[", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "3"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"m5", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can then modify the entries of the constant matrix using the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator. Note that the locations within the matrix specified with the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator are consistent with the usual mathematical notation, that is, \
row then column."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"m5", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "=", "5"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"m5", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "3"}], "]"}], "]"}], "=", "3"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"m5", "[", 
    RowBox[{"[", 
     RowBox[{"3", ",", "2"}], "]"}], "]"}], "=", "7"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"m5", "[", 
    RowBox[{"[", 
     RowBox[{"4", ",", "1"}], "]"}], "]"}], "=", 
   RowBox[{"-", "2"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"m5", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"5", "0", "3"},
     {"0", "0", "0"},
     {"0", "7", "0"},
     {
      RowBox[{"-", "2"}], "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can merge two matrices, effectively gluing one next to the other or one \
on top of the other, provided the dimensions match, with the ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " function. "
}], "Text"],

Cell[TextData[{
 "For example, if you have two matrices with the same number of columns, you \
can stack one on top of the other by calling ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " on the two matrices. The top matrix should be given as the first \
argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"IdentityMatrix", "[", "3", "]"}], ",", "m5"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"0", "1", "0"},
     {"0", "0", "1"},
     {"5", "0", "3"},
     {"0", "0", "0"},
     {"0", "7", "0"},
     {
      RowBox[{"-", "2"}], "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Given two matrices with the same number of rows, you can combine them side \
by side by calling ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " with the two matrices as the first two arguments and the number 2 as a \
third argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"IdentityMatrix", "[", "3", "]"}], ",", "m4", ",", "2"}], "]"}], "//",
   "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "5"]}]},
     {"0", "1", "0", 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "5"]}]},
     {"0", "0", "1", 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "5"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " function is in fact a very useful general function. It accepts as \
arguments a number of expressions and an optional integer, provided that the \
expressions all have the same head, e.g., ",
 StyleBox["List", "Input"],
 ". If the optional integer is not present, ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " combines the expressions into one expression with that common head. The \
optional argument specifies the level at which to join. So by giving the \
argument 2, it means that rather than joining the expressions, it joins their \
corresponding subexpressions."
}], "Text"],

Cell[TextData[{
 "Finally, the ",
 StyleBox[ButtonBox["ArrayPad",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ArrayPad"], "FunctionLink"],
 " function can be used to extend a matrix in any direction by padding it \
with a constant. The first argument is the existing list or matrix. The \
second argument specifies the dimensions of the padding, that is, how many \
additional rows or columns to add on each side. The third (optional) argument \
specifies the object to pad with. The padding defaults to 0."
}], "Text"],

Cell[TextData[{
 "The most general form of the second argument, the dimensions of the \
padding, is a list of the form ",
 Cell[BoxData[
  RowBox[{"{", "{", 
   StyleBox["top", "CodeVar"], ",", 
   StyleBox["bottom", "CodeVar"], "}", ",", "{", 
   StyleBox["left", "CodeVar"], ",", 
   StyleBox["right", "CodeVar"], "}", "}"}]], "myCode"],
 ", where ",
 StyleBox["top", "CodeVar"],
 " stands for the number of rows to add above the matrix, etc. For example, \
the following adds 1s along the bottom and right of the matrix ",
 StyleBox["m4", "Input"],
 " and two columns along the left."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ArrayPad", "[", 
   RowBox[{"m4", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}]}], "}"}], ",", "1"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {"1", "1", "1", "1", "1", "1", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second argument can be simplified when the padding is consistent. For \
example, to add columns of 1s along the bottom and right, rather than \
providing ",
 StyleBox["{{0,1},{0,1}}", "Input"],
 ", you can simply use ",
 StyleBox["{0,1}", "Input"],
 ". The correct interpretation of this is that we\[CloseCurlyQuote]re adding \
nothing at the beginning and 1 entry at the end of every level (row and \
column)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ArrayPad", "[", 
   RowBox[{"m4", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "1"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "5"]}], "1"},
     {"1", "1", "1", "1", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
If you give just an integer as the second argument, then that number of \
elements will be added in every direction.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ArrayPad", "[", 
   RowBox[{"m4", ",", "2", ",", "1"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "1", "1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1", "1", "1", "1"},
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "3"], " ", 
       SuperscriptBox["y", "5"]}], "1", "1"},
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "4"], " ", 
       SuperscriptBox["y", "5"]}], "1", "1"},
     {"1", "1", 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "3"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "7"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{
       SuperscriptBox["x", "5"], " ", 
       SuperscriptBox["y", "5"]}], "1", "1"},
     {"1", "1", "1", "1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1", "1", "1", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Matrix Arithmetic", "Subsection",
 CellTags->"Section6b"],

Cell[TextData[{
 "The textbook defines addition and multiplication of matrices. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " implements these operations on matrices in a fairly intuitive way. To add \
two matrices, you use the ",
 StyleBox["+", "Input"],
 " operator, as you would expect."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m6", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"m6", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "2", "3"},
     {"4", "5", "6"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m7", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "3", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "5", ",", "2"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"m7", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "2"}], "3", 
      RowBox[{"-", "1"}]},
     {"1", "5", "2"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"m6", "+", "m7"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "5", "2"},
     {"5", "10", "8"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s syntax for multiplying a matrix by a scalar is also \
intuitive."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"3", "*", "m7"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "6"}], "9", 
      RowBox[{"-", "3"}]},
     {"3", "15", "6"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This produces the matrix whose entries are three times the entries of ",
 StyleBox["m7", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Matrix multiplication is computed using the ",
 StyleBox[ButtonBox["Dot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dot"], "FunctionLink"],
 " (",
 StyleBox[".", "Input"],
 ") operator, not the ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " (",
 StyleBox["*", "Input"],
 ") operator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m8", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"3", ",", "6", ",", "11", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "5", ",", "2", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "8", ",", "9", ",", 
       RowBox[{"-", "3"}]}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"m8", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"3", "6", "11", "1"},
     {
      RowBox[{"-", "2"}], "5", "2", "0"},
     {"4", "8", "9", 
      RowBox[{"-", "3"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m9", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"m9", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"2", "5"},
     {"1", 
      RowBox[{"-", "2"}]},
     {"3", "7"},
     {
      RowBox[{"-", "1"}], "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"m8", ".", "m9"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"44", "80"},
     {"7", 
      RowBox[{"-", "6"}]},
     {"46", "67"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reason that addition and scalar multiplication work in the natural way \
is that the functions ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " have the ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 " attribute, which means that they are automatically threaded over lists \
(and thus matrices). However, since ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " is ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " would compute the product ",
 StyleBox["m6*m7", "Input"],
 " by multiplying corresponding entries, even though this is not the correct \
operation mathematically."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Powers and Transposes of Matrices", "Subsection",
 CellTags->"Section6c"],

Cell[TextData[{
 "Computing powers of square matrices is done with the ",
 StyleBox[ButtonBox["MatrixPower",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixPower"], "FunctionLink"],
 " function. (Note that the usual ",
 StyleBox[ButtonBox["Power",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Power"], "FunctionLink"],
 " (",
 StyleBox["^", "Input"],
 ") operator will compute the power of each element of the matrix.) As you \
would expect, the first argument is the matrix and the second the exponent."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m10", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"7", ",", "8", ",", "9"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"m10", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "2", "3"},
     {"4", "5", "6"},
     {"7", "8", "9"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MatrixPower", "[", 
   RowBox[{"m10", ",", "5"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"121824", "149688", "177552"},
     {"275886", "338985", "402084"},
     {"429948", "528282", "626616"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
Note that if the exponent is negative, the result will be the power of the \
inverse, provided one exists. \
\>", "Text"],

Cell[TextData[{
 "The transpose of a matrix can be computed with the ",
 StyleBox[ButtonBox["Transpose",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Transpose"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", "m10", "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "4", "7"},
     {"2", "5", "8"},
     {"3", "6", "9"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Zero-one Matrices", "Subsection",
 CellTags->"Section6d"],

Cell[TextData[{
 "With ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can create and manipulate zero-one matrices as well. In particular, we\
\[CloseCurlyQuote]ll consider how to compute the meet, join, and Boolean \
product of zero-one matrices."
}], "Text"],

Cell[TextData[{
 "Recall from the previous chapter that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " functions for computing the bitwise AND and OR operations. Also recall \
from the textbook that the meet and join of two zero-one matrices correspond \
to computing the bitwise AND and OR on corresponding entries. Since the ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " functions has the ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 " attribute, they automatically thread over lists. Thus, applying ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " to matrices of the same dimension exactly computes the meet and join."
}], "Text"],

Cell[TextData[{
 "Below, we define two example matrices and use ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " to compute their meet. The reader can verify that the result is correct."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"zerooneEx1", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"zerooneEx1", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "1"},
     {"1", "1", "0"},
     {"0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"zerooneEx2", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"zerooneEx2", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"1", "1", "1"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BitOr", "[", 
   RowBox[{"zerooneEx1", ",", "zerooneEx2"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "1"},
     {"1", "1", "1"},
     {"0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Type Checking Zero-one Matrices", "Subsubsection"],

Cell[TextData[{
 "Recall from Section 1.1 of this manual that the ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " functions will accept as input any integers, not just the bits 0 and 1. It \
is a good habit, when writing functions, to ensure that the input to a \
function is of the right type. Otherwise, the function may attempt to operate \
on the bad input and produce output without telling you that the input was \
bad. To prevent this, you use \[OpenCurlyDoubleQuote]type checking.\
\[CloseCurlyDoubleQuote] Here, we will see how to type check zero-one \
matrices and build those checks into functions ",
 StyleBox["meet", "Input"],
 " and ",
 StyleBox["join", "Input"],
 ". The body of these functions will simply be applications of ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 ", respectively. "
}], "Text"],

Cell[TextData[{
 "Our test will be based on ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in ",
 StyleBox[ButtonBox["MatrixQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixQ"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["MatrixQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixQ"], "FunctionLink"],
 " function accepts an expression and returns true if the object is a list of \
lists that represents a rectangular matrix and false otherwise. It also \
accepts, as a second optional argument, a function in one argument that \
imposes a condition on the elements of the matrix. Specifically, the second \
argument is a function that is applied to each element of the matrix and the ",
 StyleBox[ButtonBox["MatrixQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixQ"], "FunctionLink"],
 " function only returns true if the first argument is a matrix and every \
element of the matrix satisfies the test specified by the second argument."
}], "Text"],

Cell[TextData[{
 "For example, the following checks that the ",
 StyleBox["zerooneEx1", "Input"],
 " matrix is a matrix of integers, using ",
 StyleBox[ButtonBox["IntegerQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerQ"], "FunctionLink"],
 " as the test function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixQ", "[", 
  RowBox[{"zerooneEx1", ",", "IntegerQ"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
However, given a matrix with even one entry that is not an integer, it will \
return false.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"3", "/", "4"}], ",", "5"}], "}"}]}], "}"}], ",", "IntegerQ"}],
   "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To check whether an expression represents a zero-one matrix, we will need \
to design a function that returns true when its input is 0 or 1 and false \
otherwise. It is easy to create such a function using a pure function. Recall \
from Section 2.3 of this manual that a pure function of one argument is \
constructed as an expression using the ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ") in place of the argument and ending with an ampersand, that is, the ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") operator. To test whether the argument is 0 or 1, we just need to \
evaluate the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "=", "0"}], ")"}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{"x", "=", "1"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Here is the definition of ",
 StyleBox["zeroOneMatrixQ", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"zeroOneMatrixQ", "[", "m_", "]"}], ":=", 
  RowBox[{"MatrixQ", "[", 
   RowBox[{"m", ",", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"#", "\[Equal]", "0"}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{"#", "\[Equal]", "1"}], ")"}]}], "&"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
This function will return true for a zero-one matrix, but false if given an \
input that is not a rectangular matrix or contains elements that are not 0 or \
1.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"zeroOneMatrixQ", "[", "zerooneEx1", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"zeroOneMatrixQ", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"zeroOneMatrixQ", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "2"}], "}"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can now use ",
 StyleBox["zeroOneMatrixQ", "Input"],
 " to impose type checking in order to build functions ",
 StyleBox["meet", "Input"],
 " and ",
 StyleBox["join", "Input"],
 ". We use the ",
 StyleBox["_?", "Input"],
 " construction to ensure that the arguments are zero-one matrices."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"meet", "[", 
   RowBox[{
    RowBox[{"a_", "?", "zeroOneMatrixQ"}], ",", 
    RowBox[{"b_", "?", "zeroOneMatrixQ"}]}], "]"}], ":=", 
  RowBox[{"BitAnd", "[", 
   RowBox[{"a", ",", "b"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"join", "[", 
   RowBox[{
    RowBox[{"a_", "?", "zeroOneMatrixQ"}], ",", 
    RowBox[{"b_", "?", "zeroOneMatrixQ"}]}], "]"}], ":=", 
  RowBox[{"BitOr", "[", 
   RowBox[{"a", ",", "b"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
These functions now perform their respective operations, but only on zero-one \
matrices.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meet", "[", 
   RowBox[{"zerooneEx1", ",", "zerooneEx2"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"1", "1", "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"join", "[", 
   RowBox[{"zerooneEx1", ",", "zerooneEx2"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "1"},
     {"1", "1", "1"},
     {"0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementing the Boolean Product", "Subsubsection"],

Cell[TextData[{
 "We conclude by implementing the Boolean product. Recall two key points from \
Definition 9 in Section 2.6. First, the size of the product of an ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "\[Times]", "k"}], TraditionalForm]]],
 " matrix and an ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[Times]", "n"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "\[Times]", "n"}], TraditionalForm]]],
 " and the product is undefined if the number of columns of the first matrix \
does not match the number of rows in the second. Second, the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]]],
 " entry of the product is given by the formula"
}], "Text"],

Cell[BoxData[
 FormBox[Cell[TextData[Cell[BoxData[
   FormBox[
    RowBox[{
     SubscriptBox["c", "ij"], "=", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["a", "i1"], "\[And]", 
        SubscriptBox["b", 
         RowBox[{"1", "j"}]]}], ")"}], "\[Or]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["a", "i2"], "\[And]", 
        SubscriptBox["b", 
         RowBox[{"2", "j"}]]}], ")"}], "\[Or]", "\[CenterEllipsis]", "\[Or]", 
      
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["a", "ik"], "\[And]", 
        SubscriptBox["b", "kj"]}], ")"}]}]}], TraditionalForm]]]]], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "Our Boolean product function, ",
 StyleBox["boolProduct", "Input"],
 ", needs to begin by confirming that the dimensions are correct. To do this, \
we will use the ",
 StyleBox[ButtonBox["Dimensions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dimensions"], "FunctionLink"],
 " function. For a matrix, ",
 StyleBox[ButtonBox["Dimensions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dimensions"], "FunctionLink"],
 " returns a list whose entries are the number of rows and columns."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since we know that the output will be a list with two elements, we can use ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") with a two-element list of symbols on the left hand side of the \
assignment to set two variables simultaneously, as demonstrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"numrows", ",", "numcolumns"}], "}"}], "=", 
  RowBox[{"Dimensions", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["numrows"], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["numcolumns"], "Input"],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[TextData[{
 "If the number of columns of the left-hand side matrix is not equal to the \
number of rows of the right-hand side matrix, we will display a message and \
use the ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " function to end the computation. The message is defined below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"boolProduct", "::", "dimMismatch"}], "=", 
   "\"\<The dimensions of the input matrices do not match.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Once the function confirms that the input matrices are of appropriate \
sizes, it will use ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " to initialize the ",
 StyleBox["output", "Input"],
 " matrix to the correct size and fill it with zeros."
}], "Text"],

Cell[TextData[{
 "The main work of the function is to loop over all the entries of the result \
matrix and calculate the appropriate value. We use two nested ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops with index variables ",
 StyleBox["i", "Input"],
 " and ",
 StyleBox["j", "Input"],
 " representing the rows and columns of the result matrix. Inside these ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops, we need to implement the formula for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "ij"], TraditionalForm]]],
 "."
}], "Text"],

Cell["It will be helpful to consider a specific example:", "Text"],

Cell[BoxData[
 FormBox[Cell[
  "(1\[And]0)\[Or](0\[And]0)\[Or](0\[And]1)\[Or](1\[And]1)\[Or](0\[And]1)"], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "We\[CloseCurlyQuote]ll approach this in the following way. First, compute \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "\[And]", "0"}], TraditionalForm]]],
 ", the first term, and store the result as ",
 StyleBox["cij", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cij", "=", 
  RowBox[{"BitAnd", "[", 
   RowBox[{"1", ",", "0"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then, update ",
 StyleBox["c", "Input"],
 " to be the result of applying ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " to it and the result of the next pair."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cij", "=", 
  RowBox[{"BitOr", "[", 
   RowBox[{"cij", ",", 
    RowBox[{"BitAnd", "[", 
     RowBox[{"0", ",", "0"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell["And then repeat with each successive meet.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cij", "=", 
  RowBox[{"BitOr", "[", 
   RowBox[{"cij", ",", 
    RowBox[{"BitAnd", "[", 
     RowBox[{"0", ",", "1"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cij", "=", 
  RowBox[{"BitOr", "[", 
   RowBox[{"cij", ",", 
    RowBox[{"BitAnd", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cij", "=", 
  RowBox[{"BitOr", "[", 
   RowBox[{"cij", ",", 
    RowBox[{"BitAnd", "[", 
     RowBox[{"0", ",", "1"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[TextData[{
 "In terms of the generic formula, we initialize ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "ij"], "=", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["a", "i1"], "\[And]", 
      SubscriptBox["b", 
       RowBox[{"1", "j"}]]}], ")"}]}], TraditionalForm]]],
 ". Then we begin a loop with index, say ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", from 2 through ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". At each step in the loop, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "ij"], "=", 
    RowBox[{
     SubscriptBox["c", "ij"], "\[Or]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["a", "ip"], "\[And]", 
       SubscriptBox["b", "pj"]}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Here is the implementation of ",
 StyleBox["boolProduct", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"boolProduct", "[", 
   RowBox[{
    RowBox[{"A_", "?", "zeroOneMatrixQ"}], ",", 
    RowBox[{"B_", "?", "zeroOneMatrixQ"}]}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "m", ",", "kA", ",", "kB", ",", "n", ",", "output", ",", "i", ",", "j", 
      ",", "c", ",", "p"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"m", ",", "kA"}], "}"}], "=", 
      RowBox[{"Dimensions", "[", "A", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"kB", ",", "n"}], "}"}], "=", 
      RowBox[{"Dimensions", "[", "B", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"kA", "\[NotEqual]", "kB"}], ",", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"boolProduct", "::", "dimMismatch"}], "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"output", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "m"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"j", "=", "1"}], ",", 
         RowBox[{"j", "\[LessEqual]", "n"}], ",", 
         RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"c", "=", 
           RowBox[{"BitAnd", "[", 
            RowBox[{
             RowBox[{"A", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
             RowBox[{"B", "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "j"}], "]"}], "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"p", "=", "2"}], ",", 
            RowBox[{"p", "\[LessEqual]", "kA"}], ",", 
            RowBox[{"p", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"c", "=", 
              RowBox[{"BitOr", "[", 
               RowBox[{"c", ",", 
                RowBox[{"BitAnd", "[", 
                 RowBox[{
                  RowBox[{"A", "[", 
                   RowBox[{"[", 
                    RowBox[{"i", ",", "p"}], "]"}], "]"}], ",", 
                  RowBox[{"B", "[", 
                   RowBox[{"[", 
                    RowBox[{"p", ",", "j"}], "]"}], "]"}]}], "]"}]}], "]"}]}],
              ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"output", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", "c"}], ";"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "output"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We test this function on the matrices from Example 8 in the textbook.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ex8a", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ex8a", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "1"},
     {"1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ex8b", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ex8b", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "0"},
     {"0", "1", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"boolProduct", "[", 
   RowBox[{"ex8a", ",", "ex8b"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "0"},
     {"0", "1", "1"},
     {"1", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 3", "Subsection",
 CellTags->"SolutionsCP3"],

Cell[TextData[{
 "Given fuzzy sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", find ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["A", "_"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[Union]", "B"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[Intersection]", "B"}], TraditionalForm]]],
 " (see preamble to Exercise 63 of Section 2.2)."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will compute the union and leave complement and intersection to the \
reader. Recall, from Exercise 64, that the union of fuzzy sets is the fuzzy \
set in which the degree of membership of an element is the maximum of the \
degrees of membership of that element in the given sets."
}], "Text"],

Cell[TextData[{
 "Recall, from the final subsection of Section 2.2 in this manual, that we \
developed two possible representations of fuzzy sets and the functions ",
 StyleBox["bitToRoster", "Input"],
 " and ",
 StyleBox["rosterToBit", "Input"],
 " to convert between them. We\[CloseCurlyQuote]ll design our function to \
accept the roster representation as input and return a roster representation \
of the union, since this representation is the most natural for humans to \
interact with. But in implementing the union, it is more natural to work with \
the fuzzy bit string representation of the sets."
}], "Text"],

Cell[TextData[{
 "Our ",
 StyleBox["fuzzyUnion", "Input"],
 " function will accept as input two fuzzy sets in the roster representation. \
It proceeds as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Determine the effective universe for the two sets. To do this, we will make \
use of the symbol ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " in conjunction with the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator. In particular, ",
 StyleBox["A[[All,1]]", "Input"],
 ", will produce the list of first elements of every sublist of ",
 StyleBox["A", "Input"],
 ". "
}], "Item1Numbered"],

Cell[TextData[{
 "Use ",
 StyleBox["rosterToBit", "Input"],
 " to convert both sets to their fuzzy bit representations."
}], "Item1Numbered"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Max",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Max"], "FunctionLink"],
 " function will determine the largest among its arguments. Using ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " in conjunction with ",
 StyleBox[ButtonBox["Max",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Max"], "FunctionLink"],
 " on the pair of fuzzy bit strings produces the list whose elements are the \
maximums of the corresponding entries in ",
 StyleBox["A", "Input"],
 " and ",
 StyleBox["B", "Input"],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "Use ",
 StyleBox["bitToRoster", "Input"],
 " on the result to obtain the roster representation."
}], "Item1Numbered"]
}, Open  ]],

Cell["Here is the implementation.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fuzzyUnion", "[", 
   RowBox[{"A_", ",", "B_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"U", ",", "Abits", ",", "Bbits", ",", "resultBits"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"U", "=", 
      RowBox[{"Union", "[", 
       RowBox[{
        RowBox[{"A", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "1"}], "]"}], "]"}], ",", 
        RowBox[{"B", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "1"}], "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Abits", "=", 
      RowBox[{"rosterToBit", "[", 
       RowBox[{"A", ",", "U"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Bbits", "=", 
      RowBox[{"rosterToBit", "[", 
       RowBox[{"B", ",", "U"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"resultBits", "=", 
      RowBox[{"MapThread", "[", 
       RowBox[{"Max", ",", 
        RowBox[{"{", 
         RowBox[{"Abits", ",", "Bbits"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bitToRoster", "[", 
      RowBox[{"resultBits", ",", "U"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
As an example, we will compute the union of the fuzzy sets defined below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyA", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "0.1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<b\>\"", ",", "0.3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<c\>\"", ",", "0.7"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "0.1`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "0.3`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "0.7`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyB", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "0.5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<b\>\"", ",", "0.1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\"\<d\>\"", ",", "0.2"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "0.5`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "0.1`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"d\"\>", ",", "0.2`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fuzzyUnion", "[", 
  RowBox[{"fuzzyA", ",", "fuzzyB"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "0.5`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "0.3`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "0.7`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"d\"\>", ",", "0.2`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Functions for computing intersection and complement are similar and are left \
to the reader.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computer Projects 9", "Subsection",
 CellTags->"SolutionsCP9"],

Cell["Given a square matrix, determine whether it is symmetric.", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will create a function, ",
 StyleBox["symmetricQ", "Input"],
 ", that tests a matrix to see if it is symmetric. Recall that a matrix is \
symmetric when it is equal to its transpose. So we just need to use the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") operator to compare the matrix with the result of applying ",
 StyleBox[ButtonBox["Transpose",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Transpose"], "FunctionLink"],
 ". We will use ",
 StyleBox[ButtonBox["MatrixQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixQ"], "FunctionLink"],
 " to ensure that the argument to the function is in fact a matrix."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"symmetricQ", "[", 
   RowBox[{"m_", "?", "MatrixQ"}], "]"}], ":=", 
  RowBox[{"m", "\[Equal]", 
   RowBox[{"Transpose", "[", "m", "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"symmetricExample", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "5", ",", "6"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"symmetricExample", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "2", "3"},
     {"2", "4", "5"},
     {"3", "5", "6"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"symmetricQ", "[", "symmetricExample", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"notsymmetricExample", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"7", ",", "8", ",", "9"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"notsymmetricExample", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "2", "3"},
     {"4", "5", "6"},
     {"7", "8", "9"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"symmetricQ", "[", "notsymmetricExample", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a built-in function for checking whether a matrix is symmetric: ",
 StyleBox[ButtonBox["SymmetricMatrixQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SymmetricMatrixQ"], "FunctionLink"],
 "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 2", "Subsection",
 CellTags->"SolutionsCE2"],

Cell["Given a finite set, list all elements of its power set.", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " The ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function was described in Section 2.1 above. We\[CloseCurlyQuote]ll write \
a function independent of this built-in command in order to see how such a \
command might be created."
}], "Text"],

Cell[TextData[{
 "Recall from Section 2.2 of the text that sets may be represented by bit \
strings. In particular, given a set, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "}"}], 
   TraditionalForm]]],
 ", a subset may be represented by a string of 0s and 1s provided an order \
has been imposed on the set. For example, the string ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", ",", "1", ",", "1", ",", "0", ",", "0"}], TraditionalForm]]],
 " corresponds to the subset ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"b", ",", "c"}], "}"}], TraditionalForm]]],
 ". (Refer to the textbook for a complete explanation.)"
}], "Text"],

Cell["\<\
In terms of subsets, the bit string representation indicates that, for a \
given set, there is a one-to-one correspondence between subsets and bit \
strings. This means that we can solve the problem of listing all subsets of a \
given set by producing all corresponding bit strings.\
\>", "Text"],

Cell[TextData[{
 "To create the bit strings, we\[CloseCurlyQuote]ll follow the approach used \
in the function ",
 StyleBox["nextTA", "Input"],
 " from Section 1.3 of this manual. Given any bit string, the next string is \
obtained by working left to right: if a bit is 1, then it gets changed to a \
0. When you encounter a 0 bit, it is changed to a 1 and you stop the process. \
For example, suppose the current string is"
}], "Text"],

Cell[BoxData[
 FormBox[Cell["1,1,1,0,0,1,0"], TraditionalForm]], "Equation"],

Cell["\<\
You begin on the left changing the first three 1s to 0s. Then the fourth bit \
from the left is 0, so
this is changed to a 1 and the process stops. The new bit string is\
\>", "Text"],

Cell[BoxData[
 FormBox[Cell["0,0,0,1,0,1,0"], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Here is the ",
 StyleBox["nextBitS", "Input"],
 " (next bit string) function. It accepts a bit string and implements the \
process described above to produce the next bit string. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nextBitS", "[", "lastBitS_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"newBitS", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newBitS", "=", "lastBitS"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "1"}], ",", 
         RowBox[{"i", "\[LessEqual]", 
          RowBox[{"Length", "[", "lastBitS", "]"}]}], ",", 
         RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"newBitS", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "1"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"newBitS", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "=", "0"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"newBitS", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "=", "1"}], ";", 
            RowBox[{"Throw", "[", "newBitS", "]"}]}]}], "\[IndentingNewLine]",
           "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"Throw", "[", "Null", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nextBitS", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "0", ",", "1", ",", "0"}], 
   "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0", ",", "1", ",", "0"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Next we\[CloseCurlyQuote]ll need a way to convert a bit string into a \
subset of a given set. We can do this using a combination of the built-in \
function ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 ". "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " function accepts an expression (in this case the list representing the bit \
string) and a pattern (in this case 1) and returns a list of the positions \
within the expression at which you can find the pattern. For example, if we \
apply ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " to the bit string ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "0", ",", "1", ",", "0"}],
     "}"}], TraditionalForm]]],
 " and 1, it indicates that the 1s occur in positions 1, 2, 3, and 6."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "0", ",", "1", ",", "0"}],
     "}"}], ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "6", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the format of the output is designed to accommodate nesting in \
the expression being searched. Conveniently, the format is identical to what \
is expected by the function ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 ". Recall that ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 ", first described in Section 2.2 of this manual, will output the list of \
elements from the first argument specified by the second. (",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " can also simultaneously apply a function to the result, but that feature \
is not needed here.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\""}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Position", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "1", ",", "1", ",", "0", ",", "0", ",", "1", ",", "0"}], 
      "}"}], ",", "1"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"f\"\>"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We create a function, ",
 StyleBox["bitToSubset", "Input"],
 ", based on that approach."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bitToSubset", "[", 
   RowBox[{"bitS_", ",", "set_"}], "]"}], ":=", 
  RowBox[{"Extract", "[", 
   RowBox[{"set", ",", 
    RowBox[{"Position", "[", 
     RowBox[{"bitS", ",", "1"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{3.580645596974145*^9}],

Cell["\<\
We can now combine these two functions to compute the power set of a given \
set.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"powerSet", "[", "set_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "bitS", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bitS", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{"Length", "[", "set", "]"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"bitS", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"bitToSubset", "[", 
          RowBox[{"bitS", ",", "set"}], "]"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"bitS", "=", 
         RowBox[{"nextBitS", "[", "bitS", "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We apply our function to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], TraditionalForm]]],
 " to confirm that it is functioning properly."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"powerSet", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", "\<\"a\"\>", "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", "\<\"b\"\>", "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", "\<\"c\"\>", "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}]], "Print"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[TextData[{
 "Write a function ",
 StyleBox["disjointQ", "Input"],
 " that accepts two sets as arguments and returns true if the sets are \
disjoint and false otherwise."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function, ",
 StyleBox["cartesian", "Input"],
 ", to compute the Cartesian product of two sets as a single set."
}], "Item1Exercise"],

Cell[TextData[{
 "Write functions ",
 StyleBox["fuzzyIntersection", "Input"],
 " and ",
 StyleBox["fuzzyComplement", "Input"],
 " to complete Computer Project 3."
}], "Item1Exercise"],

Cell[TextData[{
 "Write procedures for computing the complement, union, intersection, \
difference, and sum for multisets. Represent a multiset as a set of pairs ",
 StyleBox["{a,m}", "Input"],
 " where ",
 StyleBox["m", "Input"],
 " is the multiplicity of the element ",
 StyleBox["a", "Input"],
 ". (Refer to the preamble to Exercise 61 in Section 2.2 for information \
about multisets.)"
}], "Item1Exercise"],

Cell["\<\
Write procedures to compute the image of a finite set under a function. \
Create one procedure for functions defined in the usual way and a second \
procedure for functions defined via indexed variables.\
\>", "Item1Exercise"],

Cell["\<\
Write a procedure to find the inverse of a function defined by an indexed \
variable.\
\>", "Item1Exercise"],

Cell["\<\
Write a procedure to find the composition of functions defined by indexed \
variables.\
\>", "Item1Exercise"],

Cell[TextData[{
 "Use computation to discover what the largest value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "!"}], TraditionalForm]]],
 " has fewer than 1000 digits. (Hint: the ",
 StyleBox[ButtonBox["IntegerLength",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerLength"], "FunctionLink"],
 " command applied to an integer will return the number of digits of the \
integer.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function ",
 StyleBox["arithmeticSequence", "Input"],
 ", modelled on ",
 StyleBox["geometricSequence2", "Input"],
 " above, that produces an arithmetic sequence."
}], "Item1Exercise"],

Cell[CellGroupData[{

Cell["\<\
Find the first 20 terms of the sequences defined by the recurrence relations \
below\
\>", "Item1Exercise"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"2", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "+", 
     RowBox[{"3", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "0"}], TraditionalForm]]],
 "."
}], "Item2Numbered"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]], "+", 
     RowBox[{"n", " ", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}], "+", 
     RowBox[{
      SuperscriptBox["n", "2"], 
      SubscriptBox["a", 
       RowBox[{"n", "-", "3"}]]}]}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "1"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "3"], "=", "3"}], TraditionalForm]]],
 "."
}], "Item2Numbered"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]], "\[CenterDot]", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}], "+", "1"}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", 
    RowBox[{
     SubscriptBox["a", "2"], "=", "1"}]}], TraditionalForm]]],
 "."
}], "Item2Numbered"]
}, Open  ]],

Cell[TextData[{
 "The Lucas numbers satisfy the recurrence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "n"], "=", 
    RowBox[{
     SubscriptBox["L", 
      RowBox[{"n", "-", "1"}]], "+", 
     SubscriptBox["L", 
      RowBox[{"n", "-", "2"}]]}]}], TraditionalForm]]],
 " and the initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "1"], "=", "2"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "2"], "=", "2"}], TraditionalForm]]],
 ". Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to gain evidence for conjectures about the divisibility of Lucas numbers \
by different integer divisors."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function to find the first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " Ulam numbers and use the function to find as many Ulam numbers as you can. \
(Ulam numbers are defined in Exercise 28 of the Supplemental Exercises for \
Chapter 2.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find formulas for the sum of the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th powers of the first ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " positive integers for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " up to 10."
}], "Item1Exercise"],

Cell[TextData[{
 "The calculation of ",
 StyleBox["dataTable", "Input"],
 " above is very inefficient, because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " must calculate the entire list of rational numbers for each value from 2 \
to 100. Create a new function, ",
 StyleBox["listActuals", "Input"],
 ", that accepts a maximum stage as input and returns the list whose entries \
are the values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 ". You can do this by modifying ",
 StyleBox["listRationals", "Input"],
 " so that at the completion of each stage, the size of ",
 StyleBox["L", "Input"],
 ", i.e., the value ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 ", is recorded in a list."
}], "Item1Exercise"],

Cell[TextData[{
 "Find a value ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " so that the graph of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", 
    FractionBox[
     RowBox[{
      SuperscriptBox["n", "2"], "-", "n"}], "2"]}], TraditionalForm]]],
 " is just above the graph of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 ". Use your ",
 StyleBox["listActuals", "Input"],
 " function to expand the data and refine the value of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the hundredth positive rational number in the list generated by ",
 StyleBox["listRationals", "Input"],
 ". What about the thousandth? Ten thousandth? (If you completed it, the \
result of the previous exercise can be helpful.)"
}], "Item1Exercise"]
}, Closed]]
},
CellGrouping->Manual,
WindowSize->{725, 850},
WindowMargins->{{4, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section2a", "Section2b",
                "Section2c", "Section3a", "Section3b", "Section3c", 
               "Section3d", "Section4a", "Section4b", "Section4c", 
               "Section4d", "Section4e", "Section5a", "Section5b", 
               "Section6a", "Section6b", "Section6c", "Section6d", 
               "SolutionsCE2", "SolutionsCP3", "SolutionsCP9", "Section1", 
               "Section2", "Section3", "Section4", "Section5", "Section6", 
               "Solutions", "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section2", "Section2a", "Section2b", "Section2c", "Section3", 
               "Section3a", "Section3b", "Section3c", "Section3d", "Section4",
                "Section4a", "Section4b", "Section4c", "Section4d", 
               "Section4e", "Section5", "Section5a", "Section5b", "Section6", 
               "Section6a", "Section6b", "Section6c", "Section6d", 
               "Solutions", "SolutionsCE2", "SolutionsCP3", "SolutionsCP9"}}] 
           SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[2220, 71, 50, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[2548, 84, 56, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[35482, 1277, 77, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[44870, 1571, 73, 1, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section2"->{
  Cell[49266, 1736, 60, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[49824, 1755, 62, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[55025, 1951, 82, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section2c"->{
  Cell[142824, 4876, 83, 1, 26, "Subsection",
   CellTags->"Section2c"]},
 "Section3"->{
  Cell[154368, 5257, 55, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[154702, 5270, 55, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[171827, 5871, 60, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[211903, 6898, 63, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Info3580371076-5514480"->{
  Cell[215089, 7022, 80, 1, 40, "Print",
   CellTags->"Info3580371076-5514480"],
  Cell[215172, 7025, 1187, 37, 146, "Print",
   CellTags->"Info3580371076-5514480"]},
 "Section3d"->{
  Cell[241321, 7933, 70, 1, 26, "Subsection",
   CellTags->"Section3d"]},
 "Section4"->{
  Cell[244011, 8055, 70, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[245590, 8110, 124, 5, 33, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[255294, 8470, 125, 5, 27, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[264167, 8811, 142, 6, 27, "Subsection",
   CellTags->"Section4c"]},
 "Section4d"->{
  Cell[287782, 9589, 66, 1, 26, "Subsection",
   CellTags->"Section4d"]},
 "Info3580371082-5514480"->{
  Cell[291413, 9723, 78, 1, 40, "Print",
   CellTags->"Info3580371082-5514480"],
  Cell[291494, 9726, 1099, 34, 84, "Print",
   CellTags->"Info3580371082-5514480"]},
 "Info3580371083-5514480"->{
  Cell[294772, 9835, 79, 1, 40, "Print",
   CellTags->"Info3580371083-5514480"],
  Cell[294854, 9838, 1179, 36, 84, "Print",
   CellTags->"Info3580371083-5514480"],
  Cell[296245, 9890, 79, 1, 40, "Print",
   CellTags->"Info3580371083-5514480"],
  Cell[296327, 9893, 1274, 40, 115, "Print",
   CellTags->"Info3580371083-5514480"]},
 "Section4e"->{
  Cell[303075, 10131, 56, 1, 26, "Subsection",
   CellTags->"Section4e"]},
 "Section5"->{
  Cell[312215, 10483, 90, 2, 35, "Section",
   PageBreakWithin->False,
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[318938, 10715, 92, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[324321, 10898, 86, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Section6"->{
  Cell[343943, 11471, 54, 1, 35, "Section",
   CellTags->"Section6"]},
 "Section6a"->{
  Cell[344399, 11487, 67, 1, 32, "Subsection",
   CellTags->"Section6a"]},
 "Section6b"->{
  Cell[371192, 12398, 63, 1, 26, "Subsection",
   CellTags->"Section6b"]},
 "Section6c"->{
  Cell[379998, 12718, 79, 1, 26, "Subsection",
   CellTags->"Section6c"]},
 "Section6d"->{
  Cell[383632, 12845, 63, 1, 26, "Subsection",
   CellTags->"Section6d"]},
 "Solutions"->{
  Cell[410656, 13791, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP3"->{
  Cell[410796, 13798, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP3"]},
 "SolutionsCP9"->{
  Cell[417303, 14027, 68, 1, 26, "Subsection",
   CellTags->"SolutionsCP9"]},
 "SolutionsCE2"->{
  Cell[421296, 14169, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE2"]},
 "Exercises"->{
  Cell[431100, 14490, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 456068, 15100},
 {"Section1a", 456150, 15103},
 {"Section1b", 456236, 15106},
 {"Section1c", 456325, 15109},
 {"Section2", 456413, 15112},
 {"Section2a", 456498, 15115},
 {"Section2b", 456587, 15118},
 {"Section2c", 456676, 15121},
 {"Section3", 456765, 15124},
 {"Section3a", 456851, 15127},
 {"Section3b", 456941, 15130},
 {"Section3c", 457031, 15133},
 {"Info3580371076-5514480", 457134, 15136},
 {"Section3d", 457317, 15141},
 {"Section4", 457406, 15144},
 {"Section4a", 457492, 15147},
 {"Section4b", 457583, 15150},
 {"Section4c", 457674, 15153},
 {"Section4d", 457765, 15156},
 {"Info3580371082-5514480", 457868, 15159},
 {"Info3580371083-5514480", 458063, 15164},
 {"Section4e", 458411, 15173},
 {"Section5", 458501, 15176},
 {"Section5a", 458615, 15180},
 {"Section5b", 458706, 15183},
 {"Section6", 458796, 15186},
 {"Section6a", 458883, 15189},
 {"Section6b", 458974, 15192},
 {"Section6c", 459065, 15195},
 {"Section6d", 459156, 15198},
 {"Solutions", 459247, 15201},
 {"SolutionsCP3", 459339, 15204},
 {"SolutionsCP9", 459436, 15207},
 {"SolutionsCE2", 459533, 15210},
 {"Exercises", 459627, 15213}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 145, 3, 106, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 2}}],
Cell[CellGroupData[{
Cell[727, 27, 31, 0, 55, "Section"],
Cell[761, 29, 233, 6, 35, "Text"],
Cell[997, 37, 1186, 29, 95, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2220, 71, 50, 1, 55, "Section",
 CellTags->"Section1"],
Cell[2273, 74, 250, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[2548, 84, 56, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[2607, 87, 467, 13, 50, "Text"],
Cell[CellGroupData[{
Cell[3099, 104, 82, 2, 28, "Input"],
Cell[3184, 108, 83, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3304, 115, 106, 2, 28, "Input"],
Cell[3413, 119, 107, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3557, 126, 229, 8, 28, "Input"],
Cell[3789, 136, 230, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4056, 149, 44, 1, 28, "Input"],
Cell[4103, 152, 45, 1, 28, "Output"]
}, Open  ]],
Cell[4163, 156, 337, 6, 50, "Text"],
Cell[4503, 164, 364, 8, 50, "Text"],
Cell[CellGroupData[{
Cell[4892, 176, 156, 4, 28, "Input",
 InitializationCell->True],
Cell[5051, 182, 103, 2, 28, "Output"]
}, Open  ]],
Cell[5169, 187, 156, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[5350, 196, 136, 4, 28, "Input",
 InitializationCell->True],
Cell[5489, 202, 83, 2, 28, "Output"]
}, Open  ]],
Cell[5587, 207, 92, 2, 18, "Text"],
Cell[5682, 211, 536, 17, 37, "Text"],
Cell[CellGroupData[{
Cell[6243, 232, 56, 1, 28, "Input"],
Cell[6302, 235, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6371, 240, 110, 3, 28, "Input"],
Cell[6484, 245, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6553, 250, 110, 3, 28, "Input"],
Cell[6666, 255, 32, 0, 28, "Output"]
}, Open  ]],
Cell[6713, 258, 862, 25, 53, "Text"],
Cell[CellGroupData[{
Cell[7600, 287, 159, 4, 28, "Input"],
Cell[7762, 293, 93, 2, 28, "Output"]
}, Open  ]],
Cell[7870, 298, 412, 11, 36, "Text"],
Cell[CellGroupData[{
Cell[8307, 313, 120, 3, 28, "Input"],
Cell[8430, 318, 31, 0, 28, "Output"]
}, Open  ]],
Cell[8476, 321, 255, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[8756, 332, 38, 0, 17, "Subsubsection"],
Cell[8797, 334, 463, 14, 36, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[9285, 352, 259, 7, 28, "Input",
 InitializationCell->True],
Cell[9547, 361, 165, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9749, 369, 80, 2, 28, "Input"],
Cell[9832, 373, 36, 0, 28, "Output"]
}, Open  ]],
Cell[9883, 376, 143, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[10051, 383, 99, 3, 28, "Input"],
Cell[10153, 388, 36, 0, 28, "Output"]
}, Open  ]],
Cell[10204, 391, 195, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[10424, 399, 139, 4, 28, "Input"],
Cell[10566, 405, 107, 2, 28, "Output"]
}, Open  ]],
Cell[10688, 410, 315, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[11028, 423, 105, 3, 28, "Input"],
Cell[11136, 428, 128, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11313, 437, 40, 0, 17, "Subsubsection"],
Cell[11356, 439, 550, 17, 32, "Text"],
Cell[11909, 458, 482, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[12416, 475, 58, 1, 29, "Input"],
Cell[12477, 478, 139, 4, 28, "Output"]
}, Open  ]],
Cell[12631, 485, 323, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[12979, 498, 100, 3, 29, "Input"],
Cell[13082, 503, 204, 6, 28, "Output"]
}, Open  ]],
Cell[13301, 512, 380, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[13706, 522, 92, 2, 29, "Input"],
Cell[13801, 526, 145, 4, 28, "Output"]
}, Open  ]],
Cell[13961, 533, 251, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[14237, 541, 97, 2, 29, "Input"],
Cell[14337, 545, 378, 7, 44, "Output"]
}, Open  ]],
Cell[14730, 555, 706, 15, 74, "Text"],
Cell[15439, 572, 425, 10, 51, "Text"],
Cell[CellGroupData[{
Cell[15889, 586, 105, 3, 28, "Input"],
Cell[15997, 591, 126, 3, 28, "Output"]
}, Open  ]],
Cell[16138, 597, 734, 21, 60, "Text"],
Cell[16875, 620, 389, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[17289, 632, 154, 5, 28, "Input"],
Cell[17446, 639, 156, 4, 28, "Output"]
}, Open  ]],
Cell[17617, 646, 227, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[17869, 654, 165, 5, 28, "Input"],
Cell[18037, 661, 151, 4, 28, "Output"]
}, Open  ]],
Cell[18203, 668, 408, 12, 38, "Text"],
Cell[CellGroupData[{
Cell[18636, 684, 175, 5, 28, "Input"],
Cell[18814, 691, 165, 4, 28, "Output"]
}, Open  ]],
Cell[18994, 698, 815, 22, 52, "Text"],
Cell[CellGroupData[{
Cell[19834, 724, 256, 8, 28, "Input"],
Cell[20093, 734, 122, 3, 28, "Output"]
}, Open  ]],
Cell[20230, 740, 215, 6, 18, "Text"],
Cell[20448, 748, 1788, 52, 86, "Text"],
Cell[22239, 802, 675, 16, 67, "Text"],
Cell[CellGroupData[{
Cell[22939, 822, 388, 13, 29, "Input",
 InitializationCell->True],
Cell[23330, 837, 1754, 58, 59, "Output"]
}, Open  ]],
Cell[25099, 898, 683, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[25807, 920, 103, 2, 29, "Input"],
Cell[25913, 924, 1397, 44, 59, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27359, 974, 53, 0, 17, "Subsubsection"],
Cell[27415, 976, 549, 14, 46, "Text"],
Cell[27967, 992, 839, 27, 37, "Text"],
Cell[CellGroupData[{
Cell[28831, 1023, 259, 8, 28, "Input",
 InitializationCell->True],
Cell[29093, 1033, 178, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29308, 1042, 86, 2, 28, "Input"],
Cell[29397, 1046, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29465, 1051, 86, 2, 28, "Input"],
Cell[29554, 1055, 32, 0, 28, "Output"]
}, Open  ]],
Cell[29601, 1058, 196, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[29822, 1068, 62, 1, 29, "Input"],
Cell[29887, 1071, 29, 0, 28, "Output"]
}, Open  ]],
Cell[29931, 1074, 257, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[30213, 1085, 139, 4, 29, "Input"],
Cell[30355, 1091, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30420, 1096, 173, 5, 29, "Input"],
Cell[30596, 1103, 28, 0, 28, "Output"]
}, Open  ]],
Cell[30639, 1106, 651, 21, 46, "Text"],
Cell[31293, 1129, 1338, 41, 74, "Text"],
Cell[32634, 1172, 1279, 39, 69, "Text"],
Cell[33916, 1213, 775, 22, 105, "Input",
 InitializationCell->True],
Cell[34694, 1237, 58, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[34777, 1241, 147, 4, 28, "Input"],
Cell[34927, 1247, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[34995, 1252, 145, 4, 28, "Input"],
Cell[35143, 1258, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35212, 1263, 175, 5, 29, "Input"],
Cell[35390, 1270, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[35482, 1277, 77, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[35562, 1280, 522, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[36109, 1298, 144, 4, 28, "Input"],
Cell[36256, 1304, 529, 15, 28, "Output"]
}, Open  ]],
Cell[36800, 1322, 443, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[37268, 1339, 166, 5, 28, "Input"],
Cell[37437, 1346, 1189, 30, 44, "Output"]
}, Open  ]],
Cell[38641, 1379, 318, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[38984, 1393, 203, 6, 28, "Input"],
Cell[39190, 1401, 529, 15, 28, "Output"]
}, Open  ]],
Cell[39734, 1419, 258, 8, 21, "Text"],
Cell[39995, 1429, 482, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[40502, 1447, 242, 7, 29, "Input"],
Cell[40747, 1456, 455, 14, 28, "Output"]
}, Open  ]],
Cell[41217, 1473, 396, 9, 36, "Text"],
Cell[41616, 1484, 441, 13, 38, "Text"],
Cell[CellGroupData[{
Cell[42082, 1501, 169, 5, 29, "Input"],
Cell[42254, 1508, 2567, 57, 74, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[44870, 1571, 73, 1, 26, "Subsection",
 CellTags->"Section1c"],
Cell[44946, 1574, 901, 26, 54, "Text"],
Cell[45850, 1602, 172, 3, 20, "Text"],
Cell[46025, 1607, 872, 25, 109, "Input",
 InitializationCell->True],
Cell[46900, 1634, 718, 21, 52, "Text"],
Cell[47621, 1657, 54, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[47700, 1661, 34, 0, 28, "Input"],
Cell[47737, 1663, 49, 1, 21, "Print"],
Cell[47789, 1666, 72, 2, 21, "Print"],
Cell[47864, 1670, 72, 2, 21, "Print"],
Cell[47939, 1674, 72, 2, 21, "Print"],
Cell[48014, 1678, 72, 2, 21, "Print"],
Cell[48089, 1682, 82, 2, 21, "Print"],
Cell[48174, 1686, 82, 2, 21, "Print"],
Cell[48259, 1690, 82, 2, 21, "Print"],
Cell[48344, 1694, 82, 2, 21, "Print"],
Cell[48429, 1698, 82, 2, 21, "Print"],
Cell[48514, 1702, 82, 2, 21, "Print"],
Cell[48599, 1706, 92, 2, 21, "Print"],
Cell[48694, 1710, 92, 2, 21, "Print"],
Cell[48789, 1714, 92, 2, 21, "Print"],
Cell[48884, 1718, 92, 2, 21, "Print"],
Cell[48979, 1722, 102, 2, 21, "Print"]
}, Open  ]],
Cell[49096, 1727, 121, 3, 20, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[49266, 1736, 60, 1, 35, "Section",
 CellTags->"Section2"],
Cell[49329, 1739, 470, 12, 50, "Text"],
Cell[CellGroupData[{
Cell[49824, 1755, 62, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[49889, 1758, 602, 17, 36, "Text"],
Cell[CellGroupData[{
Cell[50516, 1779, 174, 5, 29, "Input",
 InitializationCell->True],
Cell[50693, 1786, 118, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50848, 1794, 146, 4, 29, "Input",
 InitializationCell->True],
Cell[50997, 1800, 128, 3, 28, "Output"]
}, Open  ]],
Cell[51140, 1806, 67, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[51232, 1810, 89, 2, 29, "Input"],
Cell[51324, 1814, 142, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51503, 1823, 96, 2, 29, "Input"],
Cell[51602, 1827, 105, 2, 28, "Output"]
}, Open  ]],
Cell[51722, 1832, 286, 5, 35, "Text"],
Cell[52011, 1839, 446, 11, 52, "Text"],
Cell[CellGroupData[{
Cell[52482, 1854, 123, 3, 29, "Input"],
Cell[52608, 1859, 118, 3, 28, "Output"]
}, Open  ]],
Cell[52741, 1865, 773, 22, 67, "Text"],
Cell[CellGroupData[{
Cell[53539, 1891, 94, 2, 29, "Input"],
Cell[53636, 1895, 50, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53723, 1901, 94, 2, 29, "Input"],
Cell[53820, 1905, 73, 2, 28, "Output"]
}, Open  ]],
Cell[53908, 1910, 794, 23, 53, "Text"],
Cell[CellGroupData[{
Cell[54727, 1937, 141, 4, 29, "Input"],
Cell[54871, 1943, 105, 2, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[55025, 1951, 82, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[55110, 1954, 257, 6, 35, "Text"],
Cell[55370, 1962, 258, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[55653, 1970, 64, 0, 18, "Subsubsection"],
Cell[55720, 1972, 644, 17, 54, "Text"],
Cell[56367, 1991, 1218, 41, 101, "Text"],
Cell[57588, 2034, 1036, 35, 50, "Text"],
Cell[CellGroupData[{
Cell[58649, 2073, 146, 4, 28, "Input",
 InitializationCell->True],
Cell[58798, 2079, 93, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58928, 2086, 126, 4, 28, "Input",
 InitializationCell->True],
Cell[59057, 2092, 73, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59167, 2099, 126, 4, 28, "Input",
 InitializationCell->True],
Cell[59296, 2105, 73, 2, 28, "Output"]
}, Open  ]],
Cell[59384, 2110, 302, 11, 24, "Text"],
Cell[CellGroupData[{
Cell[59711, 2125, 153, 4, 29, "Input"],
Cell[59867, 2131, 50, 1, 28, "Output"]
}, Open  ]],
Cell[59932, 2135, 217, 8, 24, "Text"],
Cell[CellGroupData[{
Cell[60174, 2147, 226, 6, 29, "Input"],
Cell[60403, 2155, 50, 1, 28, "Output"]
}, Open  ]],
Cell[60468, 2159, 318, 9, 38, "Text"],
Cell[60789, 2170, 432, 7, 50, "Text"],
Cell[61224, 2179, 1323, 39, 90, "Text"]
}, Open  ]],
Cell[62562, 2221, 128, 3, 28, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[62715, 2228, 63, 0, 17, "Subsubsection"],
Cell[62781, 2230, 314, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[63120, 2243, 79, 2, 28, "Input"],
Cell[63202, 2247, 51, 1, 28, "Output"]
}, Open  ]],
Cell[63268, 2251, 235, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[63528, 2261, 88, 2, 28, "Input"],
Cell[63619, 2265, 483, 10, 24, "Message"],
Cell[64105, 2277, 64, 1, 28, "Output"]
}, Open  ]],
Cell[64184, 2281, 619, 16, 50, "Text"],
Cell[CellGroupData[{
Cell[64828, 2301, 211, 6, 28, "Input"],
Cell[65042, 2309, 483, 10, 24, "Message"],
Cell[65528, 2321, 483, 10, 24, "Message"],
Cell[66014, 2333, 501, 10, 24, "Message"],
Cell[66518, 2345, 495, 10, 24, "Message"],
Cell[67016, 2357, 88, 1, 28, "Output"]
}, Open  ]],
Cell[67119, 2361, 775, 18, 68, "Text"],
Cell[CellGroupData[{
Cell[67919, 2383, 283, 8, 28, "Input"],
Cell[68205, 2393, 108, 2, 28, "Output"]
}, Open  ]],
Cell[68328, 2398, 298, 8, 32, "Text"],
Cell[68629, 2408, 425, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[69079, 2423, 313, 9, 29, "Input",
 InitializationCell->True],
Cell[69395, 2434, 164, 4, 28, "Output"]
}, Open  ]],
Cell[69574, 2441, 514, 12, 60, "Text"],
Cell[70091, 2455, 401, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[70517, 2470, 74, 1, 29, "Input"],
Cell[70594, 2473, 483, 10, 24, "Message"],
Cell[71080, 2485, 483, 10, 24, "Message"],
Cell[71566, 2497, 501, 10, 24, "Message"],
Cell[72070, 2509, 495, 10, 24, "Message"],
Cell[72568, 2521, 88, 1, 28, "Output"]
}, Open  ]],
Cell[72671, 2525, 365, 8, 46, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73073, 2538, 98, 4, 17, "Subsubsection"],
Cell[73174, 2544, 510, 15, 53, "Text"],
Cell[CellGroupData[{
Cell[73709, 2563, 309, 9, 29, "Input",
 InitializationCell->True],
Cell[74021, 2574, 191, 5, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74249, 2584, 447, 13, 45, "Input",
 InitializationCell->True],
Cell[74699, 2599, 348, 10, 29, "Output"]
}, Open  ]],
Cell[75062, 2612, 616, 15, 74, "Text"],
Cell[75681, 2629, 536, 13, 46, "Text"],
Cell[76220, 2644, 526, 13, 46, "Text"],
Cell[76749, 2659, 1035, 35, 75, "Text"],
Cell[77787, 2696, 258, 5, 35, "Text"],
Cell[78048, 2703, 144, 4, 29, "Input",
 InitializationCell->True],
Cell[78195, 2709, 678, 18, 68, "Text"],
Cell[CellGroupData[{
Cell[78898, 2731, 62, 1, 29, "Input"],
Cell[78963, 2734, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[79029, 2739, 64, 1, 29, "Input"],
Cell[79096, 2742, 66, 1, 29, "Output"]
}, Open  ]],
Cell[79177, 2746, 358, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[79560, 2758, 217, 6, 29, "Input"],
Cell[79780, 2766, 112, 3, 28, "Output"]
}, Open  ]],
Cell[79907, 2772, 242, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[80174, 2782, 255, 8, 29, "Input"],
Cell[80432, 2792, 102, 3, 28, "Output"]
}, Open  ]],
Cell[80549, 2798, 1229, 35, 88, "Text"],
Cell[81781, 2835, 791, 22, 53, "Text"],
Cell[CellGroupData[{
Cell[82597, 2861, 265, 8, 29, "Input"],
Cell[82865, 2871, 122, 3, 28, "Output"]
}, Open  ]],
Cell[83002, 2877, 275, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[83302, 2889, 666, 21, 29, "Input"],
Cell[83971, 2912, 150, 4, 28, "Output"]
}, Open  ]],
Cell[84136, 2919, 380, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[84541, 2932, 108, 2, 29, "Input"],
Cell[84652, 2936, 116, 3, 28, "Output"]
}, Open  ]],
Cell[84783, 2942, 420, 7, 60, "Text"],
Cell[85206, 2951, 474, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[85705, 2966, 167, 4, 29, "Input"],
Cell[85875, 2972, 83, 2, 28, "Output"]
}, Open  ]],
Cell[85973, 2977, 299, 9, 37, "Text"],
Cell[86275, 2988, 296, 9, 29, "Input",
 InitializationCell->True],
Cell[86574, 2999, 282, 8, 37, "Text"],
Cell[CellGroupData[{
Cell[86881, 3011, 178, 5, 29, "Input",
 InitializationCell->True],
Cell[87062, 3018, 83, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[87194, 3026, 56, 0, 17, "Subsubsection"],
Cell[87253, 3028, 382, 7, 60, "Text"],
Cell[87638, 3037, 136, 4, 22, "Equation"],
Cell[87777, 3043, 417, 10, 46, "Text"],
Cell[88197, 3055, 303, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[88525, 3067, 131, 4, 29, "Input"],
Cell[88659, 3073, 605, 18, 28, "Output"]
}, Open  ]],
Cell[89279, 3094, 138, 3, 32, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89454, 3102, 70, 0, 17, "Subsubsection"],
Cell[89527, 3104, 799, 20, 74, "Text"],
Cell[90329, 3126, 315, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[90669, 3138, 164, 5, 28, "Input"],
Cell[90836, 3145, 140, 5, 28, "Output"]
}, Open  ]],
Cell[90991, 3153, 369, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[91385, 3165, 152, 4, 29, "Input"],
Cell[91540, 3171, 50, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91627, 3177, 41, 0, 28, "Input"],
Cell[91671, 3179, 145, 5, 28, "Output"]
}, Open  ]],
Cell[91831, 3187, 89, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[91945, 3193, 191, 6, 28, "Input",
 InitializationCell->True],
Cell[92139, 3201, 140, 5, 28, "Output"]
}, Open  ]],
Cell[92294, 3209, 65, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[92384, 3213, 202, 6, 29, "Input",
 InitializationCell->True],
Cell[92589, 3221, 605, 18, 28, "Output"]
}, Open  ]],
Cell[93209, 3242, 711, 15, 68, "Text"],
Cell[CellGroupData[{
Cell[93945, 3261, 1277, 32, 156, "Input",
 InitializationCell->True],
Cell[95225, 3295, 439, 12, 21, "Print"],
Cell[95667, 3309, 445, 12, 21, "Print"],
Cell[96115, 3323, 451, 12, 21, "Print"],
Cell[96569, 3337, 511, 14, 21, "Print"],
Cell[97083, 3353, 517, 14, 21, "Print"],
Cell[97603, 3369, 560, 15, 21, "Print"],
Cell[98166, 3386, 586, 15, 21, "Print"],
Cell[98755, 3403, 628, 16, 21, "Print"]
}, Open  ]],
Cell[99398, 3422, 122, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[99545, 3429, 112, 3, 29, "Input",
 InitializationCell->True],
Cell[99660, 3434, 139, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[99848, 3444, 48, 0, 17, "Subsubsection"],
Cell[99899, 3446, 526, 12, 52, "Text"],
Cell[CellGroupData[{
Cell[100450, 3462, 223, 7, 29, "Input",
 InitializationCell->True],
Cell[100676, 3471, 386, 11, 28, "Output"]
}, Open  ]],
Cell[101077, 3485, 55, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[101157, 3489, 155, 4, 29, "Input",
 InitializationCell->True],
Cell[101315, 3495, 550, 16, 28, "Output"]
}, Open  ]],
Cell[101880, 3514, 443, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[102348, 3531, 64, 1, 28, "Input"],
Cell[102415, 3534, 548, 17, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[103000, 3556, 64, 1, 28, "Input"],
Cell[103067, 3559, 988, 31, 61, "Output"]
}, Open  ]],
Cell[104070, 3593, 417, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[104512, 3610, 102, 2, 28, "Input"],
Cell[104617, 3614, 83, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[104737, 3621, 102, 2, 28, "Input"],
Cell[104842, 3625, 83, 2, 28, "Output"]
}, Open  ]],
Cell[104940, 3630, 585, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[105550, 3653, 272, 7, 28, "Input"],
Cell[105825, 3662, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[105905, 3668, 37, 0, 17, "Subsubsection"],
Cell[105945, 3670, 220, 4, 35, "Text"],
Cell[106168, 3676, 2941, 76, 298, "Input"],
Cell[109112, 3754, 537, 18, 36, "Text"],
Cell[CellGroupData[{
Cell[109674, 3776, 492, 14, 45, "Input"],
Cell[110169, 3792, 31, 0, 28, "Output"]
}, Open  ]],
Cell[110215, 3795, 272, 11, 24, "Text"],
Cell[CellGroupData[{
Cell[110512, 3810, 482, 14, 45, "Input"],
Cell[110997, 3826, 32, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[111078, 3832, 59, 0, 17, "Subsubsection"],
Cell[111140, 3834, 243, 5, 32, "Text"],
Cell[111386, 3841, 711, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[112122, 3864, 209, 6, 29, "Input",
 InitializationCell->True],
Cell[112334, 3872, 147, 4, 28, "Output"]
}, Open  ]],
Cell[112496, 3879, 95, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[112616, 3885, 87, 2, 29, "Input"],
Cell[112706, 3889, 28, 0, 28, "Output"]
}, Open  ]],
Cell[112749, 3892, 59, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[112833, 3896, 87, 2, 29, "Input"],
Cell[112923, 3900, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112988, 3905, 87, 2, 29, "Input"],
Cell[113078, 3909, 28, 0, 28, "Output"]
}, Open  ]],
Cell[113121, 3912, 377, 10, 37, "Text"],
Cell[CellGroupData[{
Cell[113523, 3926, 111, 3, 29, "Input"],
Cell[113637, 3931, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113705, 3936, 111, 3, 29, "Input"],
Cell[113819, 3941, 28, 0, 28, "Output"]
}, Open  ]],
Cell[113862, 3944, 108, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[113995, 3952, 64, 1, 29, "Input",
 InitializationCell->True],
Cell[114062, 3955, 164, 4, 28, "Output"]
}, Open  ]],
Cell[114241, 3962, 408, 9, 52, "Text"],
Cell[CellGroupData[{
Cell[114674, 3975, 230, 7, 45, "Input",
 InitializationCell->True],
Cell[114907, 3984, 183, 5, 28, "Output"]
}, Open  ]],
Cell[115105, 3992, 799, 21, 67, "Text"],
Cell[CellGroupData[{
Cell[115929, 4017, 155, 4, 29, "Input"],
Cell[116087, 4023, 95, 2, 28, "Output"]
}, Open  ]],
Cell[116197, 4028, 780, 16, 88, "Text"],
Cell[116980, 4046, 472, 14, 43, "myCode",
 Evaluatable->False,
 InitializationCell->False],
Cell[117455, 4062, 925, 23, 74, "Text"],
Cell[118383, 4087, 338, 8, 36, "Text"],
Cell[118724, 4097, 178, 5, 29, "Input",
 InitializationCell->True],
Cell[118905, 4104, 254, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[119184, 4116, 112, 3, 29, "Input",
 InitializationCell->True],
Cell[119299, 4121, 57, 1, 21, "Print"]
}, Open  ]],
Cell[119371, 4125, 730, 22, 52, "Text"],
Cell[120104, 4149, 130, 3, 29, "Input",
 InitializationCell->True],
Cell[120237, 4154, 352, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[120614, 4169, 85, 2, 29, "Input"],
Cell[120702, 4173, 80, 2, 21, "Print"]
}, Open  ]],
Cell[120797, 4178, 995, 25, 74, "Text"],
Cell[121795, 4205, 448, 12, 37, "Text"],
Cell[122246, 4219, 280, 8, 29, "Input",
 InitializationCell->True],
Cell[122529, 4229, 405, 11, 36, "Text"],
Cell[CellGroupData[{
Cell[122959, 4244, 40, 0, 29, "Input"],
Cell[123002, 4246, 483, 10, 24, "Message"],
Cell[123488, 4258, 483, 10, 24, "Message"],
Cell[123974, 4270, 501, 10, 24, "Message"],
Cell[124478, 4282, 495, 10, 24, "Message"],
Cell[124976, 4294, 88, 1, 28, "Output"]
}, Open  ]],
Cell[125079, 4298, 137, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[125241, 4306, 75, 1, 29, "Input"],
Cell[125319, 4309, 70, 1, 22, "Print"]
}, Open  ]],
Cell[125404, 4313, 651, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[126080, 4332, 75, 1, 29, "Input"],
Cell[126158, 4335, 247, 6, 29, "Output"]
}, Open  ]],
Cell[126420, 4344, 656, 14, 60, "Text"],
Cell[127079, 4360, 198, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[127302, 4370, 109, 3, 29, "Input"],
Cell[127414, 4375, 516, 16, 56, "Output"]
}, Open  ]],
Cell[127945, 4394, 394, 10, 37, "Text"],
Cell[CellGroupData[{
Cell[128364, 4408, 150, 4, 29, "Input"],
Cell[128517, 4414, 501, 10, 24, "Message"],
Cell[129021, 4426, 88, 1, 28, "Output"]
}, Open  ]],
Cell[129124, 4430, 222, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[129371, 4440, 195, 5, 29, "Input"],
Cell[129569, 4447, 164, 4, 28, "Output"]
}, Open  ]],
Cell[129748, 4454, 1249, 30, 83, "Text"],
Cell[CellGroupData[{
Cell[131022, 4488, 145, 4, 29, "Input"],
Cell[131170, 4494, 33, 0, 29, "Output"]
}, Open  ]],
Cell[131218, 4497, 282, 8, 21, "Text"],
Cell[131503, 4507, 936, 26, 141, "Input"],
Cell[132442, 4535, 412, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[132879, 4551, 75, 1, 29, "Input"],
Cell[132957, 4554, 163, 4, 21, "Print"]
}, Open  ]],
Cell[133135, 4561, 142, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[133302, 4568, 85, 2, 29, "Input"],
Cell[133390, 4572, 80, 2, 21, "Print"]
}, Open  ]],
Cell[133485, 4577, 1189, 30, 83, "Text"],
Cell[134677, 4609, 1004, 27, 141, "Input",
 InitializationCell->True],
Cell[135684, 4638, 165, 3, 20, "Text"],
Cell[135852, 4643, 133, 3, 29, "Input",
 InitializationCell->True],
Cell[135988, 4648, 658, 18, 93, "Input",
 InitializationCell->True],
Cell[136649, 4668, 228, 6, 36, "Text"],
Cell[136880, 4676, 355, 11, 61, "Input",
 InitializationCell->True],
Cell[137238, 4689, 131, 3, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[137394, 4696, 76, 1, 29, "Input"],
Cell[137473, 4699, 163, 4, 21, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137673, 4708, 86, 2, 29, "Input"],
Cell[137762, 4712, 80, 2, 21, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137879, 4719, 119, 3, 29, "Input"],
Cell[138001, 4724, 81, 2, 22, "Print"]
}, Open  ]],
Cell[138097, 4729, 152, 6, 21, "Text"],
Cell[138252, 4737, 3375, 87, 314, "Input",
 InitializationCell->True],
Cell[141630, 4826, 176, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[141831, 4836, 418, 12, 45, "Input"],
Cell[142252, 4850, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142320, 4855, 408, 12, 29, "Input"],
Cell[142731, 4869, 32, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[142824, 4876, 83, 1, 26, "Subsection",
 CellTags->"Section2c"],
Cell[142910, 4879, 368, 7, 50, "Text"],
Cell[CellGroupData[{
Cell[143303, 4890, 58, 0, 18, "Subsubsection"],
Cell[143364, 4892, 189, 4, 35, "Text"],
Cell[143556, 4898, 697, 13, 50, "Text"],
Cell[CellGroupData[{
Cell[144278, 4915, 326, 10, 29, "Input",
 InitializationCell->True],
Cell[144607, 4927, 263, 8, 28, "Output"]
}, Open  ]],
Cell[144885, 4938, 136, 3, 20, "Text"],
Cell[145024, 4943, 443, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[145492, 4954, 243, 6, 29, "Input",
 InitializationCell->True],
Cell[145738, 4962, 183, 3, 29, "Output"]
}, Open  ]],
Cell[145936, 4968, 218, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[146179, 4977, 200, 6, 29, "Input",
 InitializationCell->True],
Cell[146382, 4985, 138, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[146569, 4995, 76, 0, 18, "Subsubsection"],
Cell[146648, 4997, 488, 10, 51, "Text"],
Cell[147139, 5009, 1162, 32, 107, "Input",
 InitializationCell->True],
Cell[148304, 5043, 299, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[148628, 5056, 105, 2, 29, "Input"],
Cell[148736, 5060, 263, 8, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[149048, 5074, 76, 0, 18, "Subsubsection"],
Cell[149127, 5076, 769, 18, 67, "Text"],
Cell[149899, 5096, 814, 18, 67, "Text"],
Cell[CellGroupData[{
Cell[150738, 5118, 204, 6, 28, "Input"],
Cell[150945, 5126, 121, 4, 28, "Output"]
}, Open  ]],
Cell[151081, 5133, 228, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[151334, 5143, 229, 7, 28, "Input"],
Cell[151566, 5152, 146, 5, 28, "Output"]
}, Open  ]],
Cell[151727, 5160, 118, 3, 18, "Text"],
Cell[151848, 5165, 580, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[152453, 5184, 173, 5, 29, "Input"],
Cell[152629, 5191, 28, 0, 28, "Output"]
}, Open  ]],
Cell[152672, 5194, 102, 4, 21, "Text"],
Cell[152777, 5200, 1250, 37, 123, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[154052, 5241, 102, 2, 29, "Input"],
Cell[154157, 5245, 138, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[154368, 5257, 55, 1, 35, "Section",
 CellTags->"Section3"],
Cell[154426, 5260, 251, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[154702, 5270, 55, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[154760, 5273, 950, 26, 82, "Text"],
Cell[155713, 5301, 210, 4, 35, "Text"],
Cell[155926, 5307, 313, 7, 35, "Text"],
Cell[156242, 5316, 147, 4, 29, "Input",
 InitializationCell->True],
Cell[156392, 5322, 419, 12, 36, "Text"],
Cell[156814, 5336, 566, 15, 51, "Text"],
Cell[CellGroupData[{
Cell[157405, 5355, 71, 1, 28, "Input"],
Cell[157479, 5358, 72, 1, 28, "Output"]
}, Open  ]],
Cell[157566, 5362, 437, 15, 36, "Text"],
Cell[158006, 5379, 185, 6, 29, "Input",
 InitializationCell->True],
Cell[158194, 5387, 150, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[158369, 5394, 137, 4, 28, "Input"],
Cell[158509, 5400, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158574, 5405, 57, 1, 28, "Input"],
Cell[158634, 5408, 58, 1, 28, "Output"]
}, Open  ]],
Cell[158707, 5412, 569, 17, 53, "Text"],
Cell[159279, 5431, 507, 12, 51, "Text"],
Cell[159789, 5445, 160, 5, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[159974, 5454, 57, 1, 28, "Input"],
Cell[160034, 5457, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160099, 5462, 57, 1, 28, "Input"],
Cell[160159, 5465, 58, 1, 28, "Output"]
}, Open  ]],
Cell[160232, 5469, 1059, 30, 74, "Text"],
Cell[161294, 5501, 1012, 26, 68, "Text"],
Cell[162309, 5529, 208, 6, 29, "Input",
 InitializationCell->True],
Cell[162520, 5537, 529, 15, 90, "Input",
 InitializationCell->True],
Cell[163052, 5554, 94, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[163171, 5560, 58, 1, 29, "Input"],
Cell[163232, 5563, 27, 0, 21, "Print"],
Cell[163262, 5565, 27, 0, 21, "Print"],
Cell[163292, 5567, 27, 0, 21, "Print"]
}, Open  ]],
Cell[163334, 5570, 205, 4, 36, "Text"],
Cell[CellGroupData[{
Cell[163564, 5578, 76, 2, 29, "Input"],
Cell[163643, 5582, 77, 2, 29, "Output"]
}, Open  ]],
Cell[163735, 5587, 187, 4, 32, "Text"],
Cell[163925, 5593, 1415, 39, 102, "Text"],
Cell[165343, 5634, 153, 4, 21, "Text"],
Cell[165499, 5640, 578, 18, 60, "Input",
 InitializationCell->True],
Cell[166080, 5660, 381, 9, 36, "Text"],
Cell[166464, 5671, 278, 9, 30, "Input",
 InitializationCell->True],
Cell[166745, 5682, 186, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[166956, 5690, 111, 3, 28, "Input"],
Cell[167070, 5695, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167135, 5700, 121, 3, 28, "Input"],
Cell[167259, 5705, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167324, 5710, 121, 3, 28, "Input"],
Cell[167448, 5715, 122, 3, 28, "Output"]
}, Open  ]],
Cell[167585, 5721, 120, 3, 20, "Text"],
Cell[167708, 5726, 361, 11, 45, "Input"],
Cell[168072, 5739, 133, 4, 18, "Text"],
Cell[168208, 5745, 332, 9, 36, "Text"],
Cell[168543, 5756, 393, 13, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[168961, 5773, 118, 3, 29, "Input"],
Cell[169082, 5778, 167, 3, 28, "Output"]
}, Open  ]],
Cell[169264, 5784, 870, 20, 82, "Text"],
Cell[170137, 5806, 942, 28, 106, "Input",
 InitializationCell->True],
Cell[171082, 5836, 107, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[171214, 5843, 128, 3, 28, "Input"],
Cell[171345, 5848, 31, 0, 28, "Output"]
}, Open  ]],
Cell[171391, 5851, 97, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[171513, 5857, 130, 3, 28, "Input"],
Cell[171646, 5862, 132, 3, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[171827, 5871, 60, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[171890, 5874, 435, 13, 36, "Text"],
Cell[172328, 5889, 595, 18, 55, "Text"],
Cell[172926, 5909, 141, 5, 28, "myCode",
 Evaluatable->False,
 InitializationCell->False],
Cell[173070, 5916, 332, 9, 38, "Text"],
Cell[CellGroupData[{
Cell[173427, 5929, 121, 4, 28, "Input"],
Cell[173551, 5935, 29, 0, 28, "Output"]
}, Open  ]],
Cell[173595, 5938, 689, 19, 54, "Text"],
Cell[CellGroupData[{
Cell[174309, 5961, 245, 8, 29, "Input"],
Cell[174557, 5971, 123, 3, 28, "Output"]
}, Open  ]],
Cell[174695, 5977, 121, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[174841, 5985, 210, 7, 28, "Input"],
Cell[175054, 5994, 123, 3, 28, "Output"]
}, Open  ]],
Cell[175192, 6000, 680, 23, 39, "Text"],
Cell[CellGroupData[{
Cell[175897, 6027, 381, 13, 29, "Input"],
Cell[176281, 6042, 98, 2, 28, "Output"]
}, Open  ]],
Cell[176394, 6047, 323, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[176742, 6060, 217, 7, 29, "Input",
 InitializationCell->True],
Cell[176962, 6069, 156, 5, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[177155, 6079, 87, 2, 29, "Input"],
Cell[177245, 6083, 29, 0, 28, "Output"]
}, Open  ]],
Cell[177289, 6086, 149, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[177463, 6093, 87, 2, 29, "Input"],
Cell[177553, 6097, 131, 4, 30, "Output"]
}, Open  ]],
Cell[177699, 6104, 359, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[178083, 6117, 49, 0, 18, "Subsubsection"],
Cell[178135, 6119, 271, 6, 32, "Text"],
Cell[178409, 6127, 607, 24, 38, "Text"],
Cell[179019, 6153, 153, 5, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[179197, 6162, 118, 4, 29, "Input",
 InitializationCell->True],
Cell[179318, 6168, 70, 2, 30, "Output"]
}, Open  ]],
Cell[179403, 6173, 90, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[179518, 6179, 54, 1, 28, "Input"],
Cell[179575, 6182, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179640, 6187, 54, 1, 29, "Input"],
Cell[179697, 6190, 30, 0, 28, "Output"]
}, Open  ]],
Cell[179742, 6193, 403, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[180170, 6211, 178, 6, 29, "Input",
 InitializationCell->True],
Cell[180351, 6219, 126, 4, 49, "Output"]
}, Open  ]],
Cell[180492, 6226, 112, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[180629, 6234, 54, 1, 28, "Input"],
Cell[180686, 6237, 50, 1, 46, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[180773, 6243, 54, 1, 28, "Input"],
Cell[180830, 6246, 120, 4, 50, "Output"]
}, Open  ]],
Cell[180965, 6253, 633, 23, 37, "Text"],
Cell[CellGroupData[{
Cell[181623, 6280, 217, 6, 45, "Input",
 InitializationCell->True],
Cell[181843, 6288, 74, 2, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181954, 6295, 217, 6, 45, "Input",
 InitializationCell->True],
Cell[182174, 6303, 124, 4, 34, "Output"]
}, Open  ]]
}, Open  ]],
Cell[182325, 6311, 200, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[182550, 6321, 208, 7, 29, "Input"],
Cell[182761, 6330, 98, 3, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[182896, 6338, 53, 0, 17, "Subsubsection"],
Cell[182952, 6340, 837, 25, 69, "Text"],
Cell[CellGroupData[{
Cell[183814, 6369, 219, 7, 28, "Input"],
Cell[184036, 6378, 5952, 105, 246, "Output"]
}, Open  ]],
Cell[190003, 6486, 252, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[190280, 6494, 189, 6, 28, "Input"],
Cell[190472, 6502, 6159, 108, 246, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196668, 6615, 189, 6, 29, "Input"],
Cell[196860, 6623, 9857, 168, 238, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206754, 6796, 193, 6, 28, "Input"],
Cell[206950, 6804, 4892, 87, 238, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[211903, 6898, 63, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[211969, 6901, 1401, 44, 82, "Text"],
Cell[213373, 6947, 490, 17, 74, "Input",
 InitializationCell->True],
Cell[213866, 6966, 201, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[214092, 6976, 70, 1, 28, "Input"],
Cell[214165, 6979, 29, 0, 28, "Output"]
}, Open  ]],
Cell[214209, 6982, 132, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[214366, 6989, 119, 3, 28, "Input",
 InitializationCell->True],
Cell[214488, 6994, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[214554, 6999, 121, 3, 28, "Input",
 InitializationCell->True],
Cell[214678, 7004, 29, 0, 28, "Output"]
}, Open  ]],
Cell[214722, 7007, 291, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[215038, 7019, 48, 1, 28, "Input"],
Cell[215089, 7022, 80, 1, 40, "Print",
 CellTags->"Info3580371076-5514480"],
Cell[215172, 7025, 1187, 37, 146, "Print",
 CellTags->"Info3580371076-5514480"]
}, Open  ]],
Cell[216374, 7065, 1008, 22, 83, "Text"],
Cell[217385, 7089, 152, 5, 29, "myCode",
 Evaluatable->False,
 InitializationCell->False],
Cell[217540, 7096, 534, 8, 50, "Text"],
Cell[218077, 7106, 251, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[218353, 7117, 67, 1, 28, "Input"],
Cell[218423, 7120, 748, 22, 59, "Output"]
}, Open  ]],
Cell[219186, 7145, 306, 7, 36, "Text"],
Cell[219495, 7154, 247, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[219767, 7166, 118, 3, 28, "Input"],
Cell[219888, 7171, 148, 4, 28, "Output"]
}, Open  ]],
Cell[220051, 7178, 282, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[220358, 7189, 142, 4, 28, "Input"],
Cell[220503, 7195, 107, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220647, 7202, 142, 4, 28, "Input"],
Cell[220792, 7208, 29, 0, 28, "Output"]
}, Open  ]],
Cell[220836, 7211, 521, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[221382, 7227, 162, 4, 28, "Input"],
Cell[221547, 7233, 38, 0, 28, "Output"]
}, Open  ]],
Cell[221600, 7236, 215, 4, 35, "Text"],
Cell[221818, 7242, 826, 24, 77, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[222669, 7270, 62, 1, 29, "Input"],
Cell[222734, 7273, 408, 12, 28, "Output"]
}, Open  ]],
Cell[223157, 7288, 442, 14, 37, "Text"],
Cell[223602, 7304, 727, 18, 67, "Text"],
Cell[CellGroupData[{
Cell[224354, 7326, 41, 0, 18, "Subsubsection"],
Cell[224398, 7328, 575, 14, 51, "Text"],
Cell[224976, 7344, 450, 10, 52, "Text"],
Cell[225429, 7356, 796, 22, 107, "Input",
 InitializationCell->True],
Cell[226228, 7380, 775, 22, 107, "Input",
 InitializationCell->True],
Cell[227006, 7404, 132, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[227163, 7412, 63, 1, 28, "Input"],
Cell[227229, 7415, 162, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[227428, 7423, 62, 1, 29, "Input"],
Cell[227493, 7426, 108, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[227650, 7434, 49, 0, 17, "Subsubsection"],
Cell[227702, 7436, 916, 34, 40, "Text"],
Cell[228621, 7472, 265, 9, 28, "Input",
 InitializationCell->True],
Cell[228889, 7483, 265, 9, 28, "Input",
 InitializationCell->True],
Cell[229157, 7494, 272, 9, 28, "Input",
 InitializationCell->True],
Cell[229432, 7505, 808, 22, 74, "Text"],
Cell[230243, 7529, 261, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[230529, 7541, 64, 1, 28, "Input"],
Cell[230596, 7544, 1479, 41, 74, "Output"]
}, Open  ]],
Cell[232090, 7588, 155, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[232270, 7598, 60, 1, 28, "Input"],
Cell[232333, 7601, 262, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[232632, 7614, 59, 1, 29, "Input"],
Cell[232694, 7617, 118, 3, 28, "Output"]
}, Open  ]],
Cell[232827, 7623, 173, 3, 35, "Text"],
Cell[233003, 7628, 255, 7, 29, "Input",
 InitializationCell->True],
Cell[233261, 7637, 460, 13, 32, "Text"],
Cell[233724, 7652, 278, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[234027, 7668, 146, 4, 29, "Input"],
Cell[234176, 7674, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[234245, 7679, 146, 4, 29, "Input"],
Cell[234394, 7685, 31, 0, 28, "Output"]
}, Open  ]],
Cell[234440, 7688, 317, 8, 36, "Text"],
Cell[234760, 7698, 288, 8, 29, "Input",
 InitializationCell->True],
Cell[235051, 7708, 156, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[235232, 7718, 64, 1, 29, "Input"],
Cell[235299, 7721, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[235368, 7726, 64, 1, 29, "Input"],
Cell[235435, 7729, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[235515, 7735, 75, 0, 17, "Subsubsection"],
Cell[235593, 7737, 762, 22, 52, "Text"],
Cell[CellGroupData[{
Cell[236380, 7763, 94, 2, 29, "Input"],
Cell[236477, 7767, 487, 12, 247, "Output"]
}, Open  ]],
Cell[236979, 7782, 1441, 46, 61, "Text"],
Cell[238423, 7830, 509, 17, 38, "Text"],
Cell[CellGroupData[{
Cell[238957, 7851, 315, 10, 29, "Input"],
Cell[239275, 7863, 492, 12, 222, "Output"]
}, Open  ]],
Cell[239782, 7878, 724, 23, 37, "Text"],
Cell[CellGroupData[{
Cell[240531, 7905, 251, 7, 29, "Input"],
Cell[240785, 7914, 475, 12, 247, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[241321, 7933, 70, 1, 26, "Subsection",
 CellTags->"Section3d"],
Cell[241394, 7936, 432, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[241851, 7953, 60, 1, 28, "Input"],
Cell[241914, 7956, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[241979, 7961, 62, 1, 29, "Input"],
Cell[242044, 7964, 28, 0, 28, "Output"]
}, Open  ]],
Cell[242087, 7967, 632, 17, 37, "Text"],
Cell[CellGroupData[{
Cell[242744, 7988, 60, 1, 28, "Input"],
Cell[242807, 7991, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[242872, 7996, 66, 1, 29, "Input"],
Cell[242941, 7999, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[243006, 8004, 84, 2, 29, "Input"],
Cell[243093, 8008, 45, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[243175, 8014, 69, 1, 28, "Input"],
Cell[243247, 8017, 46, 0, 28, "Output"]
}, Open  ]],
Cell[243308, 8020, 408, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[243741, 8037, 44, 1, 28, "Input"],
Cell[243788, 8040, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[243855, 8045, 62, 1, 28, "Input"],
Cell[243920, 8048, 30, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[244011, 8055, 70, 1, 35, "Section",
 CellTags->"Section4"],
Cell[244084, 8058, 523, 14, 50, "Text"],
Cell[244610, 8074, 265, 9, 21, "Text"],
Cell[244878, 8085, 687, 21, 37, "Text"],
Cell[CellGroupData[{
Cell[245590, 8110, 124, 5, 33, "Subsection",
 CellTags->"Section4a"],
Cell[245717, 8117, 249, 6, 21, "Text"],
Cell[245969, 8125, 634, 22, 41, "Text"],
Cell[CellGroupData[{
Cell[246628, 8151, 190, 6, 28, "Input"],
Cell[246821, 8159, 206, 5, 28, "Output"]
}, Open  ]],
Cell[247042, 8167, 274, 6, 36, "Text"],
Cell[247319, 8175, 231, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[247575, 8186, 180, 6, 28, "Input"],
Cell[247758, 8194, 193, 4, 28, "Output"]
}, Open  ]],
Cell[247966, 8201, 300, 5, 50, "Text"],
Cell[CellGroupData[{
Cell[248291, 8210, 200, 6, 28, "Input"],
Cell[248494, 8218, 137, 4, 28, "Output"]
}, Open  ]],
Cell[248646, 8225, 385, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[249056, 8239, 159, 5, 28, "Input"],
Cell[249218, 8246, 226, 5, 28, "Output"]
}, Open  ]],
Cell[249459, 8254, 675, 17, 52, "Text"],
Cell[CellGroupData[{
Cell[250159, 8275, 59, 1, 29, "Input"],
Cell[250221, 8278, 161, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[250419, 8287, 82, 2, 29, "Input"],
Cell[250504, 8291, 171, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[250712, 8300, 92, 2, 29, "Input"],
Cell[250807, 8304, 117, 3, 28, "Output"]
}, Open  ]],
Cell[250939, 8310, 485, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[251449, 8323, 256, 8, 28, "Input"],
Cell[251708, 8333, 122, 3, 28, "Output"]
}, Open  ]],
Cell[251845, 8339, 169, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[252039, 8346, 256, 8, 28, "Input"],
Cell[252298, 8356, 122, 3, 28, "Output"]
}, Open  ]],
Cell[252435, 8362, 251, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[252711, 8373, 295, 9, 28, "Input"],
Cell[253009, 8384, 122, 3, 28, "Output"]
}, Open  ]],
Cell[253146, 8390, 715, 24, 39, "Text"],
Cell[CellGroupData[{
Cell[253886, 8418, 347, 11, 29, "Input"],
Cell[254236, 8431, 357, 10, 28, "Output"]
}, Open  ]],
Cell[254608, 8444, 649, 21, 46, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[255294, 8470, 125, 5, 27, "Subsection",
 CellTags->"Section4b"],
Cell[255422, 8477, 1081, 36, 52, "Text"],
Cell[CellGroupData[{
Cell[256528, 8517, 141, 4, 29, "Input"],
Cell[256672, 8523, 93, 2, 28, "Output"]
}, Open  ]],
Cell[256780, 8528, 263, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[257068, 8539, 143, 4, 29, "Input",
 InitializationCell->True],
Cell[257214, 8545, 83, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257334, 8552, 92, 2, 29, "Input"],
Cell[257429, 8556, 93, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257559, 8563, 37, 0, 29, "Input"],
Cell[257599, 8565, 83, 2, 28, "Output"]
}, Open  ]],
Cell[257697, 8570, 419, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[258141, 8585, 153, 4, 29, "Input",
 InitializationCell->True],
Cell[258297, 8591, 93, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[258427, 8598, 37, 0, 29, "Input"],
Cell[258467, 8600, 93, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[258597, 8607, 121, 3, 29, "Input",
 InitializationCell->True],
Cell[258721, 8612, 103, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[258861, 8619, 37, 0, 29, "Input"],
Cell[258901, 8621, 103, 2, 28, "Output"]
}, Open  ]],
Cell[259019, 8626, 293, 7, 36, "Text"],
Cell[259315, 8635, 601, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[259941, 8658, 122, 3, 29, "Input",
 InitializationCell->True],
Cell[260066, 8663, 116, 3, 28, "Output"]
}, Open  ]],
Cell[260197, 8669, 384, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[260606, 8683, 138, 4, 29, "Input",
 InitializationCell->True],
Cell[260747, 8689, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[260812, 8694, 37, 0, 29, "Input"],
Cell[260852, 8696, 116, 3, 28, "Output"]
}, Open  ]],
Cell[260983, 8702, 685, 20, 54, "Text"],
Cell[CellGroupData[{
Cell[261693, 8726, 128, 3, 29, "Input"],
Cell[261824, 8731, 117, 3, 28, "Output"]
}, Open  ]],
Cell[261956, 8737, 431, 10, 46, "Text"],
Cell[262390, 8749, 609, 14, 52, "Text"],
Cell[CellGroupData[{
Cell[263024, 8767, 163, 4, 29, "Input",
 InitializationCell->True],
Cell[263190, 8773, 126, 3, 28, "Output"]
}, Open  ]],
Cell[263331, 8779, 517, 14, 37, "Text"],
Cell[CellGroupData[{
Cell[263873, 8797, 126, 3, 29, "Input"],
Cell[264002, 8802, 116, 3, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[264167, 8811, 142, 6, 27, "Subsection",
 CellTags->"Section4c"],
Cell[264312, 8819, 641, 21, 36, "Text"],
Cell[264956, 8842, 674, 21, 76, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[265655, 8867, 104, 2, 29, "Input"],
Cell[265762, 8871, 206, 5, 28, "Output"]
}, Open  ]],
Cell[265983, 8879, 389, 11, 36, "Text"],
Cell[266375, 8892, 565, 15, 91, "Input"],
Cell[CellGroupData[{
Cell[266965, 8911, 105, 2, 29, "Input"],
Cell[267073, 8915, 293, 8, 28, "Output"]
}, Open  ]],
Cell[267381, 8926, 620, 15, 53, "Text"],
Cell[268004, 8943, 1238, 35, 68, "Text"],
Cell[269245, 8980, 688, 20, 91, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[269958, 9004, 105, 2, 29, "Input"],
Cell[270066, 9008, 206, 5, 28, "Output"]
}, Open  ]],
Cell[270287, 9016, 833, 21, 52, "Text"],
Cell[CellGroupData[{
Cell[271145, 9041, 161, 4, 29, "Input"],
Cell[271309, 9047, 138, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[271484, 9056, 162, 4, 29, "Input"],
Cell[271649, 9062, 139, 4, 28, "Output"]
}, Open  ]],
Cell[271803, 9069, 418, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[272246, 9082, 599, 18, 75, "Input"],
Cell[272848, 9102, 356, 11, 28, "Output"]
}, Open  ]],
Cell[273219, 9116, 812, 25, 46, "Text"],
Cell[274034, 9143, 424, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[274483, 9156, 774, 23, 105, "Input"],
Cell[275260, 9181, 564, 16, 44, "Output"]
}, Open  ]],
Cell[275839, 9200, 291, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[276155, 9211, 822, 24, 105, "Input"],
Cell[276980, 9237, 222, 7, 28, "Output"]
}, Open  ]],
Cell[277217, 9247, 373, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[277615, 9261, 848, 25, 107, "Input"],
Cell[278466, 9288, 449, 13, 44, "Output"]
}, Open  ]],
Cell[278930, 9304, 644, 14, 51, "Text"],
Cell[CellGroupData[{
Cell[279599, 9322, 832, 24, 106, "Input"],
Cell[280434, 9348, 679, 19, 44, "Output"]
}, Open  ]],
Cell[281128, 9370, 863, 23, 67, "Text"],
Cell[CellGroupData[{
Cell[282016, 9397, 988, 28, 106, "Input",
 InitializationCell->True],
Cell[283007, 9427, 597, 17, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[283641, 9449, 117, 3, 28, "Input",
 InitializationCell->True],
Cell[283761, 9454, 137, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[283935, 9463, 34, 0, 28, "Input"],
Cell[283972, 9465, 137, 4, 28, "Output"]
}, Open  ]],
Cell[284124, 9472, 1297, 37, 84, "Text"],
Cell[285424, 9511, 771, 22, 109, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[286220, 9537, 105, 2, 29, "Input"],
Cell[286328, 9541, 206, 5, 28, "Output"]
}, Open  ]],
Cell[286549, 9549, 873, 26, 53, "Text"],
Cell[287425, 9577, 320, 7, 36, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[287782, 9589, 66, 1, 26, "Subsection",
 CellTags->"Section4d"],
Cell[287851, 9592, 605, 16, 50, "Text"],
Cell[288459, 9610, 1108, 37, 66, "Text"],
Cell[289570, 9649, 243, 8, 29, "Input",
 InitializationCell->True],
Cell[289816, 9659, 59, 0, 20, "Text"],
Cell[289878, 9661, 226, 7, 44, "Input",
 InitializationCell->True],
Cell[290107, 9670, 127, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[290259, 9679, 56, 1, 28, "Input"],
Cell[290318, 9682, 29, 0, 28, "Output"]
}, Open  ]],
Cell[290362, 9685, 203, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[290590, 9695, 162, 5, 28, "Input"],
Cell[290755, 9702, 291, 6, 28, "Output"]
}, Open  ]],
Cell[291061, 9711, 278, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[291364, 9720, 46, 1, 28, "Input"],
Cell[291413, 9723, 78, 1, 40, "Print",
 CellTags->"Info3580371082-5514480"],
Cell[291494, 9726, 1099, 34, 84, "Print",
 CellTags->"Info3580371082-5514480"]
}, Open  ]],
Cell[292608, 9763, 432, 10, 50, "Text"],
Cell[293043, 9775, 540, 19, 60, "Input",
 InitializationCell->True],
Cell[293586, 9796, 865, 23, 83, "Text"],
Cell[294454, 9821, 243, 7, 37, "Text"],
Cell[CellGroupData[{
Cell[294722, 9832, 47, 1, 28, "Input"],
Cell[294772, 9835, 79, 1, 40, "Print",
 CellTags->"Info3580371083-5514480"],
Cell[294854, 9838, 1179, 36, 84, "Print",
 CellTags->"Info3580371083-5514480"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296070, 9879, 57, 1, 28, "Input"],
Cell[296130, 9882, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296195, 9887, 47, 1, 28, "Input"],
Cell[296245, 9890, 79, 1, 40, "Print",
 CellTags->"Info3580371083-5514480"],
Cell[296327, 9893, 1274, 40, 115, "Print",
 CellTags->"Info3580371083-5514480"]
}, Open  ]],
Cell[297616, 9936, 154, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[297795, 9943, 92, 2, 29, "Input"],
Cell[297890, 9947, 158, 4, 28, "Output"]
}, Open  ]],
Cell[298063, 9954, 110, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[298198, 9962, 92, 2, 29, "Input"],
Cell[298293, 9966, 156, 4, 33, "Output"]
}, Open  ]],
Cell[298464, 9973, 222, 6, 20, "Text"],
Cell[298689, 9981, 115, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[298829, 9989, 90, 2, 29, "Input"],
Cell[298922, 9993, 141, 4, 28, "Output"]
}, Open  ]],
Cell[299078, 10000, 144, 3, 20, "Text"],
Cell[299225, 10005, 162, 4, 21, "Text"],
Cell[299390, 10011, 418, 8, 24, "Message"],
Cell[299811, 10021, 414, 10, 50, "Text"],
Cell[CellGroupData[{
Cell[300250, 10035, 203, 6, 28, "Input",
 Evaluatable->False],
Cell[300456, 10043, 242, 3, 59, "Output"]
}, Open  ]],
Cell[300713, 10049, 888, 22, 67, "Text"],
Cell[301604, 10073, 295, 7, 37, "Text"],
Cell[CellGroupData[{
Cell[301924, 10084, 80, 2, 28, "Input",
 Evaluatable->False],
Cell[302007, 10088, 242, 3, 59, "Output"]
}, Open  ]],
Cell[302264, 10094, 140, 3, 20, "Text"],
Cell[302407, 10099, 70, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[302502, 10103, 113, 3, 28, "Input",
 Evaluatable->False],
Cell[302618, 10108, 51, 0, 30, "Output"]
}, Open  ]],
Cell[302684, 10111, 27, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[302736, 10115, 199, 6, 28, "Input",
 Evaluatable->False],
Cell[302938, 10123, 51, 0, 30, "Output"]
}, Open  ]],
Cell[303004, 10126, 34, 0, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[303075, 10131, 56, 1, 26, "Subsection",
 CellTags->"Section4e"],
Cell[303134, 10134, 221, 6, 35, "Text"],
Cell[303358, 10142, 335, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[303718, 10156, 164, 5, 28, "Input"],
Cell[303885, 10163, 29, 0, 28, "Output"]
}, Open  ]],
Cell[303929, 10166, 270, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[304224, 10177, 92, 2, 29, "Input"],
Cell[304319, 10181, 31, 0, 28, "Output"]
}, Open  ]],
Cell[304365, 10184, 807, 25, 37, "Text"],
Cell[305175, 10211, 486, 12, 51, "Text"],
Cell[305664, 10225, 1987, 65, 91, "Text"],
Cell[307654, 10292, 351, 12, 25, "Text"],
Cell[CellGroupData[{
Cell[308030, 10308, 153, 5, 28, "Input"],
Cell[308186, 10315, 30, 0, 28, "Output"]
}, Open  ]],
Cell[308231, 10318, 274, 11, 31, "Text"],
Cell[CellGroupData[{
Cell[308530, 10333, 164, 5, 28, "Input"],
Cell[308697, 10340, 72, 1, 47, "Output"]
}, Open  ]],
Cell[308784, 10344, 475, 12, 36, "Text"],
Cell[309262, 10358, 704, 23, 41, "Text"],
Cell[CellGroupData[{
Cell[309991, 10385, 152, 5, 28, "Input"],
Cell[310146, 10392, 210, 7, 47, "Output"]
}, Open  ]],
Cell[310371, 10402, 192, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[310588, 10412, 218, 7, 28, "Input"],
Cell[310809, 10421, 328, 12, 52, "Output"]
}, Open  ]],
Cell[311152, 10436, 480, 16, 25, "Text"],
Cell[CellGroupData[{
Cell[311657, 10456, 237, 8, 29, "Input"],
Cell[311897, 10466, 142, 5, 50, "Output"]
}, Open  ]],
Cell[312054, 10474, 112, 3, 20, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[312215, 10483, 90, 2, 35, "Section",
 PageBreakWithin->False,
 CellTags->"Section5"],
Cell[312308, 10487, 634, 13, 65, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[312945, 10502, 1157, 41, 82, "Text"],
Cell[314105, 10545, 786, 27, 61, "Text"],
Cell[314894, 10574, 2035, 71, 102, "Text"],
Cell[316932, 10647, 46, 0, 20, "Text"],
Cell[316981, 10649, 1301, 36, 154, "Input",
 InitializationCell->True],
Cell[318285, 10687, 227, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[318537, 10699, 66, 1, 28, "Input"],
Cell[318606, 10702, 295, 8, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318938, 10715, 92, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[319033, 10718, 478, 15, 46, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[319514, 10735, 143, 4, 28, "Input",
 InitializationCell->True],
Cell[319660, 10741, 67, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[319752, 10745, 66, 1, 29, "Input"],
Cell[319821, 10748, 31, 0, 28, "Output"]
}, Open  ]],
Cell[319867, 10751, 866, 21, 67, "Text"],
Cell[CellGroupData[{
Cell[320758, 10776, 542, 14, 89, "Input"],
Cell[321303, 10792, 31, 0, 28, "Output"]
}, Open  ]],
Cell[321349, 10795, 401, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[321775, 10809, 87, 2, 28, "Input"],
Cell[321865, 10813, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321931, 10818, 89, 2, 28, "Input"],
Cell[322023, 10822, 29, 0, 28, "Output"]
}, Open  ]],
Cell[322067, 10825, 445, 12, 36, "Text"],
Cell[322515, 10839, 384, 11, 21, "Text"],
Cell[322902, 10852, 1217, 32, 154, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[324144, 10888, 93, 2, 28, "Input"],
Cell[324240, 10892, 32, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[324321, 10898, 86, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[324410, 10901, 412, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[324847, 10911, 70, 0, 18, "Subsubsection"],
Cell[324920, 10913, 1136, 33, 83, "Text"],
Cell[CellGroupData[{
Cell[326081, 10950, 162, 5, 28, "Input"],
Cell[326246, 10957, 164, 6, 46, "Output"]
}, Open  ]],
Cell[326425, 10966, 1040, 35, 76, "Text"],
Cell[327468, 11003, 198, 7, 49, "Equation"],
Cell[327669, 11012, 1441, 44, 98, "Text"],
Cell[CellGroupData[{
Cell[329135, 11060, 264, 9, 28, "Input"],
Cell[329402, 11071, 177, 4, 29, "Output"]
}, Open  ]],
Cell[329594, 11078, 165, 5, 20, "Text"],
Cell[329762, 11085, 116, 3, 28, "Input"],
Cell[CellGroupData[{
Cell[329903, 11092, 66, 1, 29, "Input"],
Cell[329972, 11095, 29, 0, 28, "Output"]
}, Open  ]],
Cell[330016, 11098, 116, 3, 28, "Input"],
Cell[CellGroupData[{
Cell[330157, 11105, 66, 1, 29, "Input"],
Cell[330226, 11108, 29, 0, 28, "Output"]
}, Open  ]],
Cell[330270, 11111, 116, 3, 28, "Input"],
Cell[CellGroupData[{
Cell[330411, 11118, 66, 1, 29, "Input"],
Cell[330480, 11121, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330547, 11126, 86, 2, 28, "Input"],
Cell[330636, 11130, 49, 1, 47, "Output"]
}, Open  ]],
Cell[330700, 11134, 191, 7, 32, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330928, 11146, 48, 0, 18, "Subsubsection"],
Cell[330979, 11148, 1139, 32, 96, "Text"],
Cell[CellGroupData[{
Cell[332143, 11184, 308, 10, 28, "Input",
 InitializationCell->True],
Cell[332454, 11196, 3469, 64, 237, "Output"]
}, Open  ]],
Cell[335938, 11263, 148, 4, 21, "Text"],
Cell[336089, 11269, 612, 20, 37, "Text"],
Cell[336704, 11291, 295, 9, 29, "Input",
 InitializationCell->True],
Cell[337002, 11302, 651, 20, 37, "Text"],
Cell[CellGroupData[{
Cell[337678, 11326, 191, 5, 29, "Input",
 InitializationCell->True],
Cell[337872, 11333, 1034, 23, 236, "Output"]
}, Open  ]],
Cell[338921, 11359, 587, 17, 37, "Text"],
Cell[CellGroupData[{
Cell[339533, 11380, 145, 3, 29, "Input"],
Cell[339681, 11385, 4189, 78, 239, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[343943, 11471, 54, 1, 35, "Section",
 CellTags->"Section6"],
Cell[344000, 11474, 374, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[344399, 11487, 67, 1, 32, "Subsection",
 CellTags->"Section6a"],
Cell[344469, 11490, 321, 9, 35, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[344815, 11503, 64, 0, 18, "Subsubsection"],
Cell[344882, 11505, 257, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[345164, 11513, 246, 8, 28, "Input",
 InitializationCell->True],
Cell[345413, 11523, 191, 6, 28, "Output"]
}, Open  ]],
Cell[345619, 11532, 395, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[346039, 11548, 60, 1, 28, "Input"],
Cell[346102, 11551, 595, 17, 45, "Output"]
}, Open  ]],
Cell[346712, 11571, 662, 17, 52, "Text"],
Cell[347377, 11590, 404, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[347806, 11606, 233, 7, 28, "Input",
 InitializationCell->True],
Cell[348042, 11615, 191, 6, 28, "Output"]
}, Open  ]],
Cell[348248, 11624, 460, 10, 51, "Text"],
Cell[CellGroupData[{
Cell[348733, 11638, 58, 1, 28, "Input"],
Cell[348794, 11641, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[348874, 11647, 92, 3, 18, "Subsubsection"],
Cell[348969, 11652, 538, 13, 52, "Text"],
Cell[349510, 11667, 122, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[349657, 11674, 362, 11, 45, "Input",
 InitializationCell->True],
Cell[350022, 11687, 679, 19, 77, "Output"]
}, Open  ]],
Cell[350716, 11709, 862, 23, 68, "Text"],
Cell[CellGroupData[{
Cell[351603, 11736, 504, 16, 45, "Input",
 InitializationCell->True],
Cell[352110, 11754, 1652, 54, 80, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[353811, 11814, 60, 0, 18, "Subsubsection"],
Cell[353874, 11816, 258, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[354157, 11827, 100, 2, 29, "Input"],
Cell[354260, 11831, 617, 18, 72, "Output"]
}, Open  ]],
Cell[354892, 11852, 388, 9, 36, "Text"],
Cell[355283, 11863, 224, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[355532, 11873, 89, 2, 29, "Input"],
Cell[355624, 11877, 103, 2, 28, "Output"]
}, Open  ]],
Cell[355742, 11882, 127, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[355894, 11889, 277, 8, 45, "Input",
 InitializationCell->True],
Cell[356174, 11899, 639, 19, 88, "Output"]
}, Open  ]],
Cell[356828, 11921, 561, 15, 37, "Text"],
Cell[CellGroupData[{
Cell[357414, 11940, 713, 26, 89, "Input",
 InitializationCell->True],
Cell[358130, 11968, 661, 20, 88, "Output"]
}, Open  ]],
Cell[358806, 11991, 286, 7, 36, "Text"],
Cell[359095, 12000, 335, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[359455, 12012, 161, 5, 29, "Input"],
Cell[359619, 12019, 727, 23, 136, "Output"]
}, Open  ]],
Cell[360361, 12045, 321, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[360707, 12057, 171, 5, 29, "Input"],
Cell[360881, 12064, 1697, 54, 80, "Output"]
}, Open  ]],
Cell[362593, 12121, 769, 17, 67, "Text"],
Cell[363365, 12140, 522, 10, 51, "Text"],
Cell[363890, 12152, 599, 15, 53, "Text"],
Cell[CellGroupData[{
Cell[364514, 12171, 293, 10, 29, "Input"],
Cell[364810, 12183, 1739, 55, 96, "Output"]
}, Open  ]],
Cell[366564, 12241, 445, 10, 51, "Text"],
Cell[CellGroupData[{
Cell[367034, 12255, 183, 6, 29, "Input"],
Cell[367220, 12263, 1699, 55, 96, "Output"]
}, Open  ]],
Cell[368934, 12321, 140, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[369099, 12328, 129, 3, 29, "Input"],
Cell[369231, 12333, 1900, 58, 144, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[371192, 12398, 63, 1, 26, "Subsection",
 CellTags->"Section6b"],
Cell[371258, 12401, 311, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[371594, 12413, 336, 11, 44, "Input",
 InitializationCell->True],
Cell[371933, 12426, 595, 17, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[372565, 12448, 382, 13, 44, "Input",
 InitializationCell->True],
Cell[372950, 12463, 639, 19, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373626, 12487, 84, 2, 28, "Input"],
Cell[373713, 12491, 618, 18, 45, "Output"]
}, Open  ]],
Cell[374346, 12512, 164, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[374535, 12521, 83, 2, 28, "Input"],
Cell[374621, 12525, 640, 19, 45, "Output"]
}, Open  ]],
Cell[375276, 12547, 133, 4, 21, "Text"],
Cell[375412, 12553, 380, 14, 21, "Text"],
Cell[CellGroupData[{
Cell[375817, 12571, 483, 14, 44, "Input",
 InitializationCell->True],
Cell[376303, 12587, 677, 20, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[377017, 12612, 488, 17, 44, "Input",
 InitializationCell->True],
Cell[377508, 12631, 663, 21, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[378208, 12657, 84, 2, 28, "Input"],
Cell[378295, 12661, 628, 19, 61, "Output"]
}, Open  ]],
Cell[378938, 12683, 1023, 30, 68, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[379998, 12718, 79, 1, 26, "Subsection",
 CellTags->"Section6c"],
Cell[380080, 12721, 524, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[380629, 12738, 411, 13, 44, "Input",
 InitializationCell->True],
Cell[381043, 12753, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[381697, 12776, 123, 3, 28, "Input"],
Cell[381823, 12781, 662, 18, 61, "Output"]
}, Open  ]],
Cell[382500, 12802, 131, 3, 20, "Text"],
Cell[382634, 12807, 207, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[382866, 12817, 97, 2, 29, "Input"],
Cell[382966, 12821, 617, 18, 61, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[383632, 12845, 63, 1, 26, "Subsection",
 CellTags->"Section6d"],
Cell[383698, 12848, 271, 7, 35, "Text"],
Cell[383972, 12857, 1292, 35, 68, "Text"],
Cell[385267, 12894, 257, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[385549, 12904, 425, 13, 44, "Input",
 InitializationCell->True],
Cell[385977, 12919, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[386631, 12942, 425, 13, 44, "Input",
 InitializationCell->True],
Cell[387059, 12957, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[387713, 12980, 136, 4, 28, "Input"],
Cell[387852, 12986, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[388506, 13009, 56, 0, 18, "Subsubsection"],
Cell[388565, 13011, 1165, 28, 83, "Text"],
Cell[389733, 13041, 1017, 22, 82, "Text"],
Cell[390753, 13065, 287, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[391065, 13077, 99, 2, 29, "Input"],
Cell[391167, 13081, 31, 0, 28, "Output"]
}, Open  ]],
Cell[391213, 13084, 115, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[391353, 13091, 276, 10, 29, "Input"],
Cell[391632, 13103, 32, 0, 28, "Output"]
}, Open  ]],
Cell[391679, 13106, 1009, 27, 67, "Text"],
Cell[392691, 13135, 101, 4, 21, "Text"],
Cell[392795, 13141, 348, 11, 29, "Input",
 InitializationCell->True],
Cell[393146, 13154, 184, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[393355, 13162, 76, 1, 28, "Input"],
Cell[393434, 13165, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[393502, 13170, 226, 7, 28, "Input"],
Cell[393731, 13179, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[393800, 13184, 236, 7, 28, "Input"],
Cell[394039, 13193, 32, 0, 28, "Output"]
}, Open  ]],
Cell[394086, 13196, 322, 10, 36, "Text"],
Cell[394411, 13208, 267, 8, 29, "Input",
 InitializationCell->True],
Cell[394681, 13218, 266, 8, 29, "Input",
 InitializationCell->True],
Cell[394950, 13228, 113, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[395088, 13235, 135, 4, 28, "Input"],
Cell[395226, 13241, 617, 18, 72, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[395880, 13264, 135, 4, 29, "Input"],
Cell[396018, 13270, 617, 18, 72, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[396684, 13294, 57, 0, 18, "Subsubsection"],
Cell[396744, 13296, 739, 21, 50, "Text"],
Cell[397486, 13319, 656, 21, 35, "Equation"],
Cell[398145, 13342, 519, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[398689, 13359, 232, 7, 28, "Input"],
Cell[398924, 13368, 73, 2, 28, "Output"]
}, Open  ]],
Cell[399012, 13373, 379, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[399416, 13386, 350, 11, 28, "Input",
 InitializationCell->True],
Cell[399769, 13399, 73, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[399879, 13406, 33, 0, 28, "Input"],
Cell[399915, 13408, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[399980, 13413, 36, 0, 28, "Input"],
Cell[400019, 13415, 28, 0, 28, "Output"]
}, Open  ]],
Cell[400062, 13418, 365, 8, 36, "Text"],
Cell[400430, 13428, 203, 6, 29, "Input",
 InitializationCell->True],
Cell[400636, 13436, 350, 9, 36, "Text"],
Cell[400989, 13447, 665, 19, 54, "Text"],
Cell[401657, 13468, 66, 0, 20, "Text"],
Cell[401726, 13470, 139, 3, 26, "Equation"],
Cell[401868, 13475, 266, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[402159, 13488, 135, 4, 29, "Input",
 InitializationCell->True],
Cell[402297, 13494, 28, 0, 28, "Output"]
}, Open  ]],
Cell[402340, 13497, 250, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[402615, 13509, 197, 6, 29, "Input",
 InitializationCell->True],
Cell[402815, 13517, 28, 0, 28, "Output"]
}, Open  ]],
Cell[402858, 13520, 58, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[402941, 13524, 197, 6, 29, "Input",
 InitializationCell->True],
Cell[403141, 13532, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403206, 13537, 197, 6, 29, "Input",
 InitializationCell->True],
Cell[403406, 13545, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403471, 13550, 197, 6, 29, "Input",
 InitializationCell->True],
Cell[403671, 13558, 28, 0, 28, "Output"]
}, Open  ]],
Cell[403714, 13561, 782, 29, 49, "Text"],
Cell[404499, 13592, 102, 4, 21, "Text"],
Cell[404604, 13598, 3114, 82, 283, "Input",
 InitializationCell->True],
Cell[407721, 13682, 93, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[407839, 13688, 380, 12, 44, "Input",
 InitializationCell->True],
Cell[408222, 13702, 602, 18, 72, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[408861, 13725, 340, 11, 44, "Input",
 InitializationCell->True],
Cell[409204, 13738, 595, 17, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[409836, 13760, 127, 3, 28, "Input"],
Cell[409966, 13765, 617, 18, 72, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[410656, 13791, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[410796, 13798, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP3"],
Cell[410867, 13801, 485, 20, 45, "Problem"],
Cell[411355, 13823, 362, 7, 50, "Text"],
Cell[411720, 13832, 619, 11, 66, "Text"],
Cell[412342, 13845, 175, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[412542, 13854, 575, 17, 31, "Item1Numbered"],
Cell[413120, 13873, 141, 4, 17, "Item1Numbered"],
Cell[413264, 13879, 634, 19, 31, "Item1Numbered"],
Cell[413901, 13900, 135, 4, 17, "Item1Numbered"]
}, Open  ]],
Cell[414051, 13907, 43, 0, 20, "Text"],
Cell[414097, 13909, 1232, 35, 122, "Input",
 InitializationCell->True],
Cell[415332, 13946, 97, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[415454, 13952, 321, 10, 29, "Input",
 InitializationCell->True],
Cell[415778, 13964, 263, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[416078, 13977, 321, 10, 29, "Input",
 InitializationCell->True],
Cell[416402, 13989, 263, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[416702, 14002, 96, 2, 29, "Input"],
Cell[416801, 14006, 333, 10, 28, "Output"]
}, Open  ]],
Cell[417149, 14019, 117, 3, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[417303, 14027, 68, 1, 26, "Subsection",
 CellTags->"SolutionsCP9"],
Cell[417374, 14030, 76, 0, 42, "Problem"],
Cell[417453, 14032, 798, 21, 53, "Text"],
Cell[418254, 14055, 212, 6, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[418491, 14065, 437, 13, 45, "Input",
 InitializationCell->True],
Cell[418931, 14080, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419585, 14103, 78, 1, 29, "Input"],
Cell[419666, 14106, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419734, 14111, 443, 13, 45, "Input",
 InitializationCell->True],
Cell[420180, 14126, 617, 18, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[420834, 14149, 81, 1, 29, "Input"],
Cell[420918, 14152, 32, 0, 28, "Output"]
}, Open  ]],
Cell[420965, 14155, 294, 9, 21, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[421296, 14169, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE2"],
Cell[421379, 14172, 74, 0, 42, "Problem"],
Cell[421456, 14174, 373, 10, 36, "Text"],
Cell[421832, 14186, 705, 19, 50, "Text"],
Cell[422540, 14207, 306, 5, 35, "Text"],
Cell[422849, 14214, 436, 8, 51, "Text"],
Cell[423288, 14224, 76, 1, 26, "Equation"],
Cell[423367, 14227, 193, 4, 35, "Text"],
Cell[423563, 14233, 76, 1, 26, "Equation"],
Cell[423642, 14236, 210, 5, 36, "Text"],
Cell[423855, 14243, 1429, 36, 197, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[425309, 14283, 162, 4, 28, "Input"],
Cell[425474, 14289, 126, 3, 28, "Output"]
}, Open  ]],
Cell[425615, 14295, 419, 12, 36, "Text"],
Cell[426037, 14309, 745, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[426807, 14332, 188, 5, 28, "Input"],
Cell[426998, 14339, 189, 6, 28, "Output"]
}, Open  ]],
Cell[427202, 14348, 790, 18, 67, "Text"],
Cell[CellGroupData[{
Cell[428017, 14370, 454, 13, 45, "Input"],
Cell[428474, 14385, 128, 3, 28, "Output"]
}, Open  ]],
Cell[428617, 14391, 118, 4, 21, "Text"],
Cell[428738, 14397, 314, 9, 29, "Input",
 InitializationCell->True],
Cell[429055, 14408, 105, 3, 20, "Text"],
Cell[429163, 14413, 880, 25, 121, "Input",
 InitializationCell->True],
Cell[430046, 14440, 214, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[430285, 14451, 145, 4, 29, "Input"],
Cell[430433, 14457, 44, 1, 21, "Print"],
Cell[430480, 14460, 57, 1, 21, "Print"],
Cell[430540, 14463, 57, 1, 21, "Print"],
Cell[430600, 14466, 88, 2, 21, "Print"],
Cell[430691, 14470, 57, 1, 21, "Print"],
Cell[430751, 14473, 88, 2, 21, "Print"],
Cell[430842, 14477, 88, 2, 21, "Print"],
Cell[430933, 14481, 106, 2, 21, "Print"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[431100, 14490, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[431155, 14493, 195, 5, 21, "Item1Exercise"],
Cell[431353, 14500, 159, 4, 21, "Item1Exercise"],
Cell[431515, 14506, 183, 6, 21, "Item1Exercise"],
Cell[431701, 14514, 411, 10, 47, "Item1Exercise"],
Cell[432115, 14526, 236, 4, 33, "Item1Exercise"],
Cell[432354, 14532, 118, 3, 20, "Item1Exercise"],
Cell[432475, 14537, 119, 3, 20, "Item1Exercise"],
Cell[432597, 14542, 482, 14, 34, "Item1Exercise"],
Cell[433082, 14558, 210, 6, 21, "Item1Exercise"],
Cell[CellGroupData[{
Cell[433317, 14568, 117, 3, 20, "Item1Exercise"],
Cell[433437, 14573, 532, 23, 15, "Item2Numbered"],
Cell[433972, 14598, 733, 31, 18, "Item2Numbered"],
Cell[434708, 14631, 469, 19, 15, "Item2Numbered"]
}, Open  ]],
Cell[435192, 14653, 718, 26, 33, "Item1Exercise"],
Cell[435913, 14681, 296, 7, 33, "Item1Exercise"],
Cell[436212, 14690, 356, 14, 20, "Item1Exercise"],
Cell[436571, 14706, 796, 23, 63, "Item1Exercise"],
Cell[437370, 14731, 569, 21, 48, "Item1Exercise"],
Cell[437942, 14754, 334, 8, 34, "Item1Exercise"]
}, Closed]]
}
]
*)

(* End of internal cache information *)
